@@ -1,18 +1,18 @@
 /**********************************************************************
 mol.h - Handle molecules. Declarations of OBMol, OBAtom, OBBond, OBResidue.
         (the main header for Open Babel)
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2003 by Michael Banck
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -29,14 +29,15 @@ GNU General Public License for more details.
 #endif
 
 #include <math.h>
+#include <float.h>
 
 #include <vector>
 #include <string>
 #include <map>
 
 // Currently includes many headers for 2.x backwards compatibility
-// \deprecated -- this will be cleaned up in 3.0 efforts 
-//      to improve compile time significantly. 
+// \deprecated -- this will be cleaned up in 3.0 efforts
+//      to improve compile time significantly.
 // Only include necessary headers and class declaration stubs.
 #include <openbabel/atom.h>
 #include <openbabel/bond.h>
@@ -59,6 +60,7 @@ namespace OpenBabel
   class OBAtom;
   class OBBond;
   class OBInternalCoord;
+  class OBConversion; //used only as a pointer
 
   // Class OBMol
   //MOL Property Macros (flags) -- 32+ bits
@@ -72,7 +74,7 @@ namespace OpenBabel
 #define OB_ATOMTYPES_MOL         (1<<4)
   //! Chirality detection has been performed. See OBMol::IsChiral
 #define OB_CHIRALITY_MOL         (1<<5)
-  //! Partial charges have been set or percieved 
+  //! Partial charges have been set or percieved
 #define OB_PCHARGE_MOL           (1<<6)
   //! Atom hybridizations have been set. See OBAtomTyper
 #define OB_HYBRID_MOL            (1<<8)
@@ -94,7 +96,13 @@ namespace OpenBabel
 #define OB_TCHARGE_MOL		       (1<<16)
   //! Total spin on this molecule has been set. See OBMol::SetTotalSpinMultiplicity
 #define OB_TSPIN_MOL             (1<<17)
-  // flags 18-32 unspecified
+  //! Ring typing has been performed. See OBRingTyper
+#define OB_RINGTYPES_MOL         (1<<18)
+  //! A pattern, not a complete molecule.
+#define OB_PATTERN_STRUCTURE     (1<<19)
+  //! Largest Set of Smallest Rings (LSSR) done. See OBRing and OBMol::FindLSSR
+#define OB_LSSR_MOL              (1<<20)
+  // flags 21-32 unspecified
 #define OB_CURRENT_CONFORMER	 -1
 
   // class introduction in mol.cpp
@@ -106,13 +114,15 @@ namespace OpenBabel
     bool                          _autoFormalCharge;//!< Assign formal charges automatically
     std::string                   _title;     	//!< Molecule title
     std::vector<OBAtom*>          _vatom;      	//!< vector of atoms
+    std::vector<OBAtom*>          _atomIds;    	//!< vector of atoms indexed by id
     std::vector<OBBond*>          _vbond;      	//!< vector of bonds
+    std::vector<OBBond*>          _bondIds;     //!< vector of bonds
     unsigned short int            _dimension;   //!< Dimensionality of coordinates
-    double                        _energy;      //!< Molecular heat of formation (if applicable)
     int				  _totalCharge; //!< Total charge on the molecule
     unsigned int                  _totalSpin;   //!< Total spin on the molecule (if not specified, assumes lowest possible spin)
-    double                       *_c;	          //!< coordinate array
+    double                        *_c;	        //!< coordinate array
     std::vector<double*>          _vconf;       //!< vector of conformers
+    double                        _energy;      //!< heat of formation
     unsigned int                  _natoms;      //!< Number of atoms
     unsigned int                  _nbonds;      //!< Number of bonds
     std::vector<OBResidue*>       _residue;     //!< Residue information (if applicable)
@@ -125,9 +135,10 @@ namespace OpenBabel
     //! \name Internal Kekulization routines -- see kekulize.cpp and NewPerceiveKekuleBonds()
     //@{
     void start_kekulize(std::vector <OBAtom*> &cycle, std::vector<int> &electron);
-    int expand_kekulize(OBAtom *atom1, OBAtom *atom2, std::vector<int> &currentState, std::vector<int> &initState, std::vector<int> &bcurrentState, std::vector<int> &binitState, std::vector<bool> &mark);
+    bool expand_kekulize(int bond_idx, std::vector<int> &atomState, std::vector<int> &bondState);
+    bool has_no_leftover_electrons(std::vector<int> &atomState);
     int getorden(OBAtom *atom);
-    void expandcycle(OBAtom *atom, OBBitVec &avisit);
+    bool expandcycle(OBAtom *atom, OBBitVec &avisit, const OBBitVec &potAromBonds);
     //@}
 
   public:
@@ -141,7 +152,7 @@ namespace OpenBabel
     //! Destructor
     virtual ~OBMol();
     //! Assignment, copies atoms,bonds and OBGenericData
-    OBMol &operator=(const OBMol &mol);      
+    OBMol &operator=(const OBMol &mol);
     //! Copies atoms and bonds but not OBGenericData
     OBMol &operator+=(const OBMol &mol);
 
@@ -149,10 +160,12 @@ namespace OpenBabel
     //! This improves performance since the internal atom vector does not grow.
     void ReserveAtoms(int natoms)
     {
-      if (natoms && _mod)
+      if (natoms > 0 && _mod) {
         _vatom.reserve(natoms);
+        _atomIds.reserve(natoms);
+      }
     }
-    
+
     //! Create a new OBAtom pointer. Does no bookkeeping
     //! \deprecated Use NewAtom instead, which ensures internal connections
     virtual OBAtom *CreateAtom(void);
@@ -185,7 +198,7 @@ namespace OpenBabel
     //! \param flags     any bond flags such as stereochemistry (default = none)
     //! \param insertpos the position index to insert the bond (default = none)
     //! \return Whether the new bond creation was successful
-    bool AddBond(int beginIdx, int endIdx, int order, 
+    bool AddBond(int beginIdx, int endIdx, int order,
                  int flags=0,int insertpos=-1);
     //! Add the specified residue to this molecule and update connections
     //! \return Whether the method was successful
@@ -194,25 +207,35 @@ namespace OpenBabel
     //! \return Whether the method was successful
     bool AddResidue(OBResidue&);
 
+    //! Create a new OBAtom in this molecule and ensure connections
+    //! (e.g. OBAtom::GetParent(). A new unique id will be assigned
+    //! to this atom.
+    OBAtom    *NewAtom();
     //! Create a new OBAtom in this molecule and ensure connections.
-    //! (e.g. OBAtom::GetParent()
-    virtual OBAtom    *NewAtom();
-    //! Create a new OBBond in this molecule and ensure connections.
-    //! (e.g. OBBond::GetParent()
-    virtual OBBond    *NewBond();
+    //! (e.g. OBAtom::GetParent(). The @p id will be assigned to this
+    //! atom.
+    OBAtom    *NewAtom(unsigned long id);
+    //! Create a new OBBond in this molecule and ensure connections
+    //! (e.g. OBBond::GetParent(). A new unique id will be assigned
+    //! to this bond.
+    OBBond    *NewBond();
+    //! Create a new OBBond in this molecule and ensure connections
+    //! (e.g. OBBond::GetParent(). The @p id will be assigned to this
+    //! bond.
+    OBBond    *NewBond(unsigned long id);
     //! Create a new OBResidue in this molecule and ensure connections.
-    virtual OBResidue *NewResidue();
+    OBResidue *NewResidue();
     //! Deletes an atom from this molecule and all appropriate bonds.
     //! Updates the molecule and atom and bond indexes accordingly.
     //! \warning Does not update any residues which may contain this atom
     //! \return Whether deletion was successful
-    bool DeleteAtom(OBAtom*);
+    bool DeleteAtom(OBAtom*, bool destroyAtom = true);
     //! Deletes an bond from this molecule and updates accordingly
     //! \return Whether deletion was successful
-    bool DeleteBond(OBBond*);
+    bool DeleteBond(OBBond*, bool destroyBond = true);
     //! Deletes a residue from this molecule and updates accordingly.
     //! \return Whether deletion was successful
-    bool DeleteResidue(OBResidue*);
+    bool DeleteResidue(OBResidue*, bool destroyResidue = true);
     //@}
 
     //! \name Molecule modification methods
@@ -227,7 +250,7 @@ namespace OpenBabel
     virtual void EndModify(bool nukePerceivedData=true);
     //! \return The number of nested BeginModify() calls. Used internally.
     int GetMod()           {      return(_mod);    }
-    //! Increase the number of nested BeginModify calls. Dangerous! 
+    //! Increase the number of nested BeginModify calls. Dangerous!
     //! Instead, properly use BeginModify as needed.
     void IncrementMod()    {      _mod++;          }
     //! Decrease the number of nested BeginModify calls. Dangerous!
@@ -240,7 +263,8 @@ namespace OpenBabel
     //! \return the entire set of flags. (Internal use, mainly.)
     int          GetFlags()               { return(_flags); }
     //! \return the title of this molecule (often the filename)
-    const char  *GetTitle() const         { return(_title.c_str()); }
+    //! \param replaceNewlines whether to replace any newline characters with spaces
+    const char  *GetTitle(bool replaceNewlines = true) const;
     //! \return the number of atoms (i.e. OBAtom children)
     unsigned int NumAtoms() const         {  return(_natoms); }
     //! \return the number of bonds (i.e. OBBond children)
@@ -248,46 +272,56 @@ namespace OpenBabel
     //! \return the number of non-hydrogen atoms
     unsigned int NumHvyAtoms();
     //! \return the number of residues (i.e. OBResidue substituents)
-    unsigned int NumResidues() const      { return(_residue.size()); }
+    unsigned int NumResidues() const      { return(static_cast<unsigned int> (_residue.size())); }
     //! \return the number of rotatble bonds. See OBBond::IsRotor() for details
     unsigned int NumRotors();
-    
+
     //! \return the atom at index @p idx or NULL if it does not exist.
     //! \warning Atom indexing will change. Use iterator methods instead.
-    OBAtom      *GetAtom(int idx);
+    OBAtom      *GetAtom(int idx) const;
+    //! \return the atom with @p id or NULL if it does not exist.
+    OBAtom      *GetAtomById(unsigned long id) const;
     //! \return the first atom in this molecule, or NULL if none exist.
     //! \deprecated Will be removed in favor of more standard iterator methods
-    OBAtom      *GetFirstAtom();
+    OBAtom      *GetFirstAtom() const;
     //! \return the bond at index @p idx or NULL if it does not exist.
     //! \warning Bond indexing may change. Use iterator methods instead.
-    OBBond      *GetBond(int idx);
+    OBBond      *GetBond(int idx) const;
+    //! \return the bond with @p id or NULL if it does not exist.
+    OBBond      *GetBondById(unsigned long id) const;
     //! \return the bond connecting the atom indexed by @p a and @p b or NULL if none exists.
     //! \warning Atom indexing will change. Use atom objects and iterators instead.
-    OBBond      *GetBond(int a, int b);
+    OBBond      *GetBond(int a, int b) const;
     // The safer version of the above method
     //! \return the bond between the atoms @p bgn and @p end or NULL if none exists
-    OBBond      *GetBond(OBAtom* bgn, OBAtom* end);
+    OBBond      *GetBond(OBAtom* bgn, OBAtom* end) const;
     //! \return the residue indexed by @p idx, or NULL if none exists
     //! \warning Residue indexing may change. Use iterator methods instead.
-    OBResidue   *GetResidue(int);
+    OBResidue   *GetResidue(int idx) const;
     std::vector<OBInternalCoord*> GetInternalCoord();
-    //! \return the dihedral angle between the four atoms supplied a1-a2-a3-a4)
+    /*! \return the dihedral angle (in degrees) between the four atoms supplied a1-a2-a3-a4)
+     *  WARNING: SetTorsion takes an angle in radians while GetTorsion returns it
+     *  in degrees
+     */
     double       GetTorsion(int,int,int,int);
-    //! \return the dihedral angle between the four atoms @p a, @p b, @p c, and @p d)
+    /*! \return the dihedral angle (in degrees) between the four atoms @p a, @p b, @p c, and @p d)
+     *  WARNING: SetTorsion takes an angle in radians while GetTorsion returns it
+     *  in degrees
+     */
     double       GetTorsion(OBAtom* a,OBAtom* b,OBAtom* c,OBAtom* d);
-    //! \return the angle between the three atoms @p a, @p b and @p c
+    //! \return the angle (in degrees) between the three atoms @p a, @p b and @p c
     //!  (where  a-> b (vertex) -> c )
     double GetAngle(OBAtom* a, OBAtom* b, OBAtom* c);
     //! \return the stochoimetric formula (e.g., C4H6O)
     std::string  GetFormula();
     //! \return the stochoimetric formula in spaced format e.g. C 4 H 6 O 1
-    std::string  GetSpacedFormula(int ones=0, const char* sp=" ");
+    std::string  GetSpacedFormula(int ones=0, const char* sp=" ", bool implicitH = true);
     //! \return the heat of formation for this molecule (in kcal/mol)
-    double       GetEnergy() const { return(_energy); }
+    double       GetEnergy() const { return _energy; }
     //! \return the standard molar mass given by IUPAC atomic masses (amu)
-    double       GetMolWt();
+    double       GetMolWt(bool implicitH = true);
     //! \return the mass given by isotopes (or most abundant isotope, if not specified)
-    double	 GetExactMass();
+    double	 GetExactMass(bool implicitH = true);
     //! \return the total charge on this molecule (i.e., 0 = neutral, +1, -1...)
     int		 GetTotalCharge();
     //! \return the total spin on this molecule (i.e., 1 = singlet, 2 = doublet...)
@@ -296,8 +330,10 @@ namespace OpenBabel
     unsigned short int GetDimension() const { return _dimension; }
     //! \return the set of all atomic coordinates. See OBAtom::GetCoordPtr for more
     double      *GetCoordinates() { return(_c); }
-    //! \return the Smallest Set of Smallest Rings has been run (see OBRing class
+    //! \return the Smallest Set of Smallest Rings has been run (see OBRing class)
     std::vector<OBRing*> &GetSSSR();
+    //! \return the Largest Set of Smallest Rings has been run (see OBRing class)
+    std::vector<OBRing*> &GetLSSR();
     //! Get the current flag for whether formal charges are set with pH correction
     bool AutomaticFormalCharge()   { return(_autoFormalCharge);  }
     //! Get the current flag for whether partial charges are auto-determined
@@ -319,9 +355,9 @@ namespace OpenBabel
     void   SetDimension(unsigned short int d) { _dimension = d; }
     //! Set the total charge of this molecule to @p charge
     void   SetTotalCharge(int charge);
-    //! Set the total spin multiplicity of this molecule to @p spin
-    //! (i.e., 0 = singlet (default), 1 = doublet, 2 = triplet, etc.)
-    void   SetTotalSpinMultiplicity(unsigned int spin);
+    //! Set the total spin multiplicity of this molecule to @p spinMultiplicity
+    //! Overrides the calculation from spin multiplicity of OBAtoms
+    void   SetTotalSpinMultiplicity(unsigned int spinMultiplicity);
     //! Set the internal coordinates to @p int_coord
     //! (Does not call InternalToCartesian to update the 3D cartesian coordinates)
     void   SetInternalCoord(std::vector<OBInternalCoord*> int_coord)
@@ -337,10 +373,14 @@ namespace OpenBabel
     void   SetAromaticPerceived()    { SetFlag(OB_AROMATIC_MOL);    }
     //! Mark that Smallest Set of Smallest Rings has been run (see OBRing class)
     void   SetSSSRPerceived()        { SetFlag(OB_SSSR_MOL);        }
+    //! Mark that Largest Set of Smallest Rings has been run (see OBRing class)
+    void   SetLSSRPerceived()        { SetFlag(OB_LSSR_MOL);        }
     //! Mark that rings have been perceived (see OBRing class for details)
     void   SetRingAtomsAndBondsPerceived(){SetFlag(OB_RINGFLAGS_MOL);}
     //! Mark that atom types have been perceived (see OBAtomTyper for details)
     void   SetAtomTypesPerceived()   { SetFlag(OB_ATOMTYPES_MOL);   }
+    //! Mark that ring types have been perceived (see OBRingTyper for details)
+    void   SetRingTypesPerceived()   { SetFlag(OB_RINGTYPES_MOL);   }
     //! Mark that chains and residues have been perceived (see OBChainsParser)
     void   SetChainsPerceived()      { SetFlag(OB_CHAINS_MOL);      }
     //! Mark that chirality has been perceived
@@ -363,21 +403,29 @@ namespace OpenBabel
     void   SetFlags(int flags)       { _flags = flags;              }
 
     void   UnsetAromaticPerceived()  { _flags &= (~(OB_AROMATIC_MOL));   }
+    void   UnsetSSSRPerceived()  { _flags &= (~(OB_SSSR_MOL));   }
+    void   UnsetRingTypesPerceived()  { _flags &= (~(OB_RINGTYPES_MOL));   }
     void   UnsetPartialChargesPerceived(){ _flags &= (~(OB_PCHARGE_MOL));}
     void   UnsetImplicitValencePerceived(){_flags &= (~(OB_IMPVAL_MOL)); }
     void   UnsetHydrogensAdded()       { UnsetFlag(OB_H_ADDED_MOL);     }
     void   UnsetFlag(int flag)       { _flags &= (~(flag));              }
+    //@}
 
     //! \name Molecule modification methods
     //@{
     // Description in transform.cpp (command-line transformations to this molecule)
-    virtual OBBase*    DoTransformations(const std::map<std::string,std::string>* pOptions);
+    virtual OBBase*    DoTransformations(const std::map<std::string,std::string>* pOptions,OBConversion* pConv);
     // Ditto (documentation on transformation options)
     static const char* ClassDescription();
-    //! Clear all information from a molecule
+    //! Clear all information from a molecule except OB_PATTERN_STRUCTURE left unchanged
     bool Clear();
     //! Renumber the atoms of this molecule according to the order in the supplied vector
     void RenumberAtoms(std::vector<OBAtom*>&);
+    //! Renumber the atoms of this molecule using the initial indexes in the supplied vector
+    void RenumberAtoms(std::vector<int>);
+    //! Set the coordinates for all atoms in this conformer.
+    //! \sa OBMol::GetCoordinates()
+    void SetCoordinates(double *c);
     //! Translate one conformer and rotate by a rotation matrix (which is returned) to the inertial frame-of-reference
     void ToInertialFrame(int conf, double *rmat);
     //! Translate all conformers to the inertial frame-of-reference
@@ -395,7 +443,7 @@ namespace OpenBabel
     //! Translate to the center of all coordinates (for this conformer)
     void Center();
     //! Transform to standard Kekule bond structure (presumably from an aromatic form)
-    
+
     bool Kekulize();
     bool PerceiveKekuleBonds();
 
@@ -417,39 +465,54 @@ namespace OpenBabel
     //! \param polaronly    Whether to add hydrogens only to polar atoms
     //! (i.e., not to C atoms)
     //! \param correctForPH Whether to call CorrectForPH() first
+    //! \param pH The pH to use for CorrectForPH() modification
     //! \return Whether any hydrogens were added
-    bool AddHydrogens(bool polaronly=false,bool correctForPH=true);
+    bool AddHydrogens(bool polaronly=false,bool correctForPH=false, double pH=7.4);
     //! Add hydrogens only to the supplied atom to fill out implicit valence
     bool AddHydrogens(OBAtom*);
     //! Add only polar hydrogens (i.e., attached to polar atoms, not C)
     bool AddPolarHydrogens();
 
-    //! Deletes all atoms except for the largest contiguous fragment
-    bool StripSalts();
+    //! If @p threshold is not specified or is zero, remove all but the largest
+    //! contiguous fragment. If @p threshold is non-zero, remove any fragments with fewer
+    //! than @p threshold atoms.
+    bool StripSalts(int threshold=0);
     //! Copies each disconnected fragment as a separate OBMol
     std::vector<OBMol> Separate(int StartIndex=1);
-    //! Converts the charged form of coordinate bonds, e.g.[N+]([O-])=O to N(=O)=O 
+    //! Iterative component of Separate to copy one fragment at a time
+    bool GetNextFragment( OpenBabel::OBMolAtomDFSIter& iter, OBMol& newMol );
+    //! Converts the charged form of coordinate bonds, e.g.[N+]([O-])=O to N(=O)=O
     bool ConvertDativeBonds();
 
     //! Correct for pH by applying the OBPhModel transformations
-    bool CorrectForPH();
+    bool CorrectForPH(double pH=7.4);
     // docs in mol.cpp
-    bool AssignSpinMultiplicity();
+    bool AssignSpinMultiplicity(bool NoImplicitH=false);
+    //! The OBMol is a pattern, not a complete molecule. Left unchanged by Clear().
+    void   SetIsPatternStructure()       { SetFlag(OB_PATTERN_STRUCTURE);}
+
     //! \return the center of the supplied conformer @p nconf
     //! \see Center() to actually center all conformers at the origin
     vector3 Center(int nconf);
-    //! Set the torsion defined by these atoms, rotating bonded neighbors
-    void SetTorsion(OBAtom*,OBAtom*,OBAtom*,OBAtom*,double);
+    /*! Set the torsion defined by these atoms, rotating bonded neighbors
+     *  \par ang The torsion angle in radians
+     *  WARNING: SetTorsion takes an angle in radians while GetTorsion returns it
+     *  in degrees
+     */
+    void SetTorsion(OBAtom*,OBAtom*,OBAtom*,OBAtom*,double ang);
     //@}
 
     //! \name Molecule utilities and perception methods
     //@{
     //! Find Smallest Set of Smallest Rings (see OBRing class for more details)
     void FindSSSR();
+    //! Find Largest Set of Smallest Rings
+    void FindLSSR();
     //! Find all ring atoms and bonds. Does not need to call FindSSSR().
     void FindRingAtomsAndBonds();
     //! Find all chiral atom centers. See OBAtom::IsChiral() for more details
-    void FindChiralCenters();
+    //! \deprecated See FindStereogenicUnits
+    void FindChiralCenters() { IsChiral(); }
     // documented in mol.cpp -- locates all atom indexes which can reach 'end'
     void FindChildren(std::vector<int> & children,int bgnIdx,int endIdx);
     // documented in mol.cpp -- locates all atoms which can reach 'end'
@@ -482,8 +545,8 @@ namespace OpenBabel
 
     //! \name Methods to check for existence of properties
     //@{
-    //! Are there non-zero coordinates in two dimensions (i.e. X and Y)?
-    bool Has2D();
+    //! Are there non-zero coordinates in two dimensions (i.e. X and Y)- and, if Not3D is true, no Z coordinates?
+    bool Has2D(bool Not3D=false);
     //! Are there non-zero coordinates in all three dimensions (i.e. X, Y, Z)?
     bool Has3D();
     //! Are there any non-zero coordinates?
@@ -492,10 +555,14 @@ namespace OpenBabel
     bool HasAromaticPerceived()     { return(HasFlag(OB_AROMATIC_MOL)); }
     //! Has the smallest set of smallest rings (FindSSSR) been performed?
     bool HasSSSRPerceived()         { return(HasFlag(OB_SSSR_MOL));     }
+    //! Has the largest set of smallest rings (FindLSSR) been performed?
+    bool HasLSSRPerceived()         { return(HasFlag(OB_LSSR_MOL));     }
     //! Have ring atoms and bonds been assigned?
     bool HasRingAtomsAndBondsPerceived(){return(HasFlag(OB_RINGFLAGS_MOL));}
     //! Have atom types been assigned by OBAtomTyper?
     bool HasAtomTypesPerceived()    { return(HasFlag(OB_ATOMTYPES_MOL));}
+    //! Have ring types been assigned by OBRingTyper?
+    bool HasRingTypesPerceived()    { return(HasFlag(OB_RINGTYPES_MOL));}
     //! Has atom chirality been assigned?
     bool HasChiralityPerceived()    { return(HasFlag(OB_CHIRALITY_MOL));}
     //! Have atomic Gasteiger partial charges been assigned by OBGastChrg?
@@ -527,7 +594,7 @@ namespace OpenBabel
     //! \name Multiple conformer member functions
     //@{
     //! \return the number of conformers in this molecule
-    int     NumConformers()    { return((_vconf.empty())?0:_vconf.size()); }
+    int     NumConformers()    { return((_vconf.empty())?0:static_cast<int> (_vconf.size())); }
     //! Set the entire set of conformers for this molecule to @p v
     void    SetConformers(std::vector<double*> &v);
     //! Add a new set of coordinates @p f as a new conformer
@@ -542,6 +609,13 @@ namespace OpenBabel
     void    DeleteConformer(int nconf);
     //! \return the coordinates to conformer @p i
     double  *GetConformer(int i)       {  return(_vconf[i]);      }
+    //! Set the entire set of conformer energies
+    void    SetEnergies(std::vector<double> &energies);
+    //! Set the entire set of conformer energies
+    std::vector<double> GetEnergies();
+    //! Get the energy for conformer ci
+    //! \par ci conformer index
+    double  GetEnergy(int ci);
     //! Set the iterator to the beginning of the conformer list
     //! \return the array of coordinates for the first conformer
     double  *BeginConformer(std::vector<double*>::iterator&i)
@@ -561,11 +635,11 @@ namespace OpenBabel
     //! \return An atom iterator pointing to the beginning of the atom list
     OBAtomIterator BeginAtoms()   { return _vatom.begin(); }
     //! \return An atom iterator pointing to the end of the atom list
-    OBAtomIterator EndAtoms()     { return _vatom.end();   }
+    OBAtomIterator EndAtoms()    { return _vatom.begin() + NumAtoms() ; }
     //! \return A bond iterator pointing to the beginning of the bond list
     OBBondIterator BeginBonds()   { return _vbond.begin(); }
     //! \return A bond iterator pointing to the end of the bond list
-    OBBondIterator EndBonds()     { return _vbond.end();   }
+    OBBondIterator EndBonds()     { return _vbond.begin() + NumBonds() ; }
     //! \return A residue iterator pointing to the beginning of the residue list
     OBResidueIterator BeginResidues() { return _residue.begin(); }
     //! \return A residue iterator pointing to the end of the residue list
@@ -607,7 +681,7 @@ namespace OpenBabel
     }
     //! Advance the iterator to the next internal coordinate record
     //! \return the next first internal coordinate record, or NULL if none exist
-    //! \see SetInternalCoord 
+    //! \see SetInternalCoord
     OBInternalCoord *NextInternalCoord(std::vector<OBInternalCoord*>::iterator &i)
     {
       ++i;
@@ -618,10 +692,7 @@ namespace OpenBabel
   };
 
   // Utility function prototypes
-  OBAPI bool tokenize(std::vector<std::string>&, const char *buf, const char *delimstr=" \t\n");
-  OBAPI bool tokenize(std::vector<std::string>&, std::string&, const char *delimstr=" \t\n", int limit=-1);
-  // Remove leading and trailing whitespace from a string (docs in tokenst.cpp)
-  OBAPI std::string& Trim(std::string& txt);
+  //tokenize and Trim declarations moved to base.h
   // Deprecated -- use OBMessageHandler class instead (docs in obutil.cpp)
   OBAPI void ThrowError(char *str);
   // Deprecated -- use OBMessageHandler class instead (docs in obutil.cpp)
@@ -653,6 +724,27 @@ namespace OpenBabel
   //! Global OBResidueData biomolecule residue database
   EXTERN  OBResidueData    resdat;
 
+  //! \brief Nested namespace for max_value templates
+  namespace detail {
+    //! \struct max_value mol.h <openbabel/mol.h>
+    //! \brief a C++ template to return the maximum value of a type (e.g., int)
+    template<typename T, int size = sizeof(T)>
+    struct max_value
+    {
+      static const T result = (static_cast<T>(0xFF) << (size-1)*8) + max_value<T, size-1>::result;
+    };
+
+    //! \brief a C++ template to return the maximum value of a type (e.g., int)
+    template<typename T>
+    struct max_value<T, 0>
+    {
+      static const T result = 0;
+    };
+  }
+
+  // No unique id
+  static const unsigned long NoId = detail::max_value<unsigned long>::result;
+
   //Utility Macros
 
 #ifndef BUFF_SIZE
@@ -690,6 +782,8 @@ namespace OpenBabel
   OBAPI double superimpose(double*,double*,int);
 #endif // __KCC
 
+//  extern OBMol* (*CreateMolecule) (void);
+
 } // end namespace OpenBabel
 
 #endif // OB_MOL_H
