@@ -1,10 +1,10 @@
 /**********************************************************************
 obconversion.h - Handle file conversions. Declaration of OBFormat, OBConversion
 
-Copyright (C) 2004-2005 by Chris Morley
+Copyright (C) 2004-2009 by Chris Morley
 
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -28,9 +28,14 @@ GNU General Public License for more details.
 #include <string>
 #include <vector>
 #include <map>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 #include <openbabel/dlhandler.h>
 #include <openbabel/oberror.h>
+#include <openbabel/format.h>
+#include <openbabel/lineend.h>
 
 // These macros are used in DLL builds. If they have not
 // been set in babelconfig.h, define them as nothing.
@@ -44,116 +49,11 @@ GNU General Public License for more details.
 //using namespace std;
 namespace OpenBabel {
 
+  // Needed to preserve deprecated API
+  typedef OBPlugin::PluginIterator Formatpos;
 
-  class OBBase;
-  class OBConversion;
   OBERROR extern  OBMessageHandler obErrorLog;
 
-  //*************************************************
-
-  /// @brief Base class for file formats.
-  // class introduction in obconversion.cpp
-  class OBCONV OBFormat
-    {
-    public:
-      /// @brief The "API" interface Read function.
-
-      /// Reads a single object.
-      /// Does not make a new object on the heap; 
-      /// can be used with a pointer to an chem object on the heap or the stack.
-      virtual bool ReadMolecule(OBBase* /*pOb*/, OBConversion* /*pConv*/)
-        { std::cerr << "Not a valid input format"; return false;}
-
-      /// @brief The "Convert" interface Read function.
-
-      /// Possibly reads multiple new objects on the heap and subjects them 
-      /// to its DoTransformations() function, which may delete them again. 
-      /// Sends result to pConv->AddChemObject()
-      virtual bool ReadChemObject(OBConversion* /*pConv*/)
-        { std::cerr << "Not a valid input format"; return false;}
-
-      /// @brief The "API" interface Write function.
-
-      /// Writes a single object
-      /// Does not delete the object; 
-      /// can be used with a pointer to an chem object on the heap or the stack.
-      /// \return false on error.
-      virtual bool WriteMolecule(OBBase* /*pOb*/, OBConversion* /*pConv*/)
-        { std::cerr << "Not a valid output format"; return false;}
-
-      /// @brief The "Convert" interface Write function.
-
-      /// Writes a single object
-      /// Deletes the object after writing 
-      /// \return false on error
-      virtual bool WriteChemObject(OBConversion* /*pConv*/)
-        { std::cerr << "Not a valid output format"; return false;}
-
-      /// @brief Information on this format. Printed out in response to -Hxxx option where xxx id the id of the format.
-
-      /// Must be provided by each format class.
-      /// Can include a list of command line Options. These may be used to construction
-      /// check boxes, radio buttons etc for GUI interface.
-      virtual const char* Description()=0;
-	
-      /// @brief A decription of the chemical object converted by this format.
-
-      /// If not provided, the object type used by the default format is used (usually OBMol). 
-      virtual const char* TargetClassDescription();
-
-      /// \return the type of chemical object used by the format.
-
-      /// Defaults to that used by the default format. Useful for checking 
-      /// that a format can handle a particular object.
-      virtual const std::type_info& GetType();
- 	
-      /// @brief Web address where the format is defined.
-      virtual const char* SpecificationURL() { return ""; }
-
-      /// @brief Chemical MIME type associated with this file type (if any)
-      virtual const char* GetMIMEType() { return ""; }
-
-      /// @brief Decribes the capabilities of the format (Read only etc.)
-   
-      /// Currently, can be a bitwise OR of any of the following
-      /// NOTREADABLE READONEONLY NOTWRITABLE WRITEONEONLY DEFAULTFORMAT
-      /// READBINARY WRITEBINARY READXML
-      virtual unsigned int Flags() { return 0;}; 
-
-      /// @brief Skip past first n objects in input stream (or current one with n=0)
-
-      /// \return 1 on success, -1 on error and 0 if not implemented 
-      virtual int SkipObjects(int /*n*/, OBConversion* /*pConv*/)
-        {
-          return 0; //shows not implemented in the format class
-        };
-
-      /// \return a pointer to a new instance of the format, or NULL if fails.
-
-      /// Normally a single global instance is used but this may cause problems
-      /// if there are member variables and the format is used in more than one place
-      /// in the program.
-      virtual OBFormat* MakeNewInstance()
-        {
-          return NULL; //shows not implemented in the format class
-        }
-
-      /// @brief Format classes do not have a destructor
-      virtual ~OBFormat(){};
-    };
-
-  //*************************************************
-  /// @struct CharPtrLess obconversion.h <openbabel/obconversion.h>
-  /// @brief Case insensitive string comparison for FormatsMap key.
-	struct CharPtrLess : public std::binary_function<const char*,const char*, bool>
-	{
-		bool operator()(const char* p1,const char* p2) const
-		{ return strcasecmp(p1,p2)<0; }
-	};
-
-	typedef std::map<const char*,OBFormat*,CharPtrLess > FMapType;
-	typedef FMapType::iterator Formatpos;
-
   //*************************************************
   /// @brief Class to convert from one format to another.
   // Class introduction in obconversion.cpp
@@ -166,23 +66,29 @@ namespace OpenBabel {
       OBConversion(std::istream* is=NULL, std::ostream* os=NULL);
       /// @brief Copy constructor
       OBConversion(const OBConversion& o);
-      virtual     ~OBConversion(); 
-      //@}	
+      virtual     ~OBConversion();
+      //@}
       /// @name Collection of formats
       //@{
       /// @brief Called once by each format class
       static int				RegisterFormat(const char* ID, OBFormat* pFormat, const char* MIME = NULL);
       /// @brief Searches registered formats
       static OBFormat*	FindFormat(const char* ID);
+      /// @brief Searches registered formats
+      /// \since version 2.3
+      static OBFormat*  FindFormat(const std::string ID);
       /// @brief Searches registered formats for an ID the same as the file extension
       static OBFormat*	FormatFromExt(const char* filename);
+      /// @brief Searches registered formats for an ID the same as the file extension
+      /// \since version 2.3
+      static OBFormat*	FormatFromExt(const std::string filename);
       /// @brief Searches registered formats for a MIME the same as the chemical MIME type passed
       static OBFormat*        FormatFromMIME(const char* MIME);
 
-      ///Repeatedly called to recover available Formats
+      ///Deprecated!.Repeatedly called to recover available Formats
       static bool	        GetNextFormat(Formatpos& itr, const char*& str,OBFormat*& pFormat);
       //@}
-		
+
       /// @name Information
       //@{
       static const char* Description(); //generic conversion options
@@ -193,7 +99,7 @@ namespace OpenBabel {
       std::istream* GetInStream() const {return pInStream;};
       std::ostream* GetOutStream() const {return pOutStream;};
       void          SetInStream(std::istream* pIn)
-        { 
+        {
           if (pInStream && NeedToFreeInStream) {
             delete pInStream; NeedToFreeInStream = false;
           }
@@ -220,12 +126,12 @@ namespace OpenBabel {
       OBFormat*   GetInFormat() const{return pInFormat;};
       OBFormat*   GetOutFormat() const{return pOutFormat;};
       std::string GetInFilename() const{return InFilename;};
-	
+
       ///Get the position in the input stream of the object being read
-      std::streampos GetInPos()const{return wInpos;}; 
+      std::streampos GetInPos()const{return wInpos;};
 
       ///Get the length in the input stream of the object being read
-      size_t GetInLen()const{return wInlen;}; 
+      size_t GetInLen()const{return wInlen;};
 
       /// \return a default title which is the filename
       const char* GetTitle() const;
@@ -234,21 +140,56 @@ namespace OpenBabel {
       OBConversion* GetAuxConv() const {return pAuxConv;};
       void          SetAuxConv(OBConversion* pConv) {pAuxConv=pConv;};
       //@}
-      /// @name Option handling
+      /** @name Option handling
+       Three types of Option provide information and control instructions to the
+       conversion process, INOPTIONS, OUTOPTIONS, GENOPTIONS, and are stored in each
+       OBConversion object in separate maps. Each option has an id and an optional
+       text string. They are set individually by AddOption() or (rarely) collectively
+       in SetOptions(). Options cannot be altered but can be replaced with AddOption()
+       and deleted with RemoveOption(), which, however, should be used in an op derived
+       from OBOp (because of iterator invalidation).
+
+       If the "Convert" interface is used, the GENOPTIONS are acted upon in the
+       OBBase::DoTransformations() functions (currently only OBMol has one). This
+       happens after the object has been input but before it has been output.
+       All the options are available to input and output formats, etc. via the IsOption()
+       function, and the interpretation of any text string needs to be done subsequently.
+
+       In the commandline interface, options with single character ids are are indicated
+       like -s, and those with multiple character ids like --gen3D. An option may have
+       one or more parameters which appear, space separated, in the option's text string.
+       With babel, unless the option is at the end of the command, it is necessary for
+       the number of its parameters to be exactly that specified in RegisterOptionParam().
+       The default is 0, but if it is more, and babel is likely to be used, this function
+       should be called in the constructor of a format or op.
+       With obabel (or the GUI), it is not necessary to call RegisterOptionParam().
+
+       New GENOPTIONS can be defined (as plugins) using the class OBOp.
+
+       It is customary for a format or op to document any INOPTIONS or OUTPTIONS it
+       uses in its Description() function. As well as providing documentation during
+       use, this is also parsed by the GUI to construct its checkboxes,etc., so it is
+       advisable to give new Descriptions the same form as existing ones.
+
+       Some conversion options, such as -f, -l, -m, are unlikely to be used in
+       programming, but are listed in OBConversion::Description().  The built-in
+       GENOPTIONS for OBMol objects are listed in OBMol::ClassDescription() which
+       is in transform.cpp and also in this documentation under AddOption().
+       */
       //@{
       ///@brief Three types of options set on the the command line by -a? , -x? , or -?
-      enum Option_type { INOPTIONS, OUTOPTIONS, GENOPTIONS };
+      enum Option_type { INOPTIONS, OUTOPTIONS, GENOPTIONS, ALL };
 
-      ///@brief Determine whether an option is set. \return NULL if option not and a pointer to the associated text if it is 
+      ///@brief Determine whether an option is set. \return NULL if option not and a pointer to the associated text if it is
       const char* IsOption(const char* opt,Option_type opttyp=OUTOPTIONS);
-	
+
       ///@brief Access the map with option name as key and any associated text as value
       const std::map<std::string,std::string>* GetOptions(Option_type opttyp)
         { return &OptionsArray[opttyp];};
 
       ///@brief Set an option of specified type, with optional text
-      void AddOption(const char* opt, Option_type opttyp, const char* txt=NULL);
-	
+      void AddOption(const char* opt, Option_type opttyp=OUTOPTIONS, const char* txt=NULL);
+
       bool RemoveOption(const char* opt, Option_type optype);
 
       ///@brief Set several single character options of specified type from string like ab"btext"c"ctext"
@@ -262,7 +203,10 @@ namespace OpenBabel {
       static int GetOptionParams(std::string name, Option_type typ);
       //@}
 
-      /// @name Supported file format 
+      ///@brief Copies the options (by default of all types) from one OBConversion Object to another.
+      void CopyOptions(OBConversion* pSourceConv, Option_type typ=ALL);
+
+      /// @name Supported file format
       //@{
       // @brief Set and return the list of supported input format
       std::vector<std::string> GetSupportedInputFormat();
@@ -286,30 +230,35 @@ namespace OpenBabel {
 
       /// @name Conversion loop control
       //@{
-      bool				AddChemObject(OBBase* pOb);///< @brief Adds to internal array during input
-      OBBase*			GetChemObject(); ///< @brief Retrieve from internal array during output
-      bool				IsLast();///< @brief True if no more objects to be output
-      bool				IsFirstInput();///< @brief True if the first input object is being processed
-      int         GetOutputIndex() const ;///< @brief Retrieves number of ChemObjects that have been actually output
-      void				SetOutputIndex(int indx);///< @brief Sets ouput index (maybe to control whether seen as first object)
-      void				SetMoreFilesToCome();///<@brief Used with multiple input files. Off by default.
-      void				SetOneObjectOnly(bool b=true);///<@brief Used with multiple input files. Off by default.
-      void        SetLast(bool b){SetOneObjectOnly(b);}///@brief.Synonym for SetOneObjectOnly()
+      int     AddChemObject(OBBase* pOb);///< @brief Adds to internal array during input
+      OBBase*  GetChemObject(); ///< @brief Retrieve from internal array during output
+      bool     IsLast();///< @brief True if no more objects to be output
+      bool     IsFirstInput();///< @brief True if the first input object is being processed
+      void     SetFirstInput(bool b=true);///< @brief Setwhether or not is the first input
+      int      GetOutputIndex() const ;///< @brief Retrieves number of ChemObjects that have been actually output
+      void     SetOutputIndex(int indx);///< @brief Sets ouput index (maybe to control whether seen as first object)
+      void     SetMoreFilesToCome();///<@brief Used with multiple input files. Off by default.
+      void     SetOneObjectOnly(bool b=true);///< @brief Used with multiple input files. Off by default.
+      void     SetLast(bool b){SetOneObjectOnly(b);}///< @brief Synonym for SetOneObjectOnly()
+      bool     IsLastFile(){ return !MoreFilesToCome;}///< @brief True if no more files to be read
+      /// @brief Number of objects read and processed
+      /// Incremented after options are processed, so 0 for first object.  Returns -1 if Convert interface not used. 
+      int      GetCount()const { return Count; }
       //@}
       /// @name Convenience functions
       //@{
-      ///The default format is set in a single OBFormat class (generally it is OBMol) 
-      static OBFormat* GetDefaultFormat(){return pDefaultFormat;};
+      ///The default format is set in a single OBFormat class (generally it is OBMol)
+      static OBFormat* GetDefaultFormat(){return OBFormat::FindType(NULL);};
 
       /// @brief Outputs an object of a class derived from OBBase.
-	
-      /// Part of "API" interface. 
+
+      /// Part of "API" interface.
       /// The output stream can be specified and the change is retained in the OBConversion instance
       bool				Write(OBBase* pOb, std::ostream* pout=NULL);
 
       /// @brief Outputs an object of a class derived from OBBase as a string
-	
-      /// Part of "API" interface. 
+
+      /// Part of "API" interface.
       /// The output stream is temporarily changed to the string and then restored
       /// This method is primarily intended for scripting languages without "stream" classes
       /// The optional "trimWhitespace" parameter allows trailing whitespace to be removed
@@ -317,8 +266,8 @@ namespace OpenBabel {
       std::string                     WriteString(OBBase* pOb, bool trimWhitespace = false);
 
       /// @brief Outputs an object of a class derived from OBBase as a file (with the supplied path)
-	
-      /// Part of "API" interface. 
+
+      /// Part of "API" interface.
       /// The output stream is changed to the supplied file and the change is retained in the
       /// OBConversion instance.
       /// This method is primarily intended for scripting languages without "stream" classes
@@ -331,28 +280,52 @@ namespace OpenBabel {
       void CloseOutFile();
 
       /// @brief Reads an object of a class derived from OBBase into pOb.
-	
-      /// Part of "API" interface. 
+
+      /// Part of "API" interface.
       /// The input stream can be specified and the change is retained in the OBConversion instance
-      /// \return false and pOb=NULL on error 
+      /// \return false and pOb=NULL on error
       bool	Read(OBBase* pOb, std::istream* pin=NULL);
 
+      /// Part of "API" interface.
+      /// The input stream can be specified and the change is retained in the OBConversion instance
+      /// \return NULL on error
+//      OBBase*	ReadObject(std::istream* pin=NULL);
+
       /// @brief Reads an object of a class derived from OBBase into pOb from the supplied string
-	
-      /// Part of "API" interface. 
+
+      /// Part of "API" interface.
       /// \return false and pOb=NULL on error
       /// This method is primarily intended for scripting languages without "stream" classes
       bool	ReadString(OBBase* pOb, std::string input);
 
       /// @brief Reads an object of a class derived from OBBase into pOb from the file specified
-	
-      /// Part of "API" interface. 
+
+      /// Part of "API" interface.
       /// The output stream is changed to the supplied file and the change is retained in the
       /// OBConversion instance.
-      /// \return false and pOb=NULL on error 
+      /// \return false and pOb=NULL on error
       /// This method is primarily intended for scripting languages without "stream" classes
       bool	ReadFile(OBBase* pOb, std::string filePath);
 
+      /// Part of the "Convert" interface
+      /// Open the files and update the streams in the OBConversion object.
+      /// This method is primarily intended for scripting languages without "stream" classes
+      /// and will usually followed by a call to Convert()
+      /// \return false if unsucessful.
+      bool OpenInAndOutFiles(std::string infilepath, std::string outfilepath);
+
+      /// @brief Sends a message like "2 molecules converted" to clog
+      /// The type of object is taken from the TargetClassDescription
+      /// of the specified class (or the output format if not specified)and
+      /// is appropriately singular or plural.
+      void ReportNumberConverted(int count, OBFormat* pFormat=NULL);
+
+      /// \return the number of objects in the inputstream,
+      /// or -1 if error or if SkipObjects for the input format is not implemented
+      /// Adjusts for the value of -f and -l options (first and last objects).
+      int NumInputObjects();
+
+
 protected:
       ///Replaces * in BaseName by InFile without extension and path
       static std::string BatchFileName(std::string& BaseName, std::string& InFile);
@@ -360,17 +333,19 @@ protected:
       static std::string IncrementedFileName(std::string& BaseName, const int Count);
       ///Checks for misunderstandings when using the -m option
       static bool CheckForUnintendedBatch(const std::string& infile, const std::string& outfile);
+      ///Adds a filtering rdbuffer to handle line endings if not already installed and not a binary or xml format.
+      void InstallStreamFilter();
 
       //@}
 
     protected:
       bool             SetStartAndEnd();
-      static FMapType& FormatsMap();///<contains ID and pointer to all OBFormat classes
-      static FMapType& FormatsMIMEMap();///<contains MIME and pointer to all OBFormat classes
+//      static FMapType& FormatsMap();///<contains ID and pointer to all OBFormat classes
+//      static FMapType& FormatsMIMEMap();///<contains MIME and pointer to all OBFormat classes
       typedef std::map<std::string,int> OPAMapType;
       static OPAMapType& OptionParamArray(Option_type typ);
       static int       LoadFormatFiles();
-      bool             OpenAndSetFormat(bool SetFormat, std::ifstream* is);
+      bool             OpenAndSetFormat(bool SetFormat, std::ifstream* is, std::stringstream* ss=NULL);
 
       std::string	  InFilename;
       std::istream*     pInStream;
@@ -393,6 +368,8 @@ protected:
       bool      CheckedForGzip;      ///< input stream is gzip-encoded
       bool      NeedToFreeInStream;
       bool      NeedToFreeOutStream;
+      typedef   FilteringInputStreambuf< LineEndingExtractor > LErdbuf;
+      LErdbuf*  pLineEndBuf;
 
       static int FormatFilesLoaded;
       OBBase*		  pOb1;
@@ -400,7 +377,7 @@ protected:
       std::streampos rInpos; ///<position in the input stream of the object being read
       size_t wInlen; ///<length in the input stream of the object being written
       size_t rInlen; ///<length in the input stream of the object being read
-	
+
       OBConversion* pAuxConv;///<Way to extend OBConversion
 
       std::vector<std::string> SupportedInputFormat; ///< list of supported input format
@@ -408,21 +385,10 @@ protected:
 
     };
 
-  ///For OBFormat::Flags()
-#define NOTREADABLE     0x01
-#define READONEONLY     0x02
-#define READBINARY      0x04
-#define ZEROATOMSOK     0x08
-#define NOTWRITABLE     0x10
-#define WRITEONEONLY    0x20
-#define WRITEBINARY     0x40
-#define READXML         0x80
-#define DEFAULTFORMAT   0x4000
-
 } //namespace OpenBabel
 #endif //OB_CONV_H
 
 //! \file
 //! \brief Handle file conversions. Declaration of OBFormat, OBConversion.
 
- 
+
