@@ -1,16 +1,16 @@
 /**********************************************************************
 parsmart.h - Daylight SMARTS parser.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2005 by Geoffrey R. Hutchison
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -23,6 +23,7 @@ GNU General Public License for more details.
 #include <string>
 #include <vector>
 
+#include <openbabel/babelconfig.h>
 #include <openbabel/mol.h>
 
 /*==========================*/
@@ -86,6 +87,7 @@ namespace OpenBabel
 #define BT_UPUNSPEC   0x07
 #define BT_DOWNUNSPEC 0x08
 #define BT_RING       0x09
+#define BT_QUAD       0x0A //quadruple bond $
 
   //! \union _BondExpr parsmart.h <openbabel/parsmart.h>
   //! \brief An internal (SMARTS parser) bond expression
@@ -147,42 +149,87 @@ namespace OpenBabel
     BondSpec *bond;
     int parts;
     bool hasExplicitH;
+    std::vector<int> bond_parse_order; // Used to recover the order in which bonds were parsed
   }
   Pattern;
+
+  //! \struct ParseState parsmart.h <openbabel/parsmart.h>
+  //! \brief A SMARTS parser internal state
+  typedef struct
+  {
+    BondExpr *closord[100];
+    int       closure[100];
+    int       closindex;
+  } ParseState;
+
 #else
   // for SWIG, just forward declare that we have some Pattern struct
   // (but this is private and not wrapped for scripting languages)
   struct Pattern;
 #endif
 
+  //! Internal class for extending OBSmartsPattern
+  class OBSmartsPrivate;
+
+  ///@addtogroup substructure Substructure Searching
+  ///@{
+
   // class introduction in parsmart.cpp
   //! \brief SMARTS (SMiles ARbitrary Target Specification) substructure searching
   class OBAPI OBSmartsPattern
   {
   protected:
+    OBSmartsPrivate                *_d;        //!< Internal data storage for future expansion
     std::vector<bool>          		  _growbond; //!< \deprecated (Not used)
     std::vector<std::vector<int> >	_mlist;    //!< The list of matches
     Pattern                        *_pat;      //!< The parsed SMARTS pattern
     std::string				              _str;      //!< The string of the SMARTS expression
 
+    char *_buffer;
+    char *LexPtr;
+    char *MainPtr;
+
+    Pattern *ParseSMARTSPattern( void );
+    Pattern *ParseSMARTSPart( Pattern*, int );
+    Pattern *SMARTSError( Pattern *pat );
+    Pattern *ParseSMARTSError( Pattern *pat, BondExpr *expr );
+    AtomExpr *ParseSimpleAtomPrimitive( void );
+    AtomExpr *ParseComplexAtomPrimitive( void );
+    AtomExpr *ParseAtomExpr( int level );
+    BondExpr *ParseBondPrimitive( void );
+    BondExpr *ParseBondExpr( int level );
+    Pattern *ParseSMARTSString( char *ptr );
+    Pattern *ParseSMARTSRecord( char *ptr );
+    int GetVectorBinding();
+    Pattern *SMARTSParser( Pattern *pat, ParseState *stat,
+                           int prev, int part );
+
   public:
-    OBSmartsPattern() : _pat(NULL) { }
+  OBSmartsPattern() : _pat(NULL), _buffer(NULL), LexPtr(NULL), MainPtr(NULL) { }
     virtual ~OBSmartsPattern();
 
-  OBSmartsPattern(const OBSmartsPattern& cp): _pat(NULL)
+  OBSmartsPattern(const OBSmartsPattern& cp): _pat(NULL), _buffer(NULL), LexPtr(NULL), MainPtr(NULL)
       {
         *this = cp;
       }
+
     OBSmartsPattern& operator=(const OBSmartsPattern& cp)
       {
+        if (this == &cp)
+          return *this;
+
         if (_pat)
-          delete [] _pat;
+          delete[] _pat;
+        if (_buffer)
+          delete[] _buffer;
+        _buffer = NULL;
         _pat = NULL;
         std::string s = cp._str;
         Init(s);
         return (*this);
       }
-    
+
+
     //! \name Initialization Methods
     //@{
     //! Parse the @p pattern SMARTS string.
@@ -234,6 +281,9 @@ namespace OpenBabel
     }
     //@}
 
+    // number and kind of matches to return
+    enum MatchType {All, Single, AllUnique};
+
     //! \name Matching methods (SMARTS on a specific OBMol)
     //@{
     //! Perform SMARTS matching for the pattern specified using Init().
@@ -242,14 +292,31 @@ namespace OpenBabel
     //! \return Whether matches occurred
     bool Match(OBMol &mol, bool single=false);
 
+    //! \name Matching methods (SMARTS on a specific OBMol)
+    //@{
+    //! Perform SMARTS matching for the pattern specified using Init().
+    //! This version is (more) thread safe.
+    //! \param mol The molecule to use for matching
+    //! \param mlist The resulting match list
+    //! \param mtype The match type to use. Default is All.
+    //! \return Whether matches occurred
+    bool Match(OBMol &mol, std::vector<std::vector<int> > & mlist, MatchType mtype = All) const;
+
+    //! \name Matching methods (SMARTS on a specific OBMol)
+    //@{
+    //! Thread safe check for any SMARTS match
+    //! \param mol The molecule to use for matching
+    //! \return Whether there exists any match
+    bool HasMatch(OBMol &mol) const;
+
     bool RestrictedMatch(OBMol &mol, std::vector<std::pair<int,int> > &pairs, bool single=false);
 
     bool RestrictedMatch(OBMol &mol, OBBitVec &bv, bool single=false);
-    //! \return the number of non-unique SMARTS matches 
+    //! \return the number of non-unique SMARTS matches
     //! To get the number of unique SMARTS matches, query GetUMapList()->size()
     unsigned int NumMatches() const
     {
-      return (unsigned int)_mlist.size();
+      return static_cast<unsigned int>(_mlist.size());
     }
 
     //! \return the entire list of non-unique matches for this pattern
@@ -271,14 +338,14 @@ namespace OpenBabel
 
     //! \return the entire list of unique matches for this pattern
     /**
-        A unique match is defined as one which does not cover the 
-        identical atoms that a previous match has covered.
-        
-        For instance, the pattern [OD1]~C~[OD1] describes a
-        carboxylate group. This pattern will match both atom number
-        permutations of the carboxylate, and if GetMapList() is called, both
-        matches will be returned. If GetUMapList() is called only unique
-        matches of the pattern will be returned.
+       A unique match is defined as one which does not cover the
+       identical atoms that a previous match has covered.
+
+       For instance, the pattern [OD1]~C~[OD1] describes a
+       carboxylate group. This pattern will match both atom number
+       permutations of the carboxylate, and if GetMapList() is called, both
+       matches will be returned. If GetUMapList() is called only unique
+       matches of the pattern will be returned.
     **/
     std::vector<std::vector<int> > &GetUMapList();
     //@}
@@ -287,19 +354,53 @@ namespace OpenBabel
     void         WriteMapList(std::ostream&);
   };
 
+  ///@}
+
+  //! \class OBSmartsMatcher parsmart.h <openbabel/parsmart.h>
+  //! \brief Internal class: performs matching; a wrapper around previous
+  //! C matching code to make it thread safe.
+  class OBAPI OBSmartsMatcher
+  {
+  protected:
+	  //recursive smarts cache
+	  std::vector<std::pair<const Pattern*,std::vector<bool> > > RSCACHE;
+	  // list of fragment patterns (e.g., (*).(*)
+	  std::vector<const Pattern*> Fragments;
+    /*
+      bool EvalAtomExpr(AtomExpr *expr,OBAtom *atom);
+      bool EvalBondExpr(BondExpr *expr,OBBond *bond);
+      int GetVectorBinding();
+      int CreateAtom(Pattern*,AtomExpr*,int,int vb=0);
+    */
+    bool EvalAtomExpr(AtomExpr *expr,OBAtom *atom);
+    bool EvalBondExpr(BondExpr *expr,OBBond *bond);
+    void SetupAtomMatchTable(std::vector<std::vector<bool> > &ttab,
+	                           const Pattern *pat, OBMol &mol);
+    void FastSingleMatch(OBMol &mol,const Pattern *pat,
+                         std::vector<std::vector<int> > &mlist);
+
+    friend class OBSSMatch;
+  public:
+    OBSmartsMatcher() {}
+    virtual ~OBSmartsMatcher() {}
+
+    bool match(OBMol &mol, const Pattern *pat,std::vector<std::vector<int> > &mlist,bool single=false);
+
+  };
+
   //! \class OBSSMatch parsmart.h <openbabel/parsmart.h>
-  //! \brief Internal class: performs fast, exhaustive matching used to find 
+  //! \brief Internal class: performs fast, exhaustive matching used to find
   //! just a single match in match() using recursion and explicit stack handling.
   class OBAPI OBSSMatch
   {
   protected:
     bool        *_uatoms;
     OBMol       *_mol;
-    Pattern     *_pat;
+    const Pattern     *_pat;
     std::vector<int>  _map;
 
   public:
-    OBSSMatch(OBMol&,Pattern*);
+    OBSSMatch(OBMol&,const Pattern*);
     ~OBSSMatch();
     void Match(std::vector<std::vector<int> > &v, int bidx=-1);
   };
