@@ -1,17 +1,17 @@
 /**********************************************************************
 atom.cpp - Handle OBAtom class.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2008 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2003 by Michael Banck
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,9 +20,11 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 
 #include <openbabel/atom.h>
+#include <openbabel/stereo/stereo.h>
 #include <openbabel/mol.h>
 #include <openbabel/molchrg.h>
 #include <openbabel/phmodel.h>
+#include <openbabel/builder.h>
 
 #include <openbabel/math/matrix3x3.h>
 
@@ -37,13 +39,13 @@ namespace OpenBabel
 
   /** \class OBAtom atom.h <openbabel/atom.h>
       \brief Atom class
- 
+
       To understand the OBAtom class it is important to state a key
       decision on which the design was based. The OBAtom class not only
       holds data, but facilitates extraction of data perceived from both
       the atom and the molecule. This prevents the OBMol class from
       becoming overly large and complicated.
- 
+
       A number of data extraction methods perform what is called
       <a href="http://en.wikipedia.org/wiki/Lazy_evaluation">`Lazy Evaluation,'</a>
       which is essentially on-the-fly evaluation.
@@ -55,7 +57,7 @@ namespace OpenBabel
       trait of additional atoms or bonds.The OBAtom class is similar to
       OBMol and the whole of Open Babel in that data access and modification
       is done through Get and Set methods.
- 
+
       The following code demonstrates how to print out the atom numbers,
       element numbers, and coordinates of a molecule:
       \code
@@ -141,6 +143,7 @@ namespace OpenBabel
     _vbond.clear();
     _vbond.reserve(4);
     _residue = (OBResidue*)NULL;
+    _id = NoId;
 
     return(OBBase::Clear());
   }
@@ -149,16 +152,18 @@ namespace OpenBabel
   //copy atom information
   //bond info is not copied here as ptrs may be invalid
   {
-    _idx = src.GetIdx();
-    Duplicate(&src);
+    if (this != &src) {
+      _idx = src.GetIdx();
+      Duplicate(&src);
+    }
     return(*this);
   }
-  
+
   void OBAtom::Duplicate(OBAtom *src)
   {
     if (!src)
       return;
-      
+
     _hyb = src->GetHyb();
     _ele = src->GetAtomicNum();
     _isotope = src->GetIsotope();
@@ -170,7 +175,7 @@ namespace OpenBabel
     _v = src->GetVector();
     _flags = src->GetFlag();
     _residue = (OBResidue*)NULL;
-    
+
     _vdata.clear();
     //Copy all the OBGenericData, providing the new atom
     vector<OBGenericData*>::iterator itr;
@@ -381,15 +386,15 @@ namespace OpenBabel
       }
   }
 
-  void OBAtom::SetVector(const double x,const double y,const double z)
+  void OBAtom::SetVector(const double v_x,const double v_y,const double v_z)
   {
     if (!_c)
-      _v.Set(x,y,z);
+      _v.Set(v_x,v_y,v_z);
     else
       {
-        (*_c)[_cidx  ] = x;
-        (*_c)[_cidx+1] = y;
-        (*_c)[_cidx+2] = z;
+        (*_c)[_cidx  ] = v_x;
+        (*_c)[_cidx+1] = v_y;
+        (*_c)[_cidx+2] = v_z;
       }
   }
 
@@ -483,7 +488,7 @@ namespace OpenBabel
         ttab.SetToType("INT");
         snprintf(num, 6, "%d", GetAtomicNum());
         ttab.Translate(_type, num);
-	
+
         ttab.SetFromType(fromType.c_str());
         ttab.SetToType(toType.c_str());
       }
@@ -495,6 +500,9 @@ namespace OpenBabel
 
   unsigned int OBAtom::GetImplicitValence() const
   {
+    //Special case for [H] to avoid infite loop: SMARTS Match() <-> AssignSpinMultiplicity()
+    if(GetAtomicNum() == 1)
+      return 1;
     OBMol *mol = (OBMol*)((OBAtom*)this)->GetParent();
     if (mol && !mol->HasImplicitValencePerceived())
       atomtyper.AssignImplicitValence(*((OBMol*)((OBAtom*)this)->GetParent()));
@@ -776,19 +784,12 @@ namespace OpenBabel
     return(false);
   }
 
+  //! @todo
   bool OBAtom::IsChiral()
   {
-    if (HasFlag(OB_CHIRAL_ATOM))
-      return(true);
-
-    if (!((OBMol*)GetParent())->HasChiralityPerceived())
-      {
-        ((OBMol*)GetParent())->FindChiralCenters();
-        if (HasFlag(OB_CHIRAL_ATOM))
-          return(true);
-      }
-
-    return(false);
+    OBMol *mol = (OBMol*) GetParent();
+    OBStereoFacade stereoFacade(mol);
+    return stereoFacade.HasTetrahedralStereo(_id);
   }
 
   bool OBAtom::IsInRingSize(int size) const
@@ -805,7 +806,7 @@ namespace OpenBabel
 
     rlist = mol->GetSSSR();
     for (i = rlist.begin();i != rlist.end();++i)
-      if ((*i)->IsInRing(GetIdx()) && (*i)->PathSize() == size)
+      if ((*i)->IsInRing(GetIdx()) && static_cast<int>((*i)->PathSize()) == size)
         return(true);
 
     return(false);
@@ -1112,153 +1113,7 @@ namespace OpenBabel
 
   bool OBAtom::GetNewBondVector(vector3 &v,double length)
   {
-    // ***experimental code***
-
-    OBAtom *atom;
-    OBBondIterator i,j;
-    v = VZero;
-
-    if (GetValence() == 0)
-      {
-        v = VX;
-        v *= length;
-        v += GetVector();
-        // Check that the vector is still finite before returning
-        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
-          v.Set(0.0, 0.0, 0.0);
-        return(true);
-      }
-
-    if (GetValence() == 1)
-      {
-        vector3 vtmp,v1,v2;
-        atom = BeginNbrAtom(i);
-        if (atom)
-          vtmp = GetVector() - atom->GetVector();
-
-        if (GetHyb() == 2 || (IsOxygen() && HasAlphaBetaUnsat()))
-          {
-            bool quit = false;
-            OBAtom *a1,*a2;
-            v2 = VZero;
-            for (a1 = BeginNbrAtom(i);a1 && !quit;a1 = NextNbrAtom(i))
-              for (a2 = a1->BeginNbrAtom(j);a2 && !quit;a2 = a1->NextNbrAtom(j))
-                if (a1 && a2 && a2 != this)
-                  {
-                    v2 = a1->GetVector() - a2->GetVector();
-                    quit = true;
-                  }
-
-            if (v2.IsApprox(VZero, 1.0e-8))
-              {
-                v1 = cross(vtmp,VX);
-                v2 = cross(vtmp,VY);
-                if (v1.length() < v2.length())
-                  v1 = v2;
-              }
-            else
-              v1 = cross(vtmp,v2);
-
-            matrix3x3 m;
-            m.RotAboutAxisByAngle(v1,60.0);
-            v = m*vtmp;
-            v.normalize();
-          }
-        else if (GetHyb() == 3)
-          {
-            v1 = cross(vtmp,VX);
-            v2 = cross(vtmp,VY);
-            if (v1.length() < v2.length())
-              v1 = v2;
-            matrix3x3 m;
-            m.RotAboutAxisByAngle(v1,70.5);
-            v = m*vtmp;
-            v.normalize();
-          }
-        if (GetHyb() == 1)
-          v = vtmp;
-
-        v *= length;
-        v += GetVector();
-        // Check that the vector is still finite before returning
-        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
-          v.Set(0.0, 0.0, 0.0);
-        return(true);
-      }
-
-    if (GetValence() == 2)
-      {
-        vector3 v1,v2,vtmp,vsum,vnorm;
-        atom = BeginNbrAtom(i);
-        if (!atom)
-          return(false);
-        v1 = GetVector() - atom->GetVector();
-        atom = NextNbrAtom(i);
-        if (!atom)
-          return(false);
-        v2 = GetVector() - atom->GetVector();
-        v1.normalize();
-        v2.normalize();
-        vsum = v1+v2;
-        vsum.normalize();
-
-        if (GetHyb() == 2)
-          v = vsum;
-        else if (GetHyb() == 3)
-          {
-            vnorm = cross(v2,v1);
-            vnorm.normalize();
-
-#ifndef ONE_OVER_SQRT3
-#define ONE_OVER_SQRT3  0.57735026918962576451
-#endif //SQRT_TWO_THIRDS
-#ifndef SQRT_TWO_THIRDS
-#define SQRT_TWO_THIRDS 0.81649658092772603272
-#endif //ONE_OVER_SQRT3
-
-            vsum *= ONE_OVER_SQRT3;
-            vnorm *= SQRT_TWO_THIRDS;
-
-            v = vsum + vnorm;
-          }
-
-        v *= length;
-
-        v += GetVector();
-        // Check that the vector is still finite before returning
-        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
-          v.Set(0.0, 0.0, 0.0);
-        return(true);
-      }
-
-    if (GetValence() == 3)
-      {
-        vector3 vtmp,vsum;
-        OBAtom *atom;
-        OBBondIterator i;
-        for (atom = BeginNbrAtom(i);atom;atom = NextNbrAtom(i))
-          {
-            vtmp = GetVector() - atom->GetVector();
-            if (vtmp.length_2() < 0.1) { // atoms are nearly on top of each other
-              // Move the other atom a small random distance apart
-              vtmp.randomUnitVector();
-              vtmp *= length;
-              atom->SetVector(vtmp + GetVector());
-            } // Now this should be a real distance, not a zero vector
-            vtmp.normalize();
-            vtmp /= 3.0;
-            vsum += vtmp;
-          }
-        vsum.normalize();
-        v = vsum;
-        v *= length;
-        v += GetVector();
-        // Check that the vector is still finite before returning
-        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
-          v.Set(0.0, 0.0, 0.0);
-        return(true);
-      }
-
+    v = OBBuilder::GetNewBondVector(this, length);
     return(true);
   }
 
@@ -1332,6 +1187,7 @@ namespace OpenBabel
 
   }
 
+  //! \deprecated This will be removed in future versions of Open Babel
   bool OBAtom::SetHybAndGeom(int hyb)
   {
     obErrorLog.ThrowError(__FUNCTION__,
@@ -1529,6 +1385,12 @@ namespace OpenBabel
                 s.normalize();
                 n = cross(v1,v2);
                 n.normalize();
+#ifndef ONE_OVER_SQRT3
+#define ONE_OVER_SQRT3  0.57735026918962576451
+#endif //SQRT_TWO_THIRDS
+#ifndef SQRT_TWO_THIRDS
+#define SQRT_TWO_THIRDS 0.81649658092772603272
+#endif //ONE_OVER_SQRT3
                 s *= ONE_OVER_SQRT3; //1/sqrt(3)
                 n *= SQRT_TWO_THIRDS; //sqrt(2/3)
                 s += n;
@@ -1814,7 +1676,7 @@ namespace OpenBabel
       return true;
     if (_ele == 7) {
       // N+ ions and sp2 hybrid N with 3 valences should not be Hbond acceptors
-      if (!((GetValence() == 4 && GetHyb() == 3) 
+      if (!((GetValence() == 4 && GetHyb() == 3)
             || (GetValence() == 3 && GetHyb() == 2)))
             return true;
     }
