@@ -1,18 +1,18 @@
 /**********************************************************************
 chains.cpp - Parse for macromolecule chains and residues.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 (original author, Roger Sayle, version 1.6, March 1998)
 (modified by Joe Corkery, OpenEye Scientific Software, March 2001)
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -41,16 +41,17 @@ using namespace std;
 
 //! The first available index for actual residues
 //! 0, 1, 2 reserved for UNK, HOH, LIG
-#define RESIDMIN       3
+#define RESIDMIN       4
 //! The maximum number of residue IDs for this code
-#define RESIDMAX       32
+#define RESIDMAX       64
 
 //! An index of the residue names perceived during a run
 //! 0, 1, and 2 reserved for UNK, HOH, LIG
 static char ChainsResName[RESIDMAX][4] = {
-  /*0*/ "UNK",  
-  /*1*/ "HOH",  
-  /*2*/ "LIG"
+  /*0*/ "UNK",
+  /*1*/ "HOH",
+  /*2*/ "LIG",
+  /*3*/ "ACE"
 };
 
 #define ATOMMINAMINO   4
@@ -154,22 +155,27 @@ namespace OpenBabel
     int n2;          //!< mask 2 used by ConstrainBackbone() and MatchConstraint()
     int n3;          //!< mask 3 used by ConstrainBackbone() and MatchConstraint()
     int n4;          //!< mask 4 used by ConstrainBackbone() and MatchConstraint()
-  }
-    Template;
+  }    Template;
 
-  //! Generic template for peptide residue backbone
+  /**
+   * Generic template for peptide residue backbone. \n
+   * col 1: bitmask \n
+   * col 2: element number \n
+   * col 3: neighbour count \n
+   * col 4-7: 1-4 bitmasks for neighbour atoms (-6 means carbon)
+   */
   static Template Peptide[MAXPEPTIDE] = {
-    /* N     */    {  0x0001, 7, 2, 0x0030, 0x0100,      0, 0 },
-    /* NTer  */    {  0x0002, 7, 1, 0x0030,      0,      0, 0 },
-    /* NPro  */    {  0x0004, 7, 3, 0x0030, 0x0100,     -6, 0 },
-    /* NPT   */    {  0x0008, 7, 2, 0x0030,     -6,      0, 0 },
-    /* CA    */    {  0x0010, 6, 3, 0x000F, 0x0700,     -6, 0 },
-    /* CAGly */    {  0x0020, 6, 2, 0x0003, 0x0700,      0, 0 },
-    /* C     */    {  0x0100, 6, 3, 0x0030, 0x1000, 0x0005, 0 },
-    /* CTer  */    {  0x0200, 6, 2, 0x0030, 0x1000,      0, 0 },
-    /* COXT  */    {  0x0400, 6, 3, 0x0030, 0x1000, 0x2000, 0 },
-    /* O     */    {  0x1000, 8, 1, 0x0700,      0,      0, 0 },
-    /* OXT   */    {  0x2000, 8, 1, 0x0400,      0,      0, 0 }
+    /* N     */    {  0x0001, 7, 2, 0x0030, 0x0100,      0, 0 }, //!< N
+    /* NTer  */    {  0x0002, 7, 1, 0x0030,      0,      0, 0 }, //!< NTre
+    /* NPro  */    {  0x0004, 7, 3, 0x0030, 0x0100,     -6, 0 }, //!< NPro
+    /* NPT   */    {  0x0008, 7, 2, 0x0030,     -6,      0, 0 }, //!< NPT
+    /* CA    */    {  0x0010, 6, 3, 0x000F, 0x0700,     -6, 0 }, //!< CA
+    /* CAGly */    {  0x0020, 6, 2, 0x0003, 0x0700,      0, 0 }, //!< CAGly
+    /* C     */    {  0x0100, 6, 3, 0x0030, 0x1000, 0x0005, 0 }, //!< C
+    /* CTer  */    {  0x0200, 6, 2, 0x0030, 0x1000,      0, 0 }, //!< CTer
+    /* COXT  */    {  0x0400, 6, 3, 0x0030, 0x1000, 0x2000, 0 }, //!< COXT
+    /* O     */    {  0x1000, 8, 1, 0x0700,      0,      0, 0 }, //!< O
+    /* OXT   */    {  0x2000, 8, 1, 0x0400,      0,      0, 0 }  //!< OXT
   };
 
   //! Generic template for peptide nucleotide backbone
@@ -279,12 +285,13 @@ namespace OpenBabel
   {
     const char *name; //!< Residue name, standardized by PDB
     const char *data; //!< pseudo-SMILES definition of side-chain
-  }
-    ResidType;
+  }    ResidType;
 
-  //! Side chains for recognized amino acids using a pseudo-SMARTS syntax
-  //!  for branching and bonds. Numbers indicate atom types defined by 
-  //!  ChainsAtomName global array above
+  /**
+   * Side chains for recognized amino acids using a pseudo-SMARTS syntax
+   * for branching and bonds. Numbers indicate atom types defined by
+   * OpenBabel::ChainsAtomName global array above.
+   */
   static ResidType AminoAcids[AMINOMAX] = {
     { "ILE", "1-4(-9-14)-10"                        },
     { "VAL", "1-4(-9)-10"                           },
@@ -313,9 +320,11 @@ namespace OpenBabel
   /* Amino-N-Butyric Acid (ABA): 1-4-7                PDB Example: 1BBO */
   /* Selenic Acid (SEC):         1-4-"SEG "(-15)-18   PDB Example: 1GP1 */
 
-  //! Side chains for recognized nucleotides using a pseudo-SMARTS syntax
-  //!  for branching and bonds. Numbers indicate atom types defined by 
-  //!  ChainsAtomName global array above
+  /**
+   * Side chains for recognized nucleotides using a pseudo-SMARTS syntax
+   * for branching and bonds. Numbers indicate atom types defined by
+   * OpenBabel::ChainsAtomName global array above.
+   */
   static ResidType Nucleotides[NUCLEOMAX] = {
     { "  A", "49-50-51-52-53-54(-56)-57-58-61-62(-53)-50"      },
     { "  C", "49-57-58(-59)-61-62(-64)-65-67-57"               },
@@ -331,23 +340,20 @@ namespace OpenBabel
     int atomid,elem;
     int bcount;
     int index;
-  }
-    MonoAtomType;
+  } MonoAtomType;
 
   typedef struct
   {
     int src,dst;
     int index;
     int flag;
-  }
-    MonoBondType;
+  } MonoBondType;
 
   typedef struct
   {
     int type;
     union _ByteCode *next;
-  }
-    MonOpStruct;
+  } MonOpStruct;
 
   typedef struct
   {
@@ -355,8 +361,7 @@ namespace OpenBabel
     int value;
     union _ByteCode *tcond;
     union _ByteCode *fcond;
-  }
-    BinOpStruct;
+  } BinOpStruct;
 
   //! Output array -- residue id, atom id, bond flags, etc.
   typedef struct
@@ -365,8 +370,7 @@ namespace OpenBabel
     int resid;
     int *atomid;
     int *bflags;
-  }
-    AssignStruct;
+  } AssignStruct;
 
   //! Chemical graph matching virtual machine
   typedef union _ByteCode
@@ -384,8 +388,7 @@ namespace OpenBabel
   {
     int atom,bond;
     int prev;
-  }
-    StackType;
+  } StackType;
 
   static MonoAtomType MonoAtom[MaxMonoAtom];
   static MonoBondType MonoBond[MaxMonoBond];
@@ -414,7 +417,7 @@ namespace OpenBabel
       }
     result->type = type;
     result->eval.next     = NULL;
-    result->count.tcond   = NULL; 
+    result->count.tcond   = NULL;
     result->count.fcond   = NULL;
     result->elem.tcond    = NULL;
     result->elem.fcond    = NULL;
@@ -447,7 +450,7 @@ namespace OpenBabel
               delete [] node->assign.bflags;
               node->assign.bflags = NULL; // prevent double-free
             }
-            
+
             break;
 
           case BC_COUNT:
@@ -686,7 +689,7 @@ namespace OpenBabel
       {
         stringstream errorMsg;
         errorMsg << "Maximum Monomer Fanout Exceeded!" << endl;
-        errorMsg << "Residue " << ChainsResName[resid] << " atom " 
+        errorMsg << "Residue " << ChainsResName[resid] << " atom "
                  << curr << endl;
         errorMsg << "Previous = " << prev << " Fanout = " << count << endl;
         obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obWarning);
@@ -723,7 +726,7 @@ namespace OpenBabel
           {
             stringstream errorMsg;
             errorMsg << "Duplicated Monomer Specification!\n";
-            errorMsg << "Residue " << ChainsResName[resid] 
+            errorMsg << "Residue " << ChainsResName[resid]
                      << " matches residue ";
             errorMsg << ChainsResName[(*node)->assign.resid] << endl;
             obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obWarning);
@@ -781,7 +784,7 @@ namespace OpenBabel
   // Setup / Cleanup Functions
   //////////////////////////////////////////////////////////////////////////////
 
-  //! Setup parsing for this molecule -- 
+  //! Setup parsing for this molecule --
   void OBChainsParser::SetupMol(OBMol &mol)
   {
     CleanupMol();
@@ -854,7 +857,7 @@ namespace OpenBabel
     unsigned int numAtoms = mol.NumAtoms();
 
     //DumpState();
-    
+
     // correct serine OG
     for ( unsigned int i = 0 ; i < numAtoms ; i++ ) {
       if (resids[i] == RESIDMIN + 17) // serine
@@ -896,7 +899,6 @@ namespace OpenBabel
           }
         } else {
           snprintf(buffer, BUFF_SIZE, "H%.2s", ChainsAtomName[atomids[i]]+2);
-          cout << " no hcounts " << endl;
         }
       } else
         snprintf(buffer, BUFF_SIZE, "%.4s", ChainsAtomName[atomids[i]]);
@@ -904,8 +906,12 @@ namespace OpenBabel
       if (buffer[3] == ' ')
         buffer[3] = '\0';
 
+      //cout << "  (2) --> = " << buffer << endl;
+
       atomid = (buffer[0] == ' ') ? buffer + 1 : buffer;
 
+      //cout << "  (3) --> = " << buffer << endl;
+
       if (resmap[chains[i]].find(resnos[i]) != resmap[chains[i]].end()) {
         residue = resmap[chains[i]][resnos[i]];
         residue->AddAtom(atom);
@@ -954,7 +960,7 @@ namespace OpenBabel
     result = DetermineNucleicBackbone(mol)   && result;
     result = DetermineNucleicSidechains(mol) && result;
     result = DetermineHydrogens(mol)         && result;
-    
+
     // Partially identified residues
     // example: CSD in 1LWF (CYS with two Os on the S)
     bool changed;
@@ -969,31 +975,38 @@ namespace OpenBabel
           FOR_NBORS_OF_ATOM (nbr, &*atom) {
             unsigned int idx2 = nbr->GetIdx() - 1;
             if (resids[idx2] != 0) { // !UNK
-              resnos[idx] = resnos[idx2];
-              resids[idx] = resids[idx2];
-              changed = true;
-	      
-              bool addResidue = true;
-              for (unsigned int i = 0; i < invalidResidues.size(); ++i)
-                if ( (invalidResidues[i].first == chains[idx2]) && 
-                     (invalidResidues[i].second == resnos[idx2]) )
-                  addResidue = false;
-              if (addResidue)
-                invalidResidues.push_back(pair<char,short>(chains[idx2], resnos[idx2]));
+	      if (atomids[idx2] == AI_N || atomids[idx2] == AI_C) {
+		// bound to backbone-N/C
+		hetflags[idx] = true;
+		resids[idx] = 3; // ACE
+		atomids[idx] = -1;
+	      } else {
+		resnos[idx] = resnos[idx2];
+		resids[idx] = resids[idx2];
+		changed = true;
+
+		bool addResidue = true;
+		for (unsigned int i = 0; i < invalidResidues.size(); ++i)
+		  if ( (invalidResidues[i].first == chains[idx2]) &&
+		       (invalidResidues[i].second == resnos[idx2]) )
+		    addResidue = false;
+		if (addResidue)
+		  invalidResidues.push_back(pair<char,short>(chains[idx2], resnos[idx2]));
+	      }
             }
           }
         }
-        
+
       }
     } while (changed);
     for (unsigned int i = 0; i < invalidResidues.size(); ++i) {
       FOR_ATOMS_OF_MOL (atom, mol) {
         idx = atom->GetIdx() - 1;
-        if ( (invalidResidues[i].first == chains[idx]) && 
+        if ( (invalidResidues[i].first == chains[idx]) &&
              (invalidResidues[i].second == resnos[idx]) ) {
           hetflags[idx] = true;
           resids[idx] = 0; // UNK
-          atomids[idx] = -1; 
+          atomids[idx] = -1;
         }
       }
     }
@@ -1003,17 +1016,17 @@ namespace OpenBabel
     short resno = 1;
     FOR_ATOMS_OF_MOL (atom, mol) {
       idx = atom->GetIdx() - 1;
-      
-      if (atom->GetHvyValence() == 0) { 
+
+      if (atom->GetHvyValence() == 0) {
         chains[idx] = ' ';
         resnos[idx] = resno;
         resno++;
-      } else { 
+      } else {
         if (resids[idx] != 0) // UNK
           continue;
         if (hetflags[idx])
           continue;
-	
+
         char chain = chains[idx];
         FOR_ATOMS_OF_MOL (b, mol) {
           unsigned int idx2 = b->GetIdx() - 1;
@@ -1024,7 +1037,7 @@ namespace OpenBabel
             resids[idx2] = 2; // LIG
           }
         }
-        
+
         resno++;
       }
 
@@ -1047,7 +1060,7 @@ namespace OpenBabel
   {
     OBAtom *atom;
     vector<OBAtom *>::iterator a;
-    
+
     // find un-connected atoms (e.g., HOH oxygen atoms)
     for (atom = mol.BeginAtom(a) ; atom ; atom = mol.NextAtom(a)) {
       if (atom->IsHydrogen() || atom->GetHvyValence() != 0)
@@ -1071,15 +1084,12 @@ namespace OpenBabel
   bool OBChainsParser::DetermineConnectedChains(OBMol &mol)
   {
     int resid;
-    int resno;
-    int count;
-    int size;
+    unsigned int size;
     unsigned int i, idx;
-    unsigned int numAtoms;
 
-    resno    = 1;
-    count    = 0;
-    numAtoms = mol.NumAtoms();
+    int resno = 1;
+    int count = 0;
+    unsigned int numAtoms = mol.NumAtoms();
 
     OBAtom *atom;
     vector<OBAtom *>::iterator a;
@@ -1087,7 +1097,7 @@ namespace OpenBabel
       idx = atom->GetIdx() - 1;
       if (!hetflags[idx] && chains[idx] == ' ' && !atom->IsHydrogen()) {
         size = RecurseChain(mol, idx, 'A' + count);
-            
+
         // size = number of heavy atoms in residue chain
         if (size < 4) { // small ligand, probably
           if (size == 1 && atom->IsOxygen())
@@ -1112,7 +1122,7 @@ namespace OpenBabel
       }
     }
 
-    /* 
+    /*
        if( count == 1 )
        for ( i = 0 ; i < numAtoms ; i++ )
        chains[i] = ' ';
@@ -1121,27 +1131,31 @@ namespace OpenBabel
     return true;
   }
 
-  int OBChainsParser::RecurseChain(OBMol &mol, int i, int c)
+  ///@todo atom index
+  unsigned int OBChainsParser::RecurseChain(OBMol &mol, unsigned int i, int c)
   {
     OBAtom *atom, *nbr;
     vector<OBBond *>::iterator b;
-    int result, index;
+    unsigned int result, index;
 
-    atom      = mol.GetAtom(i+1);
+    atom = mol.GetAtom(i + 1);
+
+    // ignore hydrogens
     if (atom->IsHydrogen() )
       return 0;
 
     result    = 1;
     chains[i] = c;
 
-    for (nbr = atom->BeginNbrAtom(b); nbr; nbr = atom->NextNbrAtom(b))
-      {
-        index = nbr->GetIdx() - 1;
-        if (chains[index] == ' ')
-          result += RecurseChain(mol, index,c);
-      }
+    // recurse till we have all atoms for this chain
+    for (nbr = atom->BeginNbrAtom(b); nbr; nbr = atom->NextNbrAtom(b)) {
+      index = nbr->GetIdx() - 1;
+      if (chains[index] == ' ')
+        result += RecurseChain(mol, index, c);
+    }
 
-    return (result);
+    // and return how many we found
+    return result;
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -1154,7 +1168,7 @@ namespace OpenBabel
 
     unsigned int i, numAtoms = mol.NumAtoms();
 
-    // Cyclic peptides have no NTer (1SKI) 
+    // Cyclic peptides have no NTer (1SKI)
     // timvdm 30/06/08
     bool foundNTer = false;
     for (i = 0 ; i < numAtoms; i++)
@@ -1165,7 +1179,7 @@ namespace OpenBabel
         if (bitmasks[i] & BitNAll)
           bitmasks[i] |= BitNTer;
 
-    
+
     /* Order Peptide Backbone */
 
     for ( i = 0 ; i < numAtoms ; i++ )
@@ -1204,8 +1218,8 @@ namespace OpenBabel
     OBAtom *na,*nb,*nc,*nd;
     OBAtom *atom, *nbr;
     bool change, result;
-    int count;
-    unsigned int i, idx;
+    int i, count;
+    unsigned int idx;
 
     vector<OBAtom *>::iterator a;
     vector<OBBond *>::iterator b;
@@ -1331,15 +1345,15 @@ namespace OpenBabel
     return( false );
   }
 
-  void OBChainsParser::TracePeptideChain(OBMol &mol, int i, int r)
+  void OBChainsParser::TracePeptideChain(OBMol &mol, unsigned int i, int r)
   {
-    int neighbour[4];
-    int na,nb,nc;
+    unsigned int neighbour[4];
+    unsigned int na,nb,nc;
     OBAtom *atom, *nbr;
     int count;
     int j,k;
-    int idx;
-    
+    unsigned int idx;
+
     j = k = 0; // ignore warning
 
     vector<OBBond *>::iterator b;
@@ -1476,7 +1490,7 @@ namespace OpenBabel
         resids[j] = i;
   }
 
-  int OBChainsParser::IdentifyResidue(void *tree, OBMol &mol, int seed,
+  int OBChainsParser::IdentifyResidue(void *tree, OBMol &mol, unsigned int seed,
                                       int resno)
   {
     ByteCode *ptr;
@@ -1574,7 +1588,7 @@ namespace OpenBabel
             ptr = ptr->count.fcond;
           break;
 
-        case(BC_ASSIGN): 
+        case(BC_ASSIGN):
           for( i=0; i<AtomCount; i++ ) {
             if( !bitmasks[ResMonoAtom[i]])
               {
@@ -1627,10 +1641,10 @@ namespace OpenBabel
     return true;
   }
 
-  void OBChainsParser::TraceNucleicChain(OBMol &mol, int i, int r)
+  void OBChainsParser::TraceNucleicChain(OBMol &mol, unsigned int i, int r)
   {
-    int neighbour[4];
-    int na,nb,nc;
+    unsigned int neighbour[4];
+    unsigned int na,nb,nc;
     int count;
     int j,k;
 
