@@ -2,11 +2,11 @@
 Copyright (C) 2006 by Fredrik Wallner
 Some portions Copyright (C) 2006-2007 by Geoffrey Hutchsion
 Some portions Copyright (C) 2004 by Chris Morley
- 
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -15,6 +15,7 @@ GNU General Public License for more details.
 
 #include <openbabel/babelconfig.h>
 #include <openbabel/obmolecformat.h>
+#include <openbabel/reaction.h>
 #include "chemdrawcdx.h"
 
 #include <iostream>
@@ -70,7 +71,7 @@ data = bswap_32 (data);
 (stream).read ((char*)&data, sizeof(data));
 #endif
 // end endian / bigendian issues (on non-Mac systems)
-#endif 
+#endif
 // end Apple/non-Apple systems
 
 using namespace std;
@@ -97,10 +98,10 @@ namespace OpenBabel
     virtual const char* SpecificationURL()
     {return "http://www.cambridgesoft.com/services/documentation/sdk/chemdraw/cdx/IntroCDX.htm";};
 
-    virtual const char* GetMIMEType() 
+    virtual const char* GetMIMEType()
     { return "chemical/x-cdx"; };
 
-    /* Flags() can return be any of the following combined by | 
+    /* Flags() can return be any of the following combined by |
        or be omitted if none apply
        NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY */
     virtual unsigned int Flags()
@@ -111,8 +112,8 @@ namespace OpenBabel
     ////////////////////////////////////////////////////
     /// Declarations for the "API" interface functions. Definitions are below
     virtual bool ReadMolecule(OBBase* pOb, OBConversion* pConv);
+    OBBase* ReadObject(OBConversion* pConv);
     //  virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
-
   private:
     struct cdBond
     {
@@ -127,23 +128,22 @@ namespace OpenBabel
     };
 
 
-    int readFragment(istream *ifs, UINT32 fragmentId, 
+    int readFragment(istream *ifs, UINT32 fragmentId,
                      OBMol *pmol, map<UINT32, int> &atoms, list<cdBond> &bonds);
-    int readNode(istream *ifs, UINT32 nodeId, OBMol *pmol, 
-                 map<UINT32, int> &atoms, list<cdBond> &bonds, 
+    int readNode(istream *ifs, UINT32 nodeId, OBMol *pmol,
+                 map<UINT32, int> &atoms, list<cdBond> &bonds,
                  UINT32 fragmentID);
     int readBond(istream *ifs, UINT32 nodeId, OBMol *pmol, list<cdBond> &bonds);
     int readGeneric(istream *ifs, UINT32 objId);
     const char* getName(istream *ifs, UINT32 size);
 
-  };  
+  };
 	////////////////////////////////////////////////////
 
   //Make an instance of the format class
   ChemDrawBinaryFormat theChemDrawBinaryFormat;
 
   /////////////////////////////////////////////////////////////////
-
   bool ChemDrawBinaryFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
     OBMol* pmol = pOb->CastAndClear<OBMol>();
@@ -164,11 +164,13 @@ namespace OpenBabel
     //    OBPairData *pd;
     int iInt=0, depth=1;
 
+    if (!ifs.good() || ifs.peek() == EOF)
+      return false;
 
     pmol->BeginModify();
 
     pmol->SetTitle(pConv->GetTitle());
-	
+
     if((int)ifs.tellg() == 0)	// Beginning of file
       {
         ifs.read(buffer,kCDX_HeaderStringLen);
@@ -189,7 +191,7 @@ namespace OpenBabel
         if(tag & kCDXTag_Object)	// Object
           {
             READ_INT32 (ifs, id);
-            sprintf(errorMsg, "Object ID: %08X in root has type: %04X\n", id, tag);
+            snprintf(errorMsg, BUFF_SIZE, "Object ID: %08X in root has type: %04X\n", id, tag);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
             if(tag == kCDXObj_Fragment)
@@ -214,7 +216,7 @@ namespace OpenBabel
                               {
                                 obErrorLog.ThrowError(__FUNCTION__, "Can't assign all bonds!", obDebug);
                                 break;
-                              }	
+                              }
                             bonds.push_back(*bondsIter);
                           }
                         else
@@ -244,12 +246,12 @@ namespace OpenBabel
                 break;
               }
             else if ((tag == kCDXObj_Graphic) || (tag == kCDXObj_Text)
-                     || (tag == kCDXObj_BracketedGroup) 
-                     || (tag == kCDXObj_BracketAttachment) 
-                     || (tag == kCDXObj_CrossingBond) 
-                     || (tag == kCDXObj_ReactionStep) 
-                     || (tag == kCDXObj_Curve) 
-                     || (tag == kCDXObj_EmbeddedObject)) 
+                     || (tag == kCDXObj_BracketedGroup)
+                     || (tag == kCDXObj_BracketAttachment)
+                     || (tag == kCDXObj_CrossingBond)
+                     || (tag == kCDXObj_ReactionStep)
+                     || (tag == kCDXObj_Curve)
+                     || (tag == kCDXObj_EmbeddedObject))
               {	// Objects that can be ignored
                 readGeneric(&ifs, id);
               }
@@ -259,7 +261,7 @@ namespace OpenBabel
             else
               {
                 depth++;
-                sprintf(errorMsg, "New object in root, type %04X\n", tag);
+                snprintf(errorMsg, BUFF_SIZE, "New object in root, type %04X\n", tag);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
               }
           }
@@ -276,10 +278,10 @@ namespace OpenBabel
             READ_INT16 (ifs ,size);
             switch(tag)
               {
-              case kCDXProp_Name: 
-                pmol->SetTitle(getName(&ifs, size)); 
+              case kCDXProp_Name:
+                pmol->SetTitle(getName(&ifs, size));
                 break;
-              case kCDXProp_FontTable: 
+              case kCDXProp_FontTable:
               case kCDXProp_BoundingBox:
               case kCDXProp_Window_Position:
               case kCDXProp_Window_Size:
@@ -319,11 +321,11 @@ namespace OpenBabel
               case kCDXProp_ColorTable:
               case kCDXProp_CreationProgram:
               case kCDXProp_Group_Integral:
-                ifs.seekg(size, ios_base::cur); 
+                ifs.seekg(size, ios_base::cur);
                 break;
               default: // some unknown tag
-                ifs.seekg(size, ios_base::cur); 
-                sprintf(errorMsg, "Root Tag: %04X\tSize: %04X\n", tag, size); 
+                ifs.seekg(size, ios_base::cur);
+                snprintf(errorMsg, BUFF_SIZE, "Root Tag: %04X\tSize: %04X\n", tag, size);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
                 break;
               }
@@ -336,12 +338,210 @@ namespace OpenBabel
     return true;
   }
 
+  OBBase* ChemDrawBinaryFormat::ReadObject(OBConversion* pConv)
+  {
+    istream& ifs = *pConv->GetInStream();
+    char buffer[BUFF_SIZE];
+    char errorMsg[BUFF_SIZE];
+    UINT16 tag;
+    UINT16 size;
+    UINT32 id;
+    map<UINT32, int> atoms;
+    list<cdBond> bonds;
+    list<cdBond>::const_iterator bondsIter;
+    cdBond prevBond;
+    cdBond oneBond;
+    //    OBPairData *pd;
+    int iInt=0, depth=1;
+
+    if((int)ifs.tellg() == 0)	// Beginning of file
+      {
+        ifs.read(buffer,kCDX_HeaderStringLen);
+        if(strncmp(buffer, kCDX_HeaderString, kCDX_HeaderStringLen) == 0)  // File header
+          {
+            ifs.seekg (kCDX_HeaderLength - kCDX_HeaderStringLen, ios_base::cur);	// Discard rest of header.
+          }
+        else
+          {
+            cout << "Invalid file, no ChemDraw Header" << endl;	// No header, error.
+            ifs.seekg(0, ios_base::end);
+            return (OBBase*)NULL;
+          }
+      }
+    while(ifs.good())
+      {
+        READ_INT16 (ifs, tag);
+        if(tag & kCDXTag_Object)	// Object
+          {
+            READ_INT32 (ifs, id);
+            snprintf(errorMsg, BUFF_SIZE, "Object ID: %08X in root has type: %04X\n", id, tag);
+            obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
+
+            if(tag == kCDXObj_Fragment)
+              {
+                OBMol *pmol = new OBMol();
+                if(pmol==NULL)
+                  {
+                  	return NULL;
+                  }
+
+                pmol->BeginModify();
+
+  	            pmol->SetTitle(pConv->GetTitle());
+                if(readFragment(&ifs, id, pmol, atoms, bonds) != 0)
+                  {
+                    obErrorLog.ThrowError(__FUNCTION__, "Error reading fragment", obWarning);
+                    delete pmol;
+                    return NULL;
+                  }
+                iInt = 0;
+                prevBond = cdBond(0,0,0);
+                for (bondsIter=bonds.begin(); bondsIter != bonds.end(); bondsIter++)
+                  {
+                    // printf("Bond between %08X (%d) and %08x (%d)\n", bondsIter->begin, atoms[bondsIter->begin], bondsIter->end, atoms[bondsIter->end]);
+                    if(atoms[bondsIter->begin] == -1)
+                      {
+                        //						printf("Bond starts at a non-atom\n");
+                        if(atoms[bondsIter->end] == -1)
+                          {
+                            //							printf("Bond between two non-atoms pushed back\n");
+                            if((prevBond.begin == bondsIter->begin) && (prevBond.end == bondsIter->end))
+                              {
+                                obErrorLog.ThrowError(__FUNCTION__, "Can't assign all bonds!", obDebug);
+                                break;
+                              }
+                            bonds.push_back(*bondsIter);
+                          }
+                        else
+                          atoms[bondsIter->begin] = atoms[bondsIter->end];
+                      }
+                    else if(atoms[bondsIter->end] == -1)
+                      atoms[bondsIter->end] = atoms[bondsIter->begin];
+                    else
+                      {
+                        int flags = 0;
+                        switch (bondsIter->stereo)
+                          {
+                          case kCDXBondDisplay_WedgedHashBegin:
+                          case kCDXBondDisplay_WedgeEnd:
+                            flags = OB_HASH_BOND;
+                            break;
+                          case kCDXBondDisplay_WedgedHashEnd:
+                          case kCDXBondDisplay_WedgeBegin:
+                            flags = OB_WEDGE_BOND;
+                            break;
+                          default:;
+                          }
+                        pmol->AddBond(atoms[bondsIter->begin], atoms[bondsIter->end], bondsIter->order, flags);
+                      }
+                    prevBond = *bondsIter;
+                  }
+                pmol->SetDimension(2);
+                pmol->EndModify();
+                return pmol;
+              }
+            else if (tag == kCDXObj_ReactionScheme)
+              {
+                readGeneric(&ifs, id);
+puts("found a reaction");
+                return new OBReaction();
+              }
+           else if ((tag == kCDXObj_Graphic) || (tag == kCDXObj_Text)
+                     || (tag == kCDXObj_BracketedGroup)
+                     || (tag == kCDXObj_BracketAttachment)
+                     || (tag == kCDXObj_CrossingBond)
+                     || (tag == kCDXObj_Curve)
+                     || (tag == kCDXObj_EmbeddedObject))
+              {	// Objects that can be ignored
+                readGeneric(&ifs, id);
+              }
+            else if((tag == kCDXObj_Page) || ( tag == kCDXObj_Group))
+              {	// Objects where the header can be ignored
+              }
+            else
+              {
+                depth++;
+                snprintf(errorMsg, BUFF_SIZE, "New object in root, type %04X\n", tag);
+                obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
+              }
+          }
+        else if(tag == 0)	// End of object
+          {
+            if(depth > 1)
+              {
+                obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
+                depth--;
+              }
+          }
+        else	// Property
+          {
+            READ_INT16 (ifs ,size);
+            switch(tag)
+              {
+              case kCDXProp_Name:
+puts("found name");
+                /*pmol->SetTitle(*/puts(getName(&ifs, size));//);
+                break;
+              case kCDXProp_FontTable:
+              case kCDXProp_BoundingBox:
+              case kCDXProp_Window_Position:
+              case kCDXProp_Window_Size:
+              case kCDXProp_Atom_ShowQuery:
+              case kCDXProp_Atom_ShowStereo:
+              case kCDXProp_Atom_ShowAtomNumber:
+              case kCDXProp_Bond_ShowQuery:
+              case kCDXProp_Bond_ShowStereo:
+              case kCDXProp_MacPrintInfo:
+              case kCDXProp_DrawingSpaceType:
+              case kCDXProp_Width:
+              case kCDXProp_Height:
+              case kCDXProp_PageOverlap:
+              case kCDXProp_Magnification:
+              case kCDXProp_ChainAngle:
+              case kCDXProp_BondSpacing:
+              case kCDXProp_BondSpacingAbs:
+              case kCDXProp_BondLength:
+              case kCDXProp_BoldWidth:
+              case kCDXProp_LineWidth:
+              case kCDXProp_MarginWidth:
+              case kCDXProp_HashSpacing:
+              case kCDXProp_LabelStyle:
+              case kCDXProp_CaptionStyle:
+              case kCDXProp_CaptionJustification:
+              case kCDXProp_LabelJustification:
+              case kCDXProp_FractionalWidths:
+              case kCDXProp_LabelLineHeight:
+              case kCDXProp_CaptionLineHeight:
+              case kCDXProp_Window_IsZoomed:
+              case kCDXProp_WidthPages:
+              case kCDXProp_HeightPages:
+              case kCDXProp_HeaderPosition:
+              case kCDXProp_FooterPosition:
+              case kCDXProp_PrintTrimMarks:
+              case kCDXProp_PrintMargins:
+              case kCDXProp_ColorTable:
+              case kCDXProp_CreationProgram:
+              case kCDXProp_Group_Integral:
+                ifs.seekg(size, ios_base::cur);
+                break;
+              default: // some unknown tag
+                ifs.seekg(size, ios_base::cur);
+                snprintf(errorMsg, BUFF_SIZE, "Root Tag: %04X\tSize: %04X\n", tag, size);
+                obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
+                break;
+              }
+          }
+      }
+
+    return NULL;
+  }
+
   const char* ChemDrawBinaryFormat::getName(istream *ifs, UINT32 size)
   {
     char *buff;
     UINT16 temp;
     UINT32 skipsize;
-		
+
     //	buff = new char[size];
     READ_INT16 ((*ifs), temp);
     if(temp == 0)
@@ -369,7 +569,7 @@ namespace OpenBabel
 
     READ_INT32 (*ifs, y);
     READ_INT32 (*ifs, x);
-    
+
     // 	printf("2D coordinates - x: %d, y: %d\n", x, y);
     return 0;
   }
@@ -377,10 +577,10 @@ namespace OpenBabel
   UINT32 getBondStart(istream *ifs, UINT32 size)
   {
     UINT32 atomID;
-	
+
     if(size != 4)
       return -1;
-	
+
     READ_INT32 (*ifs, atomID);
     //	printf("Bond starts at atom nr: %08X\n", atomID);
     return atomID;
@@ -389,10 +589,10 @@ namespace OpenBabel
   UINT32 getBondEnd(istream *ifs, UINT32 size)
   {
     UINT32 atomID;
-	
+
     if(size != 4)
       return -1;
-	
+
     READ_INT32 (*ifs, atomID);
     //	printf("Bond ends at atom nr: %08X\n", atomID);
     return atomID;
@@ -401,10 +601,10 @@ namespace OpenBabel
   int getBondOrder(istream *ifs, UINT32 size)
   {
     UINT16 order;
-	
+
     if(size != 2)
       return -1;
-	
+
     READ_INT16 (*ifs, order);
     //	printf("Bond order is: %d\n", order);
     return (int) order;
@@ -413,10 +613,10 @@ namespace OpenBabel
   int getBondDisplay(istream *ifs, UINT32 size)
   {
     UINT16 stereo;
-	
+
     if(size != 2)
       return -1;
-	
+
     READ_INT16 (*ifs, stereo);
     //    printf("Bond stereo is: %d\n", stereo);
     return (int) stereo;
@@ -425,10 +625,10 @@ namespace OpenBabel
   int getNodeType(istream *ifs, UINT32 size)
   {
     UINT16 nodeType;
-	
+
     if(size != 2)
       return -1;
-	
+
     READ_INT16 (*ifs, nodeType);
     //	printf("Node type is: %d\n", nodeType);
     return (int) nodeType;
@@ -437,10 +637,10 @@ namespace OpenBabel
   int getElement(istream *ifs, UINT32 size, OBAtom &atom)
   {
     UINT16 element;
-	
+
     if(size != 2)
       return -1;
-	
+
     READ_INT16 (*ifs, element);
     atom.SetAtomicNum(element);
     //	printf("Atomic number: %d\n", element);
@@ -450,10 +650,10 @@ namespace OpenBabel
   int getIsotope(istream *ifs, UINT32 size, OBAtom &atom)
   {
     UINT16 isotope;
-	
+
     if(size != 2)
       return -1;
-	
+
     READ_INT16 (*ifs, isotope);
     atom.SetIsotope(isotope);
     return 0;
@@ -467,7 +667,7 @@ namespace OpenBabel
 #if __BYTE_ORDER == __BIG_ENDIAN
     radical = radical >> 24;
 #endif
-	
+
     //    cout << " Atomic radical " << size << " " << radical << endl;
     switch (radical)
       {
@@ -489,7 +689,7 @@ namespace OpenBabel
   int getCharge(istream *ifs, UINT32 size)
   {
     int charge;
-	
+
     if(size == 4)		// Bug in ChemDraw 8.0, see http://www.cambridgesoft.com/services/documentation/sdk/chemdraw/cdx/properties/Atom_Charge.htm
       {
         READ_INT32 (*ifs, charge);
@@ -504,7 +704,7 @@ namespace OpenBabel
         }
       else
         return 0;
-	
+
     //	printf("Charge: %d\n",charge);
     return charge;
   }
@@ -512,10 +712,10 @@ namespace OpenBabel
   int getAtomHydrogens(istream *ifs, UINT32 size)
   {
     UINT16 hydrogens;
-	
+
     if(size != 2)
       return -1;
-	
+
     READ_INT16 (*ifs, hydrogens);
     //	printf("Number of explicit hydrogens: %d\n", hydrogens);
     return 0;
@@ -537,7 +737,7 @@ namespace OpenBabel
             depth++;
             READ_INT32 (*ifs, id);
             //			printf("Object ID (in text-object %08X): %08X has type: %04X\n", textId, id, tag);
-            sprintf(errorMsg, "New object in text, type %04X\n", tag);
+            snprintf(errorMsg, BUFF_SIZE, "New object in text, type %04X\n", tag);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
           }
         else if(tag == 0)	// End of object
@@ -549,10 +749,12 @@ namespace OpenBabel
           {
             READ_INT16 (*ifs, size);
             //			printf("Tag: %04X\tSize: %04X\n", tag, size);
-            switch(tag)
-              {
-              default: ifs->seekg(size, ios_base::cur); break;
-              }
+            //switch(tag)
+            //  {
+              //default:
+              ifs->seekg(size, ios_base::cur);
+              //break;
+            //  }
           }
         if(depth < 1)
           return 0;
@@ -560,7 +762,7 @@ namespace OpenBabel
     return -1;
   }
 
-		
+
   int ChemDrawBinaryFormat::readNode(istream *ifs, UINT32 nodeId,
                                      OBMol *pmol, map<UINT32, int> &atoms,
                                      list<cdBond> &bonds, UINT32 fragmentID)
@@ -579,16 +781,16 @@ namespace OpenBabel
 
     atom.SetAtomicNum(6);
     //	data->SetAttribute("nodeId");
-    //	sprintf(strNodeId, "%d", nodeId);
+    //	snprintf(strNodeId, 20, "%d", nodeId);
     //	data->SetValue(strNodeId);
-    //	atom.SetData(data);	
+    //	atom.SetData(data);
     //	if(atom.HasData("nodeId"))
     //	{
     //		printf("Adding data to atom - attr: %s\tvalue: %s\n", dynamic_cast<OBPairData *> (atom.GetData("nodeId"))->GetAttribute().c_str(), dynamic_cast<OBPairData *> (atom.GetData("nodeId"))->GetValue().c_str());
     //	}
     //	else
     //		printf("No data added\n");
-		
+
     while(ifs->good())
       {
         READ_INT16 (*ifs, tag);
@@ -597,7 +799,7 @@ namespace OpenBabel
             depth++;
             READ_INT32 (*ifs, id);
 
-            sprintf(errorMsg, "Object ID (in node %08X): %08X has type: %04X\n", nodeId, id, tag);
+            snprintf(errorMsg, BUFF_SIZE, "Object ID (in node %08X): %08X has type: %04X\n", nodeId, id, tag);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
             if(tag == kCDXObj_Fragment)
@@ -622,12 +824,12 @@ namespace OpenBabel
               }
             else if(tag == kCDXObj_Group)
               {	// Objects where we can ignore the header
-              }			
+              }
             else
               {
-                sprintf(errorMsg, "New object in node, type %04X\n", tag);
+                snprintf(errorMsg, BUFF_SIZE, "New object in node, type %04X\n", tag);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
-              }			
+              }
           }
         else if(tag == 0)	// End of object
           {
@@ -637,26 +839,26 @@ namespace OpenBabel
         else	// Property
           {
             READ_INT16 (*ifs, size);
-            sprintf(errorMsg, "Node Tag: %04X\tSize: %04X\n", tag, size);
+            snprintf(errorMsg, BUFF_SIZE, "Node Tag: %04X\tSize: %04X\n", tag, size);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
             switch(tag)
               {
               case kCDXProp_Atom_NumHydrogens:
-                getAtomHydrogens(ifs, size); 
+                getAtomHydrogens(ifs, size);
                 break;
               case kCDXProp_2DPosition:
-                get2DPosition(ifs, size, x, y); 
+                get2DPosition(ifs, size, x, y);
                 atom.SetVector((double) x / 500000., (double) y / -500000.0, (double) 0.0);
                 break;
               case kCDXProp_Node_Element:
-                getElement(ifs, size, atom); 
+                getElement(ifs, size, atom);
                 break;
               case kCDXProp_Atom_Charge:
-                atom.SetFormalCharge(getCharge(ifs, size)); 
+                atom.SetFormalCharge(getCharge(ifs, size));
                 break;
               case kCDXProp_Node_Type:
-                nodeType = getNodeType(ifs, size); 
+                nodeType = getNodeType(ifs, size);
                 break;
               case kCDXProp_Atom_Isotope:
                 getIsotope(ifs, size, atom);
@@ -669,7 +871,7 @@ namespace OpenBabel
               case kCDXProp_Name:
               case kCDXProp_IgnoreWarnings:
               case kCDXProp_ChemicalWarning:
-                ifs->seekg(size, ios_base::cur); 
+                ifs->seekg(size, ios_base::cur);
                 break;  // Maybe use? (any of the above properties)
               case kCDXProp_MarginWidth:
               case kCDXProp_ZOrder:
@@ -681,11 +883,11 @@ namespace OpenBabel
               case kCDXProp_LabelStyle:
               case kCDXProp_ForegroundColor:
               case kCDXProp_BackgroundColor:
-                ifs->seekg(size, ios_base::cur); 
+                ifs->seekg(size, ios_base::cur);
                 break;
               default: // some unknown node tag
-                ifs->seekg(size, ios_base::cur); 
-                sprintf(errorMsg, "Node Tag: %04X\tSize: %04X\n", tag, size); 
+                ifs->seekg(size, ios_base::cur);
+                snprintf(errorMsg, BUFF_SIZE, "Node Tag: %04X\tSize: %04X\n", tag, size);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
                 break;
               }
@@ -696,17 +898,23 @@ namespace OpenBabel
             switch(nodeType)
               {
               case kCDXNodeType_Fragment:
-                atoms[nodeId] = -1; 
+                atoms[nodeId] = -1;
                 break;
               case kCDXNodeType_ExternalConnectionPoint:
                 atoms[nodeId] = -1;
-                bonds.push_back(cdBond(nodeId,fragmentID,1)); 
+                bonds.push_back(cdBond(nodeId,fragmentID,1));
                 break;
               default:
                 atom2 = pmol->NewAtom();
                 atom2->SetVector(atom.GetVector());
                 atom2->SetFormalCharge(atom.GetFormalCharge());
                 atom2->SetAtomicNum(atom.GetAtomicNum());
+                /*
+                if (atom.IsClockwise())
+                  atom2->SetClockwiseStereo();
+                else if (atom.IsAntiClockwise())
+                  atom2->SetAntiClockwiseStereo();
+                */
                 atoms[nodeId] = atom2->GetIdx();
                 break;
               }
@@ -716,7 +924,7 @@ namespace OpenBabel
     return -1; // file ended early
   }
 
-  int ChemDrawBinaryFormat::readBond(istream *ifs, UINT32 bondId, 
+  int ChemDrawBinaryFormat::readBond(istream *ifs, UINT32 bondId,
                                      OBMol *pmol, list<cdBond> &bonds)
   {
     char errorMsg[BUFF_SIZE];
@@ -732,7 +940,7 @@ namespace OpenBabel
           {
             depth++;
             READ_INT32 (*ifs, id);
-            sprintf(errorMsg, "Object ID (in bond %08X): %08X has type: %04X\n", bondId, id, tag);
+            snprintf(errorMsg, BUFF_SIZE, "Object ID (in bond %08X): %08X has type: %04X\n", bondId, id, tag);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
             if(tag == kCDXObj_Text)
@@ -742,7 +950,7 @@ namespace OpenBabel
               }
             else
               {
-                sprintf(errorMsg, "New object in bond, type %04X\n", tag);
+                snprintf(errorMsg, BUFF_SIZE, "New object in bond, type %04X\n", tag);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
               }
           }
@@ -755,22 +963,39 @@ namespace OpenBabel
           {
             READ_INT16 (*ifs, size);
 
-            sprintf(errorMsg, "Bond Tag: %04X\tSize: %04X\n", tag, size);
+            snprintf(errorMsg, BUFF_SIZE, "Bond Tag: %04X\tSize: %04X\n", tag, size);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
             switch(tag)
               {
               case kCDXProp_Bond_Begin:
-                bgnID = getBondStart(ifs, size); 
+                bgnID = getBondStart(ifs, size);
                 break;
               case kCDXProp_Bond_End:
-                endID = getBondEnd(ifs, size); 
+                endID = getBondEnd(ifs, size);
                 break;
               case kCDXProp_Bond_Order:
-                order = getBondOrder(ifs, size); 
+                order = getBondOrder(ifs, size);
+                switch (order)
+                  {
+                  case 0xFFFF: // undefined, keep 1 for now
+                    order = 1;
+                  case 0x0001:
+                  case 0x0002:
+                    break;
+                  case 0x0004:
+                    order = 3;
+                    break;
+                  case 0x0080: // aromatic bond
+                    order = 5;
+                    break;
+                  default: // other cases are just not supported, keep 1
+                    order = 1;
+                    break;
+                  }
                 break;
               case kCDXProp_Bond_Display:
-                stereo = getBondDisplay(ifs, size); 
+                stereo = getBondDisplay(ifs, size);
                 break; // Stereo info
 
               case kCDXProp_BondLength:
@@ -778,7 +1003,7 @@ namespace OpenBabel
               case kCDXProp_Bond_BeginAttach:
               case kCDXProp_Bond_EndAttach:
               case kCDXProp_ChemicalWarning:
-                ifs->seekg(size, ios_base::cur); 
+                ifs->seekg(size, ios_base::cur);
                 break; // Maybe use? (any of the above cases)
 
               case kCDXProp_IgnoreWarnings:
@@ -794,12 +1019,12 @@ namespace OpenBabel
               case kCDXProp_ForegroundColor:
               case kCDXProp_BackgroundColor:
               case kCDXProp_LabelStyle:
-                ifs->seekg(size, ios_base::cur); 
+                ifs->seekg(size, ios_base::cur);
                 break;
 
               default: // some unknown, undocumented property
                 ifs->seekg(size, ios_base::cur);
-                sprintf(errorMsg, "Bond Tag: %04X\tSize: %04X\n", tag, size);
+                snprintf(errorMsg, BUFF_SIZE, "Bond Tag: %04X\tSize: %04X\n", tag, size);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
                 break;
               }
@@ -829,12 +1054,12 @@ namespace OpenBabel
             depth++;
             READ_INT32 (*ifs, id);
 
-            sprintf(errorMsg, "Object ID (in generic %08X): %08X has type: %04X\n", objId, id, tag);
+            snprintf(errorMsg, BUFF_SIZE, "Object ID (in generic %08X): %08X has type: %04X\n", objId, id, tag);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
-            if ((tag == kCDXObj_BracketAttachment) 
-                || (tag == kCDXObj_CrossingBond) 
-                || (tag == kCDXObj_BracketedGroup) 
+            if ((tag == kCDXObj_BracketAttachment)
+                || (tag == kCDXObj_CrossingBond)
+                || (tag == kCDXObj_BracketedGroup)
                 || (tag == kCDXObj_Text)
                 || (tag == kCDXObj_Fragment))
               {	// Objects to ignore (Fragment might be worth parsing)
@@ -842,28 +1067,28 @@ namespace OpenBabel
                 depth--;
               }
             else {
-              sprintf(errorMsg, "New object in generic, type %04X\n", tag);
+              snprintf(errorMsg, BUFF_SIZE, "New object in generic, type %04X\n", tag);
               obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
             }
           }
         else if(tag == 0)	// End of object
           {
             depth--;
-            sprintf(errorMsg, "End of Object in generic %08X\n", objId);
+            snprintf(errorMsg, BUFF_SIZE, "End of Object in generic %08X\n", objId);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
           }
         else	// Property
           {
             READ_INT16 (*ifs, size);
 
-            sprintf(errorMsg, "Generic Tag: %04X\tSize: %04X\n", tag, size);
+            snprintf(errorMsg, BUFF_SIZE, "Generic Tag: %04X\tSize: %04X\n", tag, size);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
-            switch(tag)
-              {
-              default:
-                ifs->seekg(size, ios_base::cur); 
-                break;
-              }
+            //switch(tag)
+            //  {
+            //  default:
+                ifs->seekg(size, ios_base::cur);
+            //   break;
+            //  }
           }
         if(depth < 1) // matched begin and end tags, so we're good
           {
@@ -881,10 +1106,11 @@ namespace OpenBabel
     UINT16 tag;
     UINT16 size;
     UINT32 id;
-    // char u32[4];
+    //char u32[4];
     int depth = 1;
 
-    atoms[fragmentId] = -1;
+ cerr<<"Reading "<<pmol<<endl;
+   atoms[fragmentId] = -1;
     while(ifs->good())
       {
         READ_INT16 ((*ifs), tag);
@@ -893,7 +1119,7 @@ namespace OpenBabel
             depth++;
             READ_INT32 (*ifs, id);
 
-            sprintf(errorMsg, "Object ID (in fragment %08X): %08X has type: %04X\n", fragmentId, id, tag);
+            snprintf(errorMsg, BUFF_SIZE, "Object ID (in fragment %08X): %08X has type: %04X\n", fragmentId, id, tag);
             obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
             if(tag == kCDXObj_Fragment)
@@ -924,7 +1150,7 @@ namespace OpenBabel
             //			}
             else
               {
-                sprintf(errorMsg, "New object in fragment, type %04X\n", tag);
+                snprintf(errorMsg, BUFF_SIZE, "New object in fragment, type %04X\n", tag);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
               }
 
@@ -939,14 +1165,14 @@ namespace OpenBabel
             switch(tag)
               {
               case kCDXProp_Frag_ConnectionOrder:
-                ifs->seekg(size, ios_base::cur); 
+                ifs->seekg(size, ios_base::cur);
                 break;  // Should use
               case kCDXProp_BoundingBox:
-                ifs->seekg(size, ios_base::cur); 
+                ifs->seekg(size, ios_base::cur);
                 break;
               default:
-                ifs->seekg(size, ios_base::cur); 
-                sprintf(errorMsg, "Fragment Tag: %04X\tSize: %04X\n", tag, size);
+                ifs->seekg(size, ios_base::cur);
+                snprintf(errorMsg, BUFF_SIZE, "Fragment Tag: %04X\tSize: %04X\n", tag, size);
                 obErrorLog.ThrowError(__FUNCTION__, errorMsg, obDebug);
 
                 break;
@@ -954,7 +1180,8 @@ namespace OpenBabel
           }
         if(depth < 1)
           {
-            return 0; // all begin and end tags matched -- everything good 
+cerr<<"Done reading "<<pmol<<endl;
+            return 0; // all begin and end tags matched -- everything good
           }
       } // while reading
     return -1; // file ended before reading was finished
