@@ -4,12 +4,12 @@ cifformat.cpp - Implementation of subclass of OBFormat for conversion of OBMol.
 Copyright (C) 2006 Vincent Favre-Nicolin
 
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,6 +26,16 @@ GNU General Public License for more details.
 #include <map>
 #include <set>
 
+#ifdef _MSC_VER
+ #pragma warning( disable : 4503 )
+ // The decorated name was longer than the compiler limit (4096), and was truncated.
+ // This is due to the use of templates specialized on templates repeatedly.
+ // The correctness of the program, however, is unaffected by the truncated name,
+ // but if you get link time errors on a truncated symbol, it will be more difficult
+ // to determine the type of the symbol in the error. Debugging will also be more difficult;
+ // the debugger will also have difficultly mapping symbol name to type name.
+#endif
+
 using namespace std;
 namespace OpenBabel
 {
@@ -41,8 +51,9 @@ namespace OpenBabel
     virtual const char* Description() //required
     {
       return
-        "Crystallographic Information File\n\n"
-        "Read Options e.g. -b:\n"
+        "Crystallographic Information File\n"
+        "The CIF file format is the standard interchange format for small-molecule chemical structures\n\n"
+        "Read Options e.g. -ab:\n"
         "  v  Verbose CIF conversion\n"
         "  s  Output single bonds only\n"
         "  b  Disable bonding entirely\n"
@@ -59,7 +70,7 @@ namespace OpenBabel
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
   };
   //############################## Case-insensituve string####################################################
-  // :TODO: This duplicates normal case-insensitive string comparison in OpenBabel
+  // :@todo: This duplicates normal case-insensitive string comparison in OpenBabel
   /// Case-insensitive string class
   /// From: Guru of the Week #29
   /// e.g.: http://gcc.gnu.org/onlinedocs/libstdc++/21_strings/gotw29a.txt
@@ -68,16 +79,16 @@ namespace OpenBabel
   struct ci_char_traits : public std::char_traits<char>
   {
     static bool eq( char c1, char c2 );
-   
+
     static bool ne( char c1, char c2 );
-   
+
     static bool lt( char c1, char c2 );
-   
+
     static int compare(const char* s1,const char* s2,size_t n );
-   
+
     static const char* find( const char* s, int n, char a );
   };
-   
+
   typedef std::basic_string<char, ci_char_traits> ci_string;
   int strnicmp(const char *s1, const char *s2, int len)
   {
@@ -98,19 +109,19 @@ namespace OpenBabel
       }
     return 0;
   }
-   
+
   bool ci_char_traits::eq( char c1, char c2 )
   {return tolower(c1) == tolower(c2);}
-   
+
   bool ci_char_traits::ne( char c1, char c2 )
   {return tolower(c1) != tolower(c2);}
-   
+
   bool ci_char_traits::lt( char c1, char c2 )
   {return tolower(c1) < tolower(c2);}
-   
+
   int ci_char_traits::compare(const char* s1,const char* s2,size_t n )
   {return strnicmp( s1, s2, n );}
-   
+
   const char* ci_char_traits::find( const char* s, int n, char a )
   {
     while( n-- > 0 && tolower(*s) != tolower(a) ) ++s;
@@ -118,7 +129,7 @@ namespace OpenBabel
   }
   //############################## CIF CLASSES headers####################################################
   /** The CIFData class holds all the information from a \e single data_ block from a cif file.
-   * 
+   *
    * It is a placeholder for all comments, item and loop data, as raw strings copied from
    * a cif file.
    *
@@ -137,14 +148,14 @@ namespace OpenBabel
    *
    * Cartesian coordinates are stored in Angstroems, angles in radians.
    *
-   * If another data field is needed, it is possible to directly access the string data 
+   * If another data field is needed, it is possible to directly access the string data
    * (CIFData::mvComment , CIFData::mvItem and CIFData::mvLoop) to search for the correct tags.
    */
   class CIFData
   {
   public:
     CIFData();
-         
+
     /// Extract lattice parameters, spacegroup (symbol or number), atomic positions,
     /// chemical name and formula if available.
     /// All other data is ignored
@@ -194,7 +205,7 @@ namespace OpenBabel
     std::string mName;
     /// Formula. Or empty string if none is available.
     std::string mFormula;
-    /// Atom record 
+    /// Atom record
     struct CIFAtom
     {
       CIFAtom();
@@ -205,7 +216,7 @@ namespace OpenBabel
       /// Fractionnal coordinates (_atom_site_fract_{x,y,z}) or empty vector.
       std::vector<float> mCoordFrac;
       /// Cartesian coordinates in Angstroem (_atom_site_Cartn_{x,y,z}) or empty vector.
-      /// Transformation to fractionnal coordinates currently assumes 
+      /// Transformation to fractionnal coordinates currently assumes
       /// "a parallel to x; b in the plane of y and z" (see _atom_sites_Cartn_transform_axes)
       std::vector<float> mCoordCart;
       /// Site occupancy, or -1
@@ -258,22 +269,22 @@ namespace OpenBabel
 
   //############################## CIF CLASSES CODE ####################################################
   CIFData::CIFAtom::CIFAtom():
-    mLabel(""),mSymbol(""),mOccupancy(1.0)
+    mLabel(""),mSymbol(""),mOccupancy(1.0f)
   {}
-   
+
   CIFData::CIFData()
   {}
-   
+
   void CIFData::ExtractAll(const bool verbose)
   {
-    // :TODO: Take care of values listed as "." and "?" instead of a real value.
+    // :@todo: Take care of values listed as "." and "?" instead of a real value.
     this->ExtractName(verbose);
     this->ExtractUnitCell(verbose);
     this->ExtractSpacegroup(verbose);
     this->ExtractAtomicPositions(verbose);
     this->ExtractBonds(verbose);
   }
-   
+
   void CIFData::ExtractUnitCell(const bool verbose)
   {
     map<ci_string,string>::const_iterator positem;
@@ -299,13 +310,13 @@ namespace OpenBabel
           mvLatticePar[5]=CIFNumeric2Float(positem->second);
         if(verbose) cout<<"Found Lattice parameters:" <<mvLatticePar[0]<<" , "<<mvLatticePar[1]<<" , "<<mvLatticePar[2]
                         <<" , "<<mvLatticePar[3]<<" , "<<mvLatticePar[4]<<" , "<<mvLatticePar[5]<<endl;
-        mvLatticePar[3]*=DEG_TO_RAD;// pi/180
-        mvLatticePar[4]*=DEG_TO_RAD;
-        mvLatticePar[5]*=DEG_TO_RAD;
+        mvLatticePar[3] = static_cast<float> (mvLatticePar[3] * DEG_TO_RAD);// pi/180
+		mvLatticePar[4] = static_cast<float> (mvLatticePar[4] * DEG_TO_RAD);
+		mvLatticePar[5] = static_cast<float> (mvLatticePar[5] * DEG_TO_RAD);
         this->CalcMatrices();
       }
   }
-   
+
   void CIFData::ExtractSpacegroup(const bool verbose)
   {
     map<ci_string,string>::const_iterator positem;
@@ -326,10 +337,19 @@ namespace OpenBabel
             found = true;
             if(verbose) cout<<"Found spacegroup IT number (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupNumberIT<<endl;
           }
-        else
-          mSpacegroupNumberIT=0;
+        else {
+          positem=mvItem.find("_symmetry_group_IT_number");
+          if(positem!=mvItem.end())
+          {
+            mSpacegroupNumberIT=CIFNumeric2Int(positem->second);
+            found = true;
+            if(verbose) cout<<"Found spacegroup IT number (with NON-STANDARD CIF TAG):"<<mSpacegroupNumberIT<<endl;
+          }
+          else
+            mSpacegroupNumberIT=0;
+        }
       }
-      
+
     positem=mvItem.find("_space_group_name_Hall");
     if(positem!=mvItem.end())
       {
@@ -347,7 +367,7 @@ namespace OpenBabel
             if(verbose) cout<<"Found spacegroup Hall symbol (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupSymbolHall<<endl;
           }
       }
-      
+
     positem=mvItem.find("_space_group_name_H-M_alt");
     if(positem!=mvItem.end())
       {
@@ -365,50 +385,50 @@ namespace OpenBabel
             if(verbose) cout<<"Found spacegroup Hermann-Mauguin symbol (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupHermannMauguin<<endl;
           }
       }
-    if (mSpacegroupSymbolHall.length() > 0)
-      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupSymbolHall);
-    else
-      mSpaceGroup=NULL;
-    if (!mSpaceGroup && mSpacegroupHermannMauguin.length() > 0)
-      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupHermannMauguin);
-    if (!mSpaceGroup)
+    mSpaceGroup=NULL;
+    if (mSpacegroupNumberIT != 0) {
       mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupNumberIT);
-    SpaceGroup *sg = new SpaceGroup();
-    if (mSpacegroupSymbolHall.length() > 0)
-      sg->SetHallName(mSpacegroupSymbolHall);
-    if (mSpacegroupHermannMauguin.length() > 0)
-      sg->SetHMName(mSpacegroupHermannMauguin);
-    if (mSpacegroupNumberIT > 0)
-      sg->SetId(mSpacegroupNumberIT);
-    positem=mvItem.find("_symmetry_equiv_pos_as_xyz");
-    if(positem!=mvItem.end())
-      {
-        sg->AddTransform (positem->second);
-        found = true;
-      }
-    else for(map<set<ci_string>,map<ci_string,vector<string> > >::const_iterator loop=mvLoop.begin();
-      loop!=mvLoop.end();++loop)
-      {
-        map<ci_string,vector<string> >::const_iterator pos;
-        unsigned i, nb;
-        pos=loop->second.find("_symmetry_equiv_pos_as_xyz");
-        if (pos!=loop->second.end())
-          {
-            nb=pos->second.size();
-            found = true;
-            for (i = 0; i < nb; i++)
-              sg->AddTransform(pos->second[i]);
-            break; // found the transforms, so we have done with them
-          }
     }
-    if (found)
-      mSpaceGroup = SpaceGroup::Find(sg);
-    delete sg;
+    else if (mSpacegroupSymbolHall.length() > 0) {
+      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupSymbolHall);
+    }
+    else if (mSpacegroupHermannMauguin.length() > 0) {
+      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupHermannMauguin);
+    }
+    else {
+      SpaceGroup *sg = new SpaceGroup();
+      positem=mvItem.find("_symmetry_equiv_pos_as_xyz");
+      if(positem!=mvItem.end())
+        {
+          sg->AddTransform (positem->second);
+          found = true;
+        }
+      else {
+        for(map<set<ci_string>,map<ci_string,vector<string> > >::const_iterator loop=mvLoop.begin();
+            loop!=mvLoop.end();++loop)
+          {
+            map<ci_string,vector<string> >::const_iterator pos;
+            unsigned i, nb;
+            pos=loop->second.find("_symmetry_equiv_pos_as_xyz");
+            if (pos!=loop->second.end())
+              {
+                nb=pos->second.size();
+                found = true;
+                for (i = 0; i < nb; i++)
+                  sg->AddTransform(pos->second[i]);
+                break; // found the transforms, so we have done with them
+              }
+          }
+        if (found)
+          mSpaceGroup = SpaceGroup::Find(sg);
+        delete sg;
+      }
+    }
     if (mSpaceGroup != NULL)
       // set the space group name to Hall symbol
       mSpacegroupSymbolHall = mSpaceGroup->GetHallName();
   }
-   
+
   void CIFData::ExtractName(const bool verbose)
   {
     map<ci_string,string>::const_iterator positem;
@@ -480,7 +500,7 @@ namespace OpenBabel
           }
       }
   }
-   
+
   void CIFData::ExtractAtomicPositions(const bool verbose)
   {
     map<ci_string,string>::const_iterator positem;
@@ -609,7 +629,7 @@ namespace OpenBabel
 
   void CIFData::CalcMatrices(const bool verbose)
   {
-    if(mvLatticePar.size()==0) return;//:TODO: throw error
+    if(mvLatticePar.size()==0) return;//:@todo: throw error
     float a,b,c,alpha,beta,gamma;//direct space parameters
     float aa,bb,cc,alphaa,betaa,gammaa;//reciprocal space parameters
     float v;//volume of the unit cell
@@ -619,40 +639,40 @@ namespace OpenBabel
     alpha=mvLatticePar[3];
     beta=mvLatticePar[4];
     gamma=mvLatticePar[5];
-      
+
     v=sqrt(1-cos(alpha)*cos(alpha)-cos(beta)*cos(beta)-cos(gamma)*cos(gamma)
            +2*cos(alpha)*cos(beta)*cos(gamma));
-      
+
     aa=sin(alpha)/a/v;
     bb=sin(beta )/b/v;
     cc=sin(gamma)/c/v;
-      
+
     alphaa=acos( (cos(beta )*cos(gamma)-cos(alpha))/sin(beta )/sin(gamma) );
     betaa =acos( (cos(alpha)*cos(gamma)-cos(beta ))/sin(alpha)/sin(gamma) );
     gammaa=acos( (cos(alpha)*cos(beta )-cos(gamma))/sin(alpha)/sin(beta ) );
-      
+
     mOrthMatrix[0][0]=a;
     mOrthMatrix[0][1]=b*cos(gamma);
     mOrthMatrix[0][2]=c*cos(beta);
-      
+
     mOrthMatrix[1][0]=0;
     mOrthMatrix[1][1]=b*sin(gamma);
     mOrthMatrix[1][2]=-c*sin(beta)*cos(alphaa);
-      
+
     mOrthMatrix[2][0]=0;
     mOrthMatrix[2][1]=0;
     mOrthMatrix[2][2]=1/cc;
-      
+
     // Invert upper triangular matrix
     float cm[3][3];
     cm[0][0]=mOrthMatrix[0][0];
     cm[0][1]=mOrthMatrix[0][1];
     cm[0][2]=mOrthMatrix[0][2];
-      
+
     cm[1][0]=mOrthMatrix[1][0];
     cm[1][1]=mOrthMatrix[1][1];
     cm[1][2]=mOrthMatrix[1][2];
-      
+
     cm[2][0]=mOrthMatrix[2][0];
     cm[2][1]=mOrthMatrix[2][1];
     cm[2][2]=mOrthMatrix[2][2];
@@ -690,7 +710,7 @@ namespace OpenBabel
              <<mOrthMatrixInvert[2][0]<<" "<<mOrthMatrixInvert[2][1]<<" "<<mOrthMatrixInvert[2][2]<<endl<<endl;
       }
   }
-   
+
   void CIFData::f2c(float &x,float &y, float &z)
   {
     const float x0=x,y0=y,z0=z;
@@ -698,7 +718,7 @@ namespace OpenBabel
     y=mOrthMatrix[1][0]*x0+mOrthMatrix[1][1]*y0+mOrthMatrix[1][2]*z0;
     z=mOrthMatrix[2][0]*x0+mOrthMatrix[2][1]*y0+mOrthMatrix[2][2]*z0;
   }
-   
+
   void CIFData::c2f(float &x,float &y, float &z)
   {
     const float x0=x,y0=y,z0=z;
@@ -706,10 +726,10 @@ namespace OpenBabel
     y=mOrthMatrixInvert[1][0]*x0+mOrthMatrixInvert[1][1]*y0+mOrthMatrixInvert[1][2]*z0;
     z=mOrthMatrixInvert[2][0]*x0+mOrthMatrixInvert[2][1]*y0+mOrthMatrixInvert[2][2]*z0;
   }
-   
+
   void CIFData::Cartesian2FractionalCoord()
   {
-    if(mvLatticePar.size()==0) return;//:TODO: report error
+    if(mvLatticePar.size()==0) return;//:@todo: report error
     for(vector<CIFAtom>::iterator pos=mvAtom.begin();pos!=mvAtom.end();++pos)
       {
         pos->mCoordFrac.resize(3);
@@ -719,10 +739,10 @@ namespace OpenBabel
         c2f(pos->mCoordFrac[0],pos->mCoordFrac[1],pos->mCoordFrac[2]);
       }
   }
-   
+
   void CIFData::Fractional2CartesianCoord()
   {
-    if(mvLatticePar.size()==0) return;//:TODO: report error
+    if(mvLatticePar.size()==0) return;//:@todo: report error
     for(vector<CIFAtom>::iterator pos=mvAtom.begin();pos!=mvAtom.end();++pos)
       {
         pos->mCoordCart.resize(3);
@@ -732,10 +752,10 @@ namespace OpenBabel
         f2c(pos->mCoordCart[0],pos->mCoordCart[1],pos->mCoordCart[2]);
       }
   }
-   
-  ///// 
-   
-   
+
+  /////
+
+
   CIF::CIF(istream &is, const bool interpret,const bool verbose)
   {
     //Copy to an iostream so that we can put back characters if necessary
@@ -748,9 +768,9 @@ namespace OpenBabel
       for(map<string,CIFData>::iterator posd=mvData.begin();posd!=mvData.end();++posd)
         posd->second.ExtractAll(verbose);
   }
-   
+
   bool iseol(const char c) { return ((c=='\n')||(c=='\r'));}
-   
+
   /// Read one value, whether it is numeric, string or text
   string CIFReadValue(stringstream &in,char &lastc)
   {
@@ -817,7 +837,7 @@ namespace OpenBabel
     if(vv) cout<<"NormalValue:"<<value<<endl;
     return value;
   }
-   
+
   void CIF::Parse(stringstream &in)
   {
     bool vv=false;//very verbose ?
@@ -940,7 +960,7 @@ namespace OpenBabel
         // It is OK if this is just a blank line though
         string junk;
         getline(in,junk);
-        
+
         if(junk.size()>0)
         {
           stringstream errorMsg;
@@ -950,7 +970,7 @@ namespace OpenBabel
         }
       }
   }
-   
+
   float CIFNumeric2Float(const string &s)
   {
     if((s==".") || (s=="?")) return 0.0;
@@ -959,7 +979,7 @@ namespace OpenBabel
     if(n!=1) return 0.0;
     return v;
   }
-   
+
   int CIFNumeric2Int(const string &s)
   {
     if((s==".") || (s=="?")) return 0;
@@ -982,7 +1002,7 @@ namespace OpenBabel
   {
     if (!atom->IsOxygen())
       return false;
-     
+
     int nonHydrogenCount = 0;
     int hydrogenCount = 0;
     FOR_NBORS_OF_ATOM(neighbor, *atom) {
@@ -991,7 +1011,7 @@ namespace OpenBabel
       else
         hydrogenCount++;
     }
-    
+
     return (hydrogenCount == 2 && nonHydrogenCount <= 1);
   }
 
@@ -1000,12 +1020,12 @@ namespace OpenBabel
   {
     if (!mol)
       return;
-    
+
     // First look for NR4, PR4 ions,
     // or bare halides, alkali and alkaline earth metal ions
     FOR_ATOMS_OF_MOL(atom, *mol) {
-      
-      if ((atom->GetAtomicNum() == 7 || atom->GetAtomicNum() == 15) 
+
+      if ((atom->GetAtomicNum() == 7 || atom->GetAtomicNum() == 15)
           && atom->BOSum() == 4) {
         // check if we should make a positive charge?
         // i.e., 4 non-metal neighbors
@@ -1026,7 +1046,7 @@ namespace OpenBabel
         if (nonMetalNeighbors) // 4 non-metals, e.g. NH4+
           atom->SetFormalCharge(+1);
       }
-      
+
       // Now look for simple atomic ions like Na, Li, F, Cl, Br...
       // If we have an existing formal charge, keep going
       if (atom->GetFormalCharge() != 0)
@@ -1072,7 +1092,7 @@ namespace OpenBabel
     if (pConv->IsOption("v",OBConversion::INOPTIONS)) verbose=true;
 
     CIF cif(*pConv->GetInStream(),true,verbose);
-    // Loop on all data blocks until we find one structure :TODO: handle multiple structures
+    // Loop on all data blocks until we find one structure :@todo: handle multiple structures
     for(map<string,CIFData>::iterator pos=cif.mvData.begin();pos!=cif.mvData.end();++pos)
       if(pos->second.mvAtom.size()>0)
         {
@@ -1103,7 +1123,7 @@ namespace OpenBabel
 
           // Keep a map linking the cif atom label to the obatom*, for bond interpretation later
           std::map<std::string,OBAtom *> vLabelOBatom;
-          
+
           const unsigned int nbatoms=pos->second.mvAtom.size();
           pmol->ReserveAtoms(nbatoms);
           for(vector<CIFData::CIFAtom>::const_iterator posat=pos->second.mvAtom.begin();posat!=pos->second.mvAtom.end();++posat)
@@ -1120,11 +1140,11 @@ namespace OpenBabel
                   if(isalpha(tmpSymbol[0]) && isalpha(tmpSymbol[1])) nbc=2;
                   else if(isalpha(tmpSymbol[0])) nbc=1;
                 }
-              
+
               OBAtom *atom  = pmol->NewAtom();
-              
+
               vLabelOBatom.insert(make_pair(posat->mLabel,atom));
-              
+
               if(tmpSymbol.size()>nbc)
                 {// Try to find a formal charge in the symbol
                   int charge=0;
@@ -1142,10 +1162,10 @@ namespace OpenBabel
                       atom->SetFormalCharge(sign*charge);
                     }
                 }
-              
+
               if(nbc>0) tmpSymbol=tmpSymbol.substr(0,nbc);
               else tmpSymbol="C";//Something went wrong, no symbol ! Default to C ??
-              
+
               int atomicNum = etab.GetAtomicNum(tmpSymbol.c_str());
               atom->SetAtomicNum(atomicNum); //set atomic number, or '0' if the atom type is not recognized
               atom->SetType(tmpSymbol); //set atomic number, or '0' if the atom type is not recognized
