@@ -3,6 +3,7 @@
   Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
   Some portions Copyright (C) 2004 by Chris Morley
   Some portions Copyright (C) 2006 by Donald E. Curtis
+  Some portions Copyright (C) 2009-2010 by Konstantin L. Tokarev
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
 using namespace std;
 namespace Gamess
 {
-} 
+}
 
 namespace OpenBabel
 {
@@ -53,7 +54,7 @@ namespace OpenBabel
     virtual const char* SpecificationURL()
     { return "http://www.msg.ameslab.gov/GAMESS/doc.menu.html";}; //optional
 
-    virtual const char* GetMIMEType() 
+    virtual const char* GetMIMEType()
     { return "chemical/x-gamess-output"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
@@ -106,7 +107,7 @@ namespace OpenBabel
     virtual const char* SpecificationURL()
     {return "http://www.msg.ameslab.gov/GAMESS/doc.menu.html";}; //optional
 
-    virtual const char* GetMIMEType() 
+    virtual const char* GetMIMEType()
     { return "chemical/x-gamess-input"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
@@ -220,12 +221,28 @@ namespace OpenBabel
     OBAtom *atom;
     vector<string> vs;
     bool hasPartialCharges = false;
-    int HOMO = 0;
+    int aHOMO, bHOMO = 0;
     vector<double> orbitals;
+    vector<std::string> symmetries;
 
-    vector<double> frequencies, intensities;
+    // coordinates of all steps
+    // Set conformers to all coordinates we adopted
+    std::vector<double*> vconf; // index of all frames/conformers
+    std::vector<double> coordinates; // coordinates in each frame
+    int natoms = 0;
+    int ndummyatoms = 0;
+
+    // OBConformerData stores information about multiple steps
+    OBConformerData *confData = new OBConformerData();
+    confData->SetOrigin(fileformatInput);
+    std::vector<unsigned short> confDimensions = confData->GetDimension(); // to be fair, set these all to 3D
+    std::vector<double>         confEnergies   = confData->GetEnergies();
+    std::vector< std::vector< vector3 > > confForces = confData->GetForces();
+
+    vector<double> frequencies, intensities, raman_intensities;
     vector< vector<vector3> > displacements;
-    int lowFreqModes = 6; // the number of modes to ignore
+    int lowFreqModesBegin; // the number of the first low frequency mode
+    int lowFreqModesEnd; // the number of the last low frequency mode
     int numFreq, numIntens, numDisp; // GAMESS prints rotations & transl., which we ignore
     numFreq = numIntens = numDisp = 0;
 
@@ -234,37 +251,60 @@ namespace OpenBabel
     gmsset->SetAttribute("gamess");
     gmsset->SetOrigin(fileformatInput);
 
+    mol.Clear();
     mol.BeginModify();
     while (ifs.getline(buffer,BUFF_SIZE))
       {
         if(strstr(buffer,"ATOMIC                      COORDINATES (BOHR)") != NULL)
           {
-            mol.Clear();
-            mol.BeginModify();
+            //mol.Clear();
+            //mol.BeginModify();
 
             ifs.getline(buffer,BUFF_SIZE);	// column headings
             ifs.getline(buffer,BUFF_SIZE);
             tokenize(vs,buffer);
             while (vs.size() == 5)
               {
-                atom = mol.NewAtom();
-                atom->SetAtomicNum(atoi(vs[1].c_str())); // Parse the current one
+                //atom = mol.NewAtom();
+                //atom->SetAtomicNum(atoi(vs[1].c_str())); // Parse the current one
+                int atomicNum = atoi(vs[1].c_str());
                 x = atof((char*)vs[2].c_str()) * BOHR_TO_ANGSTROM;
                 y = atof((char*)vs[3].c_str()) * BOHR_TO_ANGSTROM;
                 z = atof((char*)vs[4].c_str()) * BOHR_TO_ANGSTROM;
-                atom->SetVector(x,y,z);
-                vs[1].erase(vs[1].size() - 2, 2);
+                //atom->SetVector(x,y,z);
+                //vs[1].erase(vs[1].size() - 2, 2);
+                if (natoms == 0) // first time reading the molecule, create each atom
+                  {
+                    atom = mol.NewAtom();
+                    atom->SetAtomicNum(atomicNum);
+                  }
+                coordinates.push_back(x);
+                coordinates.push_back(y);
+                coordinates.push_back(z);
 
                 if (!ifs.getline(buffer,BUFF_SIZE))
                   break;
                 tokenize(vs,buffer);
               }
+            // done with reading atoms
+            natoms = mol.NumAtoms();
+            // malloc / memcpy
+            double *tmpCoords = new double [(natoms)*3];
+            memcpy(tmpCoords, &coordinates[0], sizeof(double)*natoms*3);
+            vconf.push_back(tmpCoords);
+            coordinates.clear();
+            confDimensions.push_back(3); // always 3D -- OBConformerData allows mixing 2D and 3D structures
           }
         else if(strstr(buffer,"MULTIPOLE COORDINATES, ELECTRONIC AND NUCLEAR CHARGES") != NULL)
           {
             /*This set of EFP coordinates belongs only to the
              * conformer directly above this (ATOMIC   COORDINATES (BOHR))
              */
+            double *tmpCoords = vconf.at(0);
+            //memcpy(&coordinates[0], tmpCoords, sizeof(double)*natoms*3);
+            for (int i=0; i<natoms*3; i++)
+              coordinates.push_back(tmpCoords[i]);
+
             ifs.getline(buffer,BUFF_SIZE);      // column headings
             ifs.getline(buffer,BUFF_SIZE);
             ifs.getline(buffer,BUFF_SIZE);
@@ -277,35 +317,50 @@ namespace OpenBabel
                  * or have a non-zero nuclear charge
                  */
                 if (atof((char*)vs[5].c_str()) > 0.0) {
-                  atom = mol.NewAtom();
-                  atomicNum=etab.GetAtomicNum(vs[0].substr(0,1).c_str()); 
-                  atom->SetAtomicNum(atomicNum);
+                  //atom = mol.NewAtom();
+                  atomicNum=etab.GetAtomicNum(vs[0].substr(0,1).c_str());
+                  //atom->SetAtomicNum(atomicNum);
+                  if (natoms == 0) // first time reading the molecule, create each atom
+                  {
+                    atom = mol.NewAtom();
+                    atom->SetAtomicNum(atomicNum);
+                  }
                   x = atof((char*)vs[1].c_str())* BOHR_TO_ANGSTROM;
                   y = atof((char*)vs[2].c_str())* BOHR_TO_ANGSTROM;
                   z = atof((char*)vs[3].c_str())* BOHR_TO_ANGSTROM;
-                  atom->SetVector(x,y,z);
-
+                  //atom->SetVector(x,y,z);
+                  coordinates.push_back(x);
+                  coordinates.push_back(y);
+                  coordinates.push_back(z);
                 }
                 else if (vs[0].substr(0,1) == "Z") {
-                  atom = mol.NewAtom();
+                  //atom = mol.NewAtom();
                   atomicNum=etab.GetAtomicNum(vs[0].substr(1,1).c_str());
-                  atom->SetAtomicNum(atomicNum);
+                  //atom->SetAtomicNum(atomicNum);
                   x = atof((char*)vs[1].c_str())* BOHR_TO_ANGSTROM;
                   y = atof((char*)vs[2].c_str())* BOHR_TO_ANGSTROM;
                   z = atof((char*)vs[3].c_str())* BOHR_TO_ANGSTROM;
-                  atom->SetVector(x,y,z);
-
+                  //atom->SetVector(x,y,z);
+                  coordinates.push_back(x);
+                  coordinates.push_back(y);
+                  coordinates.push_back(z);
                 }
                 if (!ifs.getline(buffer,BUFF_SIZE))
                   break;
                 tokenize(vs,buffer);
               }
+            // done with reading atoms
+            ndummyatoms = mol.NumAtoms();
+            // malloc / memcpy
+            memcpy(tmpCoords, &coordinates[0], sizeof(double)*natoms*3);
+            vconf[0] = tmpCoords;
+            coordinates.clear();
           }
 
         else if(strstr(buffer,"COORDINATES OF ALL ATOMS ARE (ANGS)") != NULL)
           {
-            mol.Clear();
-            mol.BeginModify();
+            //mol.Clear();
+            //mol.BeginModify();
 
             ifs.getline(buffer,BUFF_SIZE);	// column headings
             ifs.getline(buffer,BUFF_SIZE);	// ---------------
@@ -313,18 +368,28 @@ namespace OpenBabel
             tokenize(vs,buffer);
             while (vs.size() == 5)
               {
-                atom = mol.NewAtom();
-                atom->SetAtomicNum(atoi(vs[1].c_str())); // Parse the current one
+                //atom = mol.NewAtom();
+                //atom->SetAtomicNum(atoi(vs[1].c_str())); // Parse the current one
+                int atomicNum = atoi(vs[1].c_str());
                 x = atof((char*)vs[2].c_str());
                 y = atof((char*)vs[3].c_str());
                 z = atof((char*)vs[4].c_str());
-                atom->SetVector(x,y,z);
-                vs[1].erase(vs[1].size() - 2, 2);
+                //atom->SetVector(x,y,z);
+                //vs[1].erase(vs[1].size() - 2, 2);
+                if (natoms == 0) // first time reading the molecule, create each atom
+                  {
+                    atom = mol.NewAtom();
+                    atom->SetAtomicNum(atomicNum);
+                  }
+                coordinates.push_back(x);
+                coordinates.push_back(y);
+                coordinates.push_back(z);
 
                 if (!ifs.getline(buffer,BUFF_SIZE))
                   break;
                 tokenize(vs,buffer);
               }
+
             if(strstr(buffer,"COORDINATES OF FRAGMENT") != NULL)
               {
                 ifs.getline(buffer,BUFF_SIZE);      // column headings
@@ -340,38 +405,72 @@ namespace OpenBabel
                     tokenize(vs2,buffer,delim);
                   }
                   else {
-                    atom = mol.NewAtom();
+                    //atom = mol.NewAtom();
                     /* For the included EFP1 potentials,
                      * the atom name may start with "Z"
                      */
                     int atomicNum;
-                    if ( vs[0].substr(0,1) == "Z" ) 
-                      atomicNum=etab.GetAtomicNum(vs[0].substr(1,1).c_str()); 
-                    else 
-                      atomicNum=etab.GetAtomicNum(vs[0].substr(0,1).c_str()); 
-                    atom->SetAtomicNum(atomicNum);
+                    if ( vs[0].substr(0,1) == "Z" )
+                      atomicNum=etab.GetAtomicNum(vs[0].substr(1,1).c_str());
+                    else
+                      atomicNum=etab.GetAtomicNum(vs[0].substr(0,1).c_str());
+                    //atom->SetAtomicNum(atomicNum);
+                    if (natoms == 0) // first time reading the molecule, create each atom
+                      {
+                        atom = mol.NewAtom();
+                        atom->SetAtomicNum(atomicNum);
+                      }
                     x = atof((char*)vs[1].c_str());
                     y = atof((char*)vs[2].c_str());
                     z = atof((char*)vs[3].c_str());
-                    atom->SetVector(x,y,z);
+                    //atom->SetVector(x,y,z);
+                    coordinates.push_back(x);
+                    coordinates.push_back(y);
+                    coordinates.push_back(z);
                   }
-			    
 
                   if (!ifs.getline(buffer,BUFF_SIZE))
                     break;
                   tokenize(vs,buffer);
                 }
               }
+
+            // done with reading atoms
+            natoms = mol.NumAtoms();
+            // malloc / memcpy
+            double *tmpCoords = new double [(natoms)*3];
+            memcpy(tmpCoords, &coordinates[0], sizeof(double)*natoms*3);
+            vconf.push_back(tmpCoords);
+            coordinates.clear();
+            confDimensions.push_back(3); // always 3D -- OBConformerData allows mixing 2D and 3D structures
+
+          }
+        else if((strstr(buffer,"NSERCH=") != NULL) && (strstr(buffer,"ENERGY=") != NULL))
+          {
+            char *tok = strtok (buffer," ="); // my tokenize
+            int n=0;
+            while (true)
+              {
+                tok = strtok (NULL, " =");
+                if (tok == NULL)
+                  break;
+                n++;
+                if (n == 3)
+                  {
+                    confEnergies.push_back(atof(tok));
+                    break;
+                  }
+              }
           }
         else if(strstr(buffer,"ELECTROSTATIC MOMENTS") != NULL)
           {
             ifs.getline(buffer,BUFF_SIZE); //-----
             ifs.getline(buffer,BUFF_SIZE);  // blank line
             ifs.getline(buffer,BUFF_SIZE);	// column headings
-            ifs.getline(buffer,BUFF_SIZE); // point charges TODO
+            ifs.getline(buffer,BUFF_SIZE); // point charges @todo
             ifs.getline(buffer,BUFF_SIZE);	// column headings dipole moment
             ifs.getline(buffer,BUFF_SIZE);
-            
+
             tokenize(vs, buffer);
             if (vs.size() == 4) {
               OBVectorData *dipoleMoment = new OBVectorData;
@@ -383,7 +482,7 @@ namespace OpenBabel
               dipoleMoment->SetData(x, y, z);
               dipoleMoment->SetOrigin(fileformatInput);
               mol.SetData(dipoleMoment);
-            } 
+            }
           }
         else if(strstr(buffer,"MOPAC CHARGES") != NULL)
           {
@@ -422,34 +521,54 @@ namespace OpenBabel
           {
             tokenize(vs, buffer);
             if (vs.size() == 7) // alpha
-              HOMO = atoi(vs[6].c_str());
+              aHOMO = atoi(vs[6].c_str());
             else if (vs.size() == 8) //beta
-              HOMO = atoi(vs[7].c_str());
+              bHOMO = atoi(vs[7].c_str());
           }
         else if (strstr(buffer, "TAKEN AS ROTATIONS AND TRANSLATIONS") != NULL)
           {
             tokenize(vs, buffer);
             if (vs.size() < 4)
               break;
-            lowFreqModes = atoi(vs[3].c_str());
+            lowFreqModesBegin = atoi(vs[1].c_str());
+            lowFreqModesEnd = atoi(vs[3].c_str());
           }
-        else if (strstr(buffer,"FREQUENCY") != NULL)
+        else if (strstr(buffer,"TOTAL ENERGY      =") != NULL)
+          {
+            tokenize(vs, buffer);
+            if (vs.size() == 4)
+              mol.SetEnergy(atof(vs[3].c_str()));
+          }
+        else if (strstr(buffer,"FREQUENCY:") != NULL)
           {
             tokenize(vs, buffer);
             for (unsigned int i = 1; i < vs.size(); ++i) {
+              if (vs[i] == "I") // artifact from previous imaginary frequency
+                continue;
               ++numFreq;
-              if (numFreq > lowFreqModes)
+              if (numFreq < lowFreqModesBegin) // imaginary frequency
+                frequencies.push_back(-atof(vs[i].c_str()));
+              if (numFreq > lowFreqModesEnd)
                 frequencies.push_back(atof(vs[i].c_str()));
             }
             ifs.getline(buffer, BUFF_SIZE); // reduced mass
             ifs.getline(buffer, BUFF_SIZE);
             tokenize(vs, buffer);
-            for (unsigned int i = 1; i < vs.size(); ++i) {
+            for (unsigned int i = 2; i < vs.size(); ++i) {
               ++numIntens;
-              if (numIntens > lowFreqModes)
-                intensities.push_back(atof(vs[i].c_str()));
+              if (numIntens < lowFreqModesBegin || numIntens > lowFreqModesEnd)
+                intensities.push_back(atof(vs[i].c_str()) * 42.255); // conver to km/mol
+            }
+            ifs.getline(buffer, BUFF_SIZE); // blank or Raman activitie
+            if (strstr(buffer,"RAMAN") != NULL) {
+              tokenize(vs, buffer);
+              for (unsigned int i = 2; i < vs.size(); ++i) {
+                if (numIntens < lowFreqModesBegin || numIntens > lowFreqModesEnd)
+                  raman_intensities.push_back(atof(vs[i].c_str()));
+              }
+              ifs.getline(buffer, BUFF_SIZE); // DEPOLARIZATION
+              ifs.getline(buffer, BUFF_SIZE); // blank
             }
-            ifs.getline(buffer, BUFF_SIZE); // blank
 
             // now real work -- read displacements
             int prevModeCount = displacements.size();
@@ -462,33 +581,39 @@ namespace OpenBabel
             ifs.getline(buffer, BUFF_SIZE);
             tokenize(vs, buffer);
             int modeCount = vs.size() - 3;
+            double massNormalization;
             vector<double> x, y, z;
-            while(modeCount > 1) {
+            while(modeCount >= 1) {
+              // 1/sqrt(atomic mass)
+              atom = mol.GetAtom(atoi(vs[0].c_str()));
+              massNormalization = 1 / sqrt( atom->GetAtomicMass() );
+
               x.clear();
               // not a typo -- e.g., atom number, atom label, x, then data
               for (unsigned int i = 3; i < vs.size(); ++i) {
-                x.push_back(atof(vs[i].c_str()));
+                x.push_back(massNormalization * atof(vs[i].c_str()));
               }
               y.clear();
               ifs.getline(buffer, BUFF_SIZE);
               tokenize(vs, buffer);
               for (unsigned int i = 1; i < vs.size(); ++i) {
-                y.push_back(atof(vs[i].c_str()));
+                y.push_back(massNormalization * atof(vs[i].c_str()));
               }
-              
+
               z.clear();
               ifs.getline(buffer, BUFF_SIZE);
               tokenize(vs, buffer);
               for (unsigned int i = 1; i < vs.size(); ++i) {
-                z.push_back(atof(vs[i].c_str()));
+                z.push_back(massNormalization * atof(vs[i].c_str()));
               }
 
               // OK, now we have x, y, z for all new modes for one atom
               if (displacements.size()) {
-                for (unsigned int i = 0; i < modeCount - 1;  ++i) {
-                  ++numDisp;
-                  if (numDisp > lowFreqModes)
-                    displacements[prevModeCount + i].push_back(vector3(x[i], y[i], z[i]));
+                numDisp = prevModeCount;
+                for (unsigned int i = 0; i < modeCount;  ++i) {
+                  if (i >= modeCount - newModes){
+                    displacements[numDisp++].push_back(vector3(x[i], y[i], z[i]));
+                  }
                 }
               }
 
@@ -504,6 +629,7 @@ namespace OpenBabel
             ifs.getline(buffer,BUFF_SIZE); // ------ line
             ifs.getline(buffer,BUFF_SIZE); // blank
             orbitals.clear();
+            symmetries.clear();
 
             while (strstr(buffer,"END OF RHF CALCULATION") == NULL &&
                    strstr(buffer,"-------") == NULL)
@@ -512,13 +638,17 @@ namespace OpenBabel
                 ifs.getline(buffer,BUFF_SIZE); // orbitals!
                 ifs.getline(buffer,BUFF_SIZE); // energies in hartree
                 tokenize(vs, buffer);
-                for (unsigned int i = 0; i < vs.size(); i++)
+                for (unsigned int i = 0; i < vs.size(); ++i)
                   orbitals.push_back(27.21 * atof(vs[i].c_str()));
 
                 ifs.getline(buffer,BUFF_SIZE); // symmetries
+                tokenize(vs, buffer);
+                for (unsigned int i = 0; i < vs.size(); ++i)
+                  symmetries.push_back(vs[i]);
+
                 // orbital coefficients
                 while (ifs.getline(buffer,BUFF_SIZE) && strlen(buffer)
-                       && strstr(buffer,"END") == NULL 
+                       && strstr(buffer,"END") == NULL
                        && strstr(buffer, "---") == NULL)
                   { }
                 if (!ifs.good())
@@ -584,14 +714,21 @@ namespace OpenBabel
           }
         */
       }
-    //    cerr << " HOMO " << HOMO - 1 << " LUMO " << HOMO << " size: " << orbitals.size() << endl;
-    //    cerr << title << " " << HOMO << " " << orbitals[HOMO - 1] << " " << orbitals[HOMO] << endl;
-
     if (mol.NumAtoms() == 0) { // e.g., if we're at the end of a file PR#1737209
       mol.EndModify();
       return false;
     }
 
+    if (orbitals.size() > 0) { // add OBOrbitalData
+      OBOrbitalData *od = new OBOrbitalData();
+
+      if (aHOMO == bHOMO) {
+        od->LoadClosedShellOrbitals(orbitals, symmetries, aHOMO);
+      }
+      od->SetOrigin(fileformatInput);
+      mol.SetData(od);
+    }
+
     const char *keywordsEnable = pConv->IsOption("k",OBConversion::GENOPTIONS);
 
     if(keywordsEnable)
@@ -647,7 +784,7 @@ namespace OpenBabel
             if(pd)
               {
                 string value = pd->GetValue();
-                if(value == "GRADIENT" || value == "HESSIAN" || value == "OPTIMIZE" || value == "SADPOINT")
+                if(value == "GRADIENT" || value == "HESSIAN" || value == "RAMAN" || value == "OPTIMIZE" || value == "SADPOINT")
                   {
                     method = pd->GetValue();
                     transform(method.begin(), method.end(), method.begin(), ::tolower);
@@ -727,12 +864,25 @@ namespace OpenBabel
         */
       }
 
+    mol.EndModify();
+
+    // Set conformers to all coordinates we adopted
+    // but remove last geometry -- it's a duplicate
+    if (vconf.size() > 1)
+      vconf.pop_back();
+    mol.SetConformers(vconf);
+    mol.SetConformer(mol.NumConformers() - 1);
+    // Copy the conformer data too
+    confData->SetDimension(confDimensions);
+    confData->SetEnergies(confEnergies);
+    confData->SetForces(confForces);
+    mol.SetData(confData);
+
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
     if (!pConv->IsOption("s",OBConversion::INOPTIONS) && !pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.PerceiveBondOrders();
 
-    mol.EndModify();
     if (hasPartialCharges) {
       mol.SetPartialChargesPerceived();
 
@@ -745,7 +895,7 @@ namespace OpenBabel
     }
     if (frequencies.size() != 0) { // we found some vibrations
       OBVibrationData *vd = new OBVibrationData;
-      vd->SetData(displacements, frequencies, intensities);
+      vd->SetData(displacements, frequencies, intensities, raman_intensities);
       vd->SetOrigin(fileformatInput);
       mol.SetData(vd);
     }
@@ -774,6 +924,7 @@ namespace OpenBabel
     OBAtom *atom;
     vector<string> vs;
     bool hasPartialCharges = false;
+    string efragName; // used to save identifiers of EFRAG sections
 
     mol.BeginModify();
     while	(ifs.getline(buffer,BUFF_SIZE))
@@ -788,7 +939,7 @@ namespace OpenBabel
             while (strstr(buffer, "$END") == NULL)
               {
                 tokenize(vs,buffer);
-                if(vs.size() == 5) 
+                if(vs.size() == 5)
                   {
                     atom = mol.NewAtom();
                     atom->SetAtomicNum(atoi(vs[1].c_str())); // Parse the current one
@@ -796,7 +947,6 @@ namespace OpenBabel
                     y = atof((char*)vs[3].c_str());
                     z = atof((char*)vs[4].c_str());
                     atom->SetVector(x,y,z);
-                    vs[1].erase(vs[1].size() - 2, 2);
                   }
 
                 if (!ifs.getline(buffer,BUFF_SIZE))
@@ -808,6 +958,9 @@ namespace OpenBabel
             while (strstr(buffer,"FRAGNAME") == NULL)
               {
                 //read $EFRAG parameters
+                tokenize(vs, buffer, "=");
+                if (vs.size() > 1)
+                  efragName = vs[1];
                 if(!ifs.getline(buffer,BUFF_SIZE))
                   break;
               }
@@ -818,7 +971,7 @@ namespace OpenBabel
                   {
                     atom = mol.NewAtom();
                     int atomicNum;
-                    if( vs[0].substr(0,1) == "Z" || vs[0].substr(0,1) == "z" ) 
+                    if( vs[0].substr(0,1) == "Z" || vs[0].substr(0,1) == "z" )
                       atomicNum=etab.GetAtomicNum(vs[0].substr(1,1).c_str());
                     else
                       atomicNum=etab.GetAtomicNum(vs[0].substr(0,1).c_str());
@@ -827,6 +980,13 @@ namespace OpenBabel
                     y = atof((char*)vs[2].c_str());
                     z = atof((char*)vs[3].c_str());
                     atom->SetVector(x,y,z);
+
+                    // Tag these atoms as part of a specific EFP fragment
+                    OBPairData *dp = new OBPairData;
+                    dp->SetAttribute("EFRAG");
+                    dp->SetValue(efragName);
+                    dp->SetOrigin(fileformatInput);
+                    atom->SetData(dp);
                   }
                 if(!ifs.getline(buffer,BUFF_SIZE))
                   break;
