@@ -1,12 +1,12 @@
 /**********************************************************************
 Copyright (C) 2000 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2010 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2004 by Chris Morley
- 
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -31,7 +31,7 @@ namespace OpenBabel
       OBConversion::RegisterFormat("g94",this);
       OBConversion::RegisterFormat("g98",this);
       OBConversion::RegisterFormat("g03",this);
-      OBConversion::RegisterFormat("g09",this); // Not tested, but should work
+      OBConversion::RegisterFormat("g09",this);
     }
 
     virtual const char* Description() //required
@@ -46,7 +46,7 @@ namespace OpenBabel
     virtual const char* SpecificationURL()
     { return "http://www.gaussian.com/";};
 
-    virtual const char* GetMIMEType() 
+    virtual const char* GetMIMEType()
     { return "chemical/x-gaussian-log"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
@@ -93,7 +93,7 @@ namespace OpenBabel
     virtual const char* SpecificationURL()
     {return "http://www.gaussian.com/g_ur/m_input.htm";};
 
-    virtual const char* GetMIMEType() 
+    virtual const char* GetMIMEType()
     { return "chemical/x-gaussian-input"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
@@ -128,14 +128,14 @@ namespace OpenBabel
     const char *keywords = pConv->IsOption("k",OBConversion::OUTOPTIONS);
     const char *keywordsEnable = pConv->IsOption("k",OBConversion::GENOPTIONS);
     const char *keywordFile = pConv->IsOption("f",OBConversion::OUTOPTIONS);
-    bool writeUnitCell = pConv->IsOption("u", OBConversion::OUTOPTIONS);
+    bool writeUnitCell = (NULL != pConv->IsOption("u", OBConversion::OUTOPTIONS));
     string defaultKeywords = "#Put Keywords Here, check Charge and Multiplicity.";
 
     if(keywords)
       {
         defaultKeywords = keywords;
       }
-    
+
     if (keywordsEnable)
       {
         string model;
@@ -179,14 +179,14 @@ namespace OpenBabel
               ofs << keyBuffer << endl;
           }
       }
-    else 
+    else
       {
         ofs << defaultKeywords << endl;
       }
     ofs << endl; // blank line after keywords
     ofs << " " << mol.GetTitle() << endl << endl;
 
-    snprintf(buffer, BUFF_SIZE, "%d  %d", 
+    snprintf(buffer, BUFF_SIZE, "%d  %d",
              mol.GetTotalCharge(),
              mol.GetTotalSpinMultiplicity());
     ofs << buffer << endl;
@@ -202,7 +202,7 @@ namespace OpenBabel
                    etab.GetSymbol(atom->GetAtomicNum()),
                    atom->GetIsotope(),
                    atom->GetX(), atom->GetY(), atom->GetZ());
-	
+
         ofs << buffer << endl;
       }
     // Translation vectors
@@ -229,7 +229,7 @@ namespace OpenBabel
       vector<OBEdgeBase*>::iterator j;
       vector<OBNodeBase*>::iterator i;
       OBAtom *bgn, *end;
-      for (bond = mol.BeginBond(j); bond; bond = mol.NextBond(j)) 
+      for (bond = mol.BeginBond(j); bond; bond = mol.NextBond(j))
         {
           if (bond->GetBeginAtomIdx() > bond->GetEndAtomIdx()) {
             bgn = bond->GetBeginAtom();
@@ -244,7 +244,7 @@ namespace OpenBabel
       for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
         {
           ofs << endl << atom->GetIdx() << " ";
-          for (bond = mol.BeginBond(j); bond; bond = mol.NextBond(j)) 
+          for (bond = mol.BeginBond(j); bond; bond = mol.NextBond(j))
             {
               if (bond->GetBeginAtomIdx() == atom->GetIdx()) {
                 snprintf(buffer, BUFF_SIZE, "%d %1.1f ", bond->GetEndAtomIdx(), (float) bond->GetBondOrder());
@@ -281,6 +281,21 @@ namespace OpenBabel
     int charge = 0;
     unsigned int spin = 1;
     bool hasPartialCharges = false;
+    string chargeModel; // descriptor for charges (e.g. "Mulliken")
+
+    // coordinates of all steps
+    // Set conformers to all coordinates we adopted
+    std::vector<double*> vconf; // index of all frames/conformers
+    std::vector<double> coordinates; // coordinates in each frame
+    int natoms = 0; // number of atoms -- ensure we don't go to a new job with a different molecule
+
+    // OBConformerData stores information about multiple steps
+    // we can change attribute later if needed (e.g., IRC)
+    OBConformerData *confData = new OBConformerData();
+    confData->SetOrigin(fileformatInput);
+    std::vector<unsigned short> confDimensions = confData->GetDimension(); // to be fair, set these all to 3D
+    std::vector<double>         confEnergies   = confData->GetEnergies();
+    std::vector< std::vector< vector3 > > confForces = confData->GetForces();
 
     //Vibrational data
     std::vector< std::vector< vector3 > > Lx;
@@ -294,8 +309,17 @@ namespace OpenBabel
     vector3 translationVectors[3];
     int numTranslationVectors = 0;
 
+    //Electronic Excitation data
+    std::vector<double> Forces, Wavelengths, EDipole,
+      RotatoryStrengthsVelocity, RotatoryStrengthsLength;
+
+    // Orbital data
+    std::vector<double> orbitals;
+    std::vector<std::string> symmetries;
+    int aHOMO, bHOMO;
+
     mol.BeginModify();
-    
+    bool have_coords = 0;
     while (ifs.getline(buffer,BUFF_SIZE))
       {
         if (strstr(buffer,"Multiplicity") != NULL)
@@ -306,15 +330,18 @@ namespace OpenBabel
                 charge = atoi(vs[2].c_str());
                 spin = atoi(vs[5].c_str());
               }
-	    
+
             ifs.getline(buffer,BUFF_SIZE);
           }
-        else if(strstr(buffer,"Coordinates (Angstroms)") != NULL)
+        else if((strstr(buffer,"Input orientation:") != NULL)
+                || ((strstr(buffer,"Standard orientation:") != NULL) && (!have_coords))
+                || ((strstr(buffer,"Z-Matrix orientation:") != NULL) && (!have_coords)))
           {
-            // mol.EndModify();
-            mol.Clear();
+            if (strstr(buffer,"Input orientation:") != NULL)
+              have_coords = 1; // if we came here from "Input orientation", disable reading "Standard orientation"
             numTranslationVectors = 0; // ignore old translationVectors
-            mol.BeginModify();
+            ifs.getline(buffer,BUFF_SIZE);      // ---------------
+            ifs.getline(buffer,BUFF_SIZE);      // column headings
             ifs.getline(buffer,BUFF_SIZE);	// column headings
             ifs.getline(buffer,BUFF_SIZE);	// ---------------
             ifs.getline(buffer,BUFF_SIZE);
@@ -324,27 +351,41 @@ namespace OpenBabel
                 x = atof((char*)vs[3].c_str());
                 y = atof((char*)vs[4].c_str());
                 z = atof((char*)vs[5].c_str());
-
                 int atomicNum = atoi((char*)vs[1].c_str());
+
                 if (atomicNum > 0) // translation vectors are "-2"
                   {
-                    atom = mol.NewAtom();
-                    atom->SetAtomicNum(atoi((char*)vs[1].c_str()));
-                    atom->SetVector(x,y,z);
+                    if (natoms == 0) { // first time reading the molecule, create each atom
+                      atom = mol.NewAtom();
+                      atom->SetAtomicNum(atoi((char*)vs[1].c_str()));
+                    }
+                    coordinates.push_back(x);
+                    coordinates.push_back(y);
+                    coordinates.push_back(z);
                   }
                 else {
                   translationVectors[numTranslationVectors++].Set(x, y, z);
                 }
-		
-                if (!ifs.getline(buffer,BUFF_SIZE)) break;
+
+                if (!ifs.getline(buffer,BUFF_SIZE)) {
+                  break;
+                }
                 tokenize(vs,buffer);
               }
+            // done with reading atoms
+            natoms = mol.NumAtoms();
+            // malloc / memcpy
+            double *tmpCoords = new double [(natoms)*3];
+            memcpy(tmpCoords, &coordinates[0], sizeof(double)*natoms*3);
+            vconf.push_back(tmpCoords);
+            coordinates.clear();
+            confDimensions.push_back(3); // always 3D -- OBConformerData allows mixing 2D and 3D structures
           }
         else if(strstr(buffer,"Dipole moment") != NULL)
             {
               ifs.getline(buffer,BUFF_SIZE); // actual components   X ###  Y #### Z ###
               tokenize(vs,buffer);
-              if (vs.size() >= 6) 
+              if (vs.size() >= 6)
                 {
                   OBVectorData *dipoleMoment = new OBVectorData;
                   dipoleMoment->SetAttribute("Dipole Moment");
@@ -362,22 +403,42 @@ namespace OpenBabel
                 strstr(buffer,"Mulliken atomic charges") != NULL)
           {
             hasPartialCharges = true;
+            chargeModel = "Mulliken";
             ifs.getline(buffer,BUFF_SIZE);	// column headings
             ifs.getline(buffer,BUFF_SIZE);
             tokenize(vs,buffer);
-            while (vs.size() >= 3 && 
+            while (vs.size() >= 3 &&
                    strstr(buffer,"Sum of ") == NULL)
               {
                 atom = mol.GetAtom(atoi(vs[0].c_str()));
                 if (!atom)
                   break;
                 atom->SetPartialCharge(atof(vs[2].c_str()));
-		
+
                 if (!ifs.getline(buffer,BUFF_SIZE)) break;
                 tokenize(vs,buffer);
               }
           }
+        else if (strstr(buffer, "Charges from ESP fit") != NULL)
+          {
+            hasPartialCharges = true;
+            chargeModel = "ESP";
+            ifs.getline(buffer,BUFF_SIZE);	// Charge / dipole line
+            ifs.getline(buffer,BUFF_SIZE); // column header
+            ifs.getline(buffer,BUFF_SIZE); // real charges
+            tokenize(vs,buffer);
+            while (vs.size() >= 3 &&
+                   strstr(buffer,"-----") == NULL)
+              {
+                atom = mol.GetAtom(atoi(vs[0].c_str()));
+                if (!atom)
+                  break;
+                atom->SetPartialCharge(atof(vs[2].c_str()));
 
+                if (!ifs.getline(buffer,BUFF_SIZE)) break;
+                tokenize(vs,buffer);
+              }
+          }
         else if(strstr(buffer, " Frequencies -- ")) //vibrational frequencies
         {
           //The info should appear only once as several blocks starting with this line
@@ -406,7 +467,7 @@ namespace OpenBabel
               x = atof(vs[i].c_str());
               y = atof(vs[i+1].c_str());
               z = atof(vs[i+2].c_str());
-              
+
               if (i == 2)
                 vib1.push_back(vector3(x, y, z));
               else if (i == 5)
@@ -414,7 +475,7 @@ namespace OpenBabel
               else if (i == 8)
                 vib3.push_back(vector3(x, y, z));
             }
-            
+
             if (!ifs.getline(buffer, BUFF_SIZE))
               break;
             tokenize(vs,buffer);
@@ -439,13 +500,118 @@ namespace OpenBabel
           ifs.getline(buffer,BUFF_SIZE); //symmetry number
           tokenize(vs, buffer);
           RotSymNum = atoi(vs[3].c_str());
-          ifs.getline(buffer,BUFF_SIZE); //Rot temperatures
-          ifs.getline(buffer,BUFF_SIZE); //Rot consts
+          do
+          {
+            ifs.getline(buffer,BUFF_SIZE);
+          }while(ifs && !strstr(buffer, "Rotational constant"));
           tokenize(vs, buffer);
           for(int i=3; i<vs.size(); ++i)
             RotConsts[i-3] = atof(vs[i].c_str());
-         
+
         }
+        else if(strstr(buffer, "alpha electrons")) // # of electrons / orbital
+        {
+          tokenize(vs, buffer);
+          if (vs.size() == 6) {
+            // # alpha electrons # beta electrons
+            aHOMO = atoi(vs[0].c_str());
+            bHOMO = atoi(vs[3].c_str());
+          }
+        }
+        else if(strstr(buffer, "rbital symmetries")) // orbital symmetries
+          {
+            symmetries.clear();
+            std::string label; // used as a temporary to remove "(" and ")" from labels
+            int offset = 0;
+
+            ifs.getline(buffer, BUFF_SIZE);
+            tokenize(vs, buffer); // parse first line "Occupied" ...
+            for (unsigned int i = 1; i < vs.size(); ++i) {
+              label = vs[i].substr(1, vs[i].length() - 2);
+              symmetries.push_back(label);
+            }
+            ifs.getline(buffer, BUFF_SIZE);
+
+            // Parse remaining lines
+            while (strstr(buffer, "(")) {
+              tokenize(vs, buffer);
+              if (strstr(buffer, "Virtual")) {
+                offset = 1; // skip first token
+              } else {
+                offset = 0;
+              }
+              for (unsigned int i = offset; i < vs.size(); ++i) {
+                label = vs[i].substr(1, vs[i].length() - 2);
+                symmetries.push_back(label);
+              }
+              ifs.getline(buffer, BUFF_SIZE); // get next line
+            } // end parsing symmetry labels
+          }
+        else if (strstr(buffer, "Alpha") && strstr(buffer, ". eigenvalues --")) {
+          orbitals.clear();
+          while (strstr(buffer, ". eigenvalues --")) {
+            tokenize(vs, buffer);
+            if (vs.size() < 4)
+              break;
+            for (unsigned int i = 4; i < vs.size(); ++i) {
+              orbitals.push_back(atof(vs[i].c_str()));
+            }
+            ifs.getline(buffer, BUFF_SIZE);
+          }
+        }
+        else if(strstr(buffer, " Excited State")) // Force and wavelength data
+        {
+          // The above line appears for each state, so just append the info to the vectors
+          tokenize(vs, buffer);
+          if (vs.size() == 9) {
+            double wavelength = atof(vs[6].c_str());
+            double force = atof(vs[8].substr(2).c_str());
+            Forces.push_back(force);
+            Wavelengths.push_back(wavelength);
+          }
+        }
+        else if(strstr(buffer, " Ground to excited state Transition electric dipole moments (Au):"))
+          // Electronic dipole moments
+        {
+          ifs.getline(buffer, BUFF_SIZE); // Headings
+          ifs.getline(buffer, BUFF_SIZE); // First entry
+          tokenize(vs, buffer);
+          while (vs.size() == 5) {
+            double s = atof(vs[4].c_str());
+            EDipole.push_back(s);
+            ifs.getline(buffer, BUFF_SIZE);
+            tokenize(vs, buffer);
+          }
+        }
+        else if(strstr(buffer, "       state          X           Y           Z     R(velocity)")) {
+          // Rotatory Strengths
+          ifs.getline(buffer, BUFF_SIZE); // First entry
+          tokenize(vs, buffer);
+          while (vs.size() == 5) {
+            double s = atof(vs[4].c_str());
+            RotatoryStrengthsVelocity.push_back(s);
+            ifs.getline(buffer, BUFF_SIZE);
+            tokenize(vs, buffer);
+          }
+        }
+        else if(strstr(buffer, "       state          X           Y           Z     R(length)")) {
+          // Rotatory Strengths
+          ifs.getline(buffer, BUFF_SIZE); // First entry
+          tokenize(vs, buffer);
+          while (vs.size() == 5) {
+            double s = atof(vs[4].c_str());
+            RotatoryStrengthsLength.push_back(s);
+            ifs.getline(buffer, BUFF_SIZE);
+            tokenize(vs, buffer);
+          }
+        }
+
+        else if (strstr(buffer, "Forces (Hartrees/Bohr)"))
+          {
+            ifs.getline(buffer, BUFF_SIZE); // column headers
+            ifs.getline(buffer, BUFF_SIZE); // ------
+            ifs.getline(buffer, BUFF_SIZE); // real data
+          }
 
         else if (strstr(buffer, "Isotropic = ")) // NMR shifts
           {
@@ -468,7 +634,10 @@ namespace OpenBabel
 #define HARTREE_TO_KCAL 627.509
             tokenize(vs,buffer);
             mol.SetEnergy(atof(vs[4].c_str()) * HARTREE_TO_KCAL);
+            confEnergies.push_back(mol.GetEnergy());
           }
+/* Temporarily commented out until the handling of energy in OBMol is sorted out
+        // MP2 energies also use a different syntax
 
         // PM3 energies use a different syntax
         else if(strstr(buffer,"E (Thermal)") != NULL)
@@ -477,15 +646,41 @@ namespace OpenBabel
             ifs.getline(buffer,BUFF_SIZE); //Total energy; what we want
             tokenize(vs,buffer);
             mol.SetEnergy(atof(vs[1].c_str()));
+            confEnergies.push_back(mol.GetEnergy());
           }
-
+*/
       } // end while
 
     if (mol.NumAtoms() == 0) { // e.g., if we're at the end of a file PR#1737209
       mol.EndModify();
       return false;
     }
-    
+
+    mol.EndModify();
+
+    // Set conformers to all coordinates we adopted
+    // but remove last geometry -- it's a duplicate
+    if (vconf.size() > 1)
+      vconf.pop_back();
+    mol.SetConformers(vconf);
+    mol.SetConformer(mol.NumConformers() - 1);
+    // Copy the conformer data too
+    confData->SetDimension(confDimensions);
+    confData->SetEnergies(confEnergies);
+    confData->SetForces(confForces);
+    mol.SetData(confData);
+
+    // Attach orbital data, if there is any
+    if (orbitals.size() > 0)
+      {
+        OBOrbitalData *od = new OBOrbitalData;
+        if (aHOMO == bHOMO) {
+          od->LoadClosedShellOrbitals(orbitals, symmetries, aHOMO);
+        }
+        od->SetOrigin(fileformatInput);
+        mol.SetData(od);
+      }
+
     //Attach vibrational data, if there is any, to molecule
     if(Frequencies.size()>0)
     {
@@ -509,28 +704,41 @@ namespace OpenBabel
       uc->SetOrigin(fileformatInput);
       mol.SetData(uc);
     }
+    //Attach electronic transition data, if there is any, to molecule
+    if(Forces.size() > 0 && Forces.size() == Wavelengths.size())
+    {
+      OBElectronicTransitionData* etd = new OBElectronicTransitionData;
+      etd->SetData(Wavelengths, Forces);
+      if (EDipole.size() == Forces.size())
+        etd->SetEDipole(EDipole);
+      if (RotatoryStrengthsLength.size() == Forces.size())
+        etd->SetRotatoryStrengthsLength(RotatoryStrengthsLength);
+      if (RotatoryStrengthsVelocity.size() == Forces.size())
+        etd->SetRotatoryStrengthsVelocity(RotatoryStrengthsVelocity);
+      etd->SetOrigin(fileformatInput);
+      mol.SetData(etd);
+    }
 
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
     if (!pConv->IsOption("s",OBConversion::INOPTIONS) && !pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.PerceiveBondOrders();
 
-    mol.EndModify();
     if (hasPartialCharges) {
       mol.SetPartialChargesPerceived();
 
       // Annotate that partial charges come from Mulliken
       OBPairData *dp = new OBPairData;
       dp->SetAttribute("PartialCharges");
-      dp->SetValue("Mulliken");
+      dp->SetValue(chargeModel); // Mulliken, ESP, etc.
       dp->SetOrigin(fileformatInput);
       mol.SetData(dp);
     }
     mol.SetTotalCharge(charge);
     mol.SetTotalSpinMultiplicity(spin);
-    
+
     mol.SetTitle(title);
     return(true);
   }
-    
+
 } //namespace OpenBabel
