@@ -9,7 +9,7 @@ IUPAC International Chemical Identifier (InChI) (contact:secretariat@iupac.org)
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,133 +21,19 @@ GNU General Public License for more details.
 #include <openbabel/obmolecformat.h>
 
 #include "inchi_api.h"
-#ifdef HAVE_SSTREAM
 #include <sstream>
-#else
-#include <strstream>
-#endif
 #include <set>
 #include <vector>
-#include "openbabel/chiral.h"
+#include <iterator>
+#include <openbabel/inchiformat.h>
+#include <openbabel/stereo/tetrahedral.h>
+#include <openbabel/stereo/cistrans.h>
 
 using namespace std;
 namespace OpenBabel
 {
 extern string GetInChI(istream& is);
 
-class InChIFormat : public OBMoleculeFormat
-{
-public:
-  InChIFormat()
-  {
-    OBConversion::RegisterFormat("inchi",this);
-    OBConversion::RegisterOptionParam("n", this, 0, OBConversion::INOPTIONS);
-    OBConversion::RegisterOptionParam("t", this);
-    OBConversion::RegisterOptionParam("l", this);
-    OBConversion::RegisterOptionParam("X", this, 1, OBConversion::OUTOPTIONS);
-    OBConversion::RegisterOptionParam("K", this, 0, OBConversion::OUTOPTIONS);
-    OBConversion::RegisterOptionParam("F", this, 0, OBConversion::OUTOPTIONS);
-    OBConversion::RegisterOptionParam("M", this, 0, OBConversion::OUTOPTIONS);
-    OBConversion::RegisterOptionParam("X", this, 1, OBConversion::INOPTIONS);
-  }
-
-  virtual const char* Description()
-  {
-    return 
-    "InChI format\n"
-    "IUPAC/NIST molecular identifier\n"
-    "Write options, e.g. -xat\n"
-    //" n do not use 'recommended' InChI options\n"
-    " X <Option string> List of additional InChI options\n"
-    //" F include fixed hydrogen layer\n"
-    //" M include bonds to metal\n"
-    " t add molecule name\n"
-    " a output auxilliary information\n"
-    " K output InChIKey\n"
-    " w don't warn on undef stereo or charge rearrangement\n"
-    " l display InChI log\n"
-    " u output only unique molecules\n"
-    " U output only unique molecules and sort them\n"
-    " e compare first molecule to others\n\n"
-
-    "Input options, e.g. -at\n"
-    " X <Option string> List of InChI options\n"
-    " n molecule name follows InChI on same line\n"
-    " a add InChI string to molecule name\n\n"
-    " The InChI options should be space delimited in a single quoted string.\n"
-    " See InChI documentation for possible options.\n\n" ;
-  };
-
-  virtual const char* SpecificationURL()
-  { return "http://www.iupac.org/inchi/";};
-
-  virtual bool ReadMolecule(OBBase* pOb, OBConversion* pConv);
-  virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
-  virtual int  SkipObjects(int n, OBConversion* pConv);
-
-  static char   CompareInchi(const char* Inchi1, const char* Inchi2);
-  static string InChIErrorMessage(const char ch);
-
-private:
-  OBAtom* GetCommonAtom(OBBond* pb1, OBBond* pb2);
-  char* GetInChIOptions(OBConversion* pConv, bool Reading);
-
-  ///Compare std::strings with embedded numbers so that 
-  // "a6b" (or "a06b") is less than "a15b"
-  // and "CH4" is less than "C2H6"
-  // and "CH4" is less than "ClH" (hydrogen chloride)
-  struct InchiLess
-    : public binary_function<const string&, const string&, bool>
-  {
-    bool operator()(const string& s1, const string& s2) const
-    {
-      string::const_iterator p1, p2;
-      p1=s1.begin(); p2=s2.begin();
-      while( p1!=s1.end() && p2!=s2.end() )
-      {
-        if(iscntrl(*p1) || iscntrl(*p2) || isspace(*p1) || isspace(*p2))
-          return false; //stop comparison at whitespace. Identical up to here 
-        int n1=-1,n2=-1;
-        if(isdigit(*p1))
-          {
-            n1 = atoi(&*p1);
-            //skip over number
-            while(p1!=s1.end() && isdigit(*p1++)); --p1;
-          }
-        if(isdigit(*p2))
-          {
-            n2 = atoi(&*p2);
-            while(p2!=s2.end() && isdigit(*p2++)); --p2;
-          }
-        if(n1<0 && n2 < 0)
-          {
-            //neither numbers
-            if(*p1 != *p2)
-        return *p1 < *p2;
-          }
-        else if(n1>=0 && n2>0)
-          {
-            //both numbers
-            if(n1!=n2)
-        return n1 < n2;
-          }
-        else if(n1>0)
-          return islower(*p2)!=0;
-        else if(n2>0)
-          return !islower(*p1);
-
-        ++p1; ++p2; // iterate
-      } // while loop
-      return false; //identical
-    }
-  };
-
-  typedef	set<string, InchiLess> nSet;
-  nSet allInchi;
-  string firstInchi;
-  string firstID;
-};
-
 //Make an instance of the format class
 InChIFormat theInChIFormat;
 
@@ -177,18 +63,22 @@ bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   inp.szInChI = strcpy(nonconstinchi, inchi.c_str());
 
   inchi_OutputStruct out;
+  memset(&out, 0, sizeof(out));
 
   //Call the conversion routine in InChI code
-  int ret = GetStructFromStdINCHI( &inp, &out );
+  int ret = GetStructFromINCHI( &inp, &out );
 
   if (ret!=inchi_Ret_OKAY)
   {
-    obErrorLog.ThrowError("InChI code", out.szMessage, obWarning);
+    string mes = out.szMessage ? out.szMessage : "Conversion failed";
+    obErrorLog.ThrowError("InChI code", mes + '\n' + inchi, obWarning);
   }
   delete[] nonconstinchi;
   delete[] opts;
+  if(ret)
+    return false;
 
-  //Read name if requested e.g InChI=1/CH4/h1H4 methane 
+  //Read name if requested e.g InChI=1/CH4/h1H4 methane
   //OR InChI=1/CH4/h1H4 "First alkane"  Quote can be any punct char and
   //uses upto the end of the line if second quote is not found
   if(pConv->IsOption("n",OBConversion::INOPTIONS))
@@ -254,70 +144,71 @@ bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     }
   }
 
-  //***TODO 0D stereo, implicit H isotopes
+  //***@todo implicit H isotopes
   //Stereochemistry
   for(i=0;i<out.num_stereo0D;++i)
   {
     inchi_Stereo0D& stereo = out.stereo0D[i];
 
-    vector<unsigned int> refs;
-    for(int i=0;i<4; ++i)
-      refs.push_back(stereo.neighbor[i] + 1);
-
     switch(stereo.type)
     {
     case INCHI_StereoType_DoubleBond:
     {
-      // Parity 'unknown', 'undefined' not handled
-      if(!(stereo.parity==INCHI_PARITY_EVEN || stereo.parity==INCHI_PARITY_ODD))
-        break;
-      OBBond* pBond1 = pmol->GetBond(refs[0], refs[1]); //attached bonds
-      OBBond* pBond2 = pmol->GetBond(refs[2], refs[3]); 
-      if(!pBond1 || !pBond2)
-        return false;
-      bool ud = (stereo.parity==INCHI_PARITY_EVEN) ? true : false;//'trans'
-      if(!pBond2->IsUp() && !pBond2->IsDown())
-      {
-        //pBond2 not previously set
-        if(pBond1->IsUp())
-          ud = !ud;
-        if(ud)
-          pBond2->SetDown();
-        else
-          pBond2->SetUp();
-        if(!pBond1->IsUp() && !pBond1->IsDown())
-          pBond1->SetUp();
+      OBCisTransStereo::Config *ct = new OBCisTransStereo::Config;
+
+      ct->begin = stereo.neighbor[1];
+      ct->end = stereo.neighbor[2];
+      unsigned long start = OBStereo::ImplicitRef;
+      unsigned long end = OBStereo::ImplicitRef;
+      FOR_NBORS_OF_ATOM(a, pmol->GetAtom(ct->begin + 1)) {
+        if ( !(a->GetId() == ct->end || a->GetId() == stereo.neighbor[0] ) ) {
+          start = a->GetId();
+          break;
+        }
       }
+      FOR_NBORS_OF_ATOM(b, pmol->GetAtom(ct->end + 1)) {
+        if ( !(b->GetId() == ct->begin || b->GetId() == stereo.neighbor[3] ) ) {
+          end = b->GetId();
+          break;
+        }
+      }
+      ct->refs = OBStereo::MakeRefs(stereo.neighbor[0], start, stereo.neighbor[3], end);
+
+      if(stereo.parity==INCHI_PARITY_EVEN)
+        ct->shape = OBStereo::ShapeU;
+      else if(stereo.parity==INCHI_PARITY_ODD)
+        ct->shape = OBStereo::ShapeZ;
       else
-      {
-        //pBond2 has been previously set
-        if(pBond2->IsUp())
-          ud = !ud;
-        if(ud)
-          pBond1->SetDown();
-        else
-          pBond1->SetUp();
-      }
+        ct->specified = false;
+
+      OBCisTransStereo *obct = new OBCisTransStereo(pmol);
+      obct->SetConfig(*ct);
+      pmol->SetData(obct);
+
       //InChI seems to prefer to define double bond stereo using H atoms.
-      //This leads to rather messy SMILES because the hydrogens have to be explicit.
-      //Could try to transfer u/d to non-Hs, but current method of u/d in congugated
-      //molecules makes it more complicated than it should be.
+
       break;
     }
     case INCHI_StereoType_Tetrahedral:
     {
-      OBChiralData* cd = new OBChiralData;
-      cd->Clear();
-      cd->SetAtom4Refs(refs, input);
-      OBAtom* patom = pmol->GetAtom(stereo.central_atom + 1);
-      if(!patom)
-        return false;
-      patom->SetData(cd);
+      OBTetrahedralStereo::Config *ts = new OBTetrahedralStereo::Config;
+      ts->center = stereo.central_atom;
+      ts->from = stereo.neighbor[0];
+      if (ts->from == ts->center) // Handle the case where there are only three neighbours
+        ts->from = OBStereo::ImplicitRef;
+      ts->refs = OBStereo::MakeRefs(stereo.neighbor[1], stereo.neighbor[2],
+                                    stereo.neighbor[3]);
 
       if(stereo.parity==INCHI_PARITY_EVEN)
-          patom->SetClockwiseStereo();
+        ts->winding = OBStereo::Clockwise;
       else if(stereo.parity==INCHI_PARITY_ODD)
-          patom->SetAntiClockwiseStereo();
+        ts->winding = OBStereo::AntiClockwise;
+      else
+        ts->specified = false;
+
+      OBTetrahedralStereo *obts = new OBTetrahedralStereo(pmol);
+      obts->SetConfig(*ts);
+      pmol->SetData(obts);
 
       break;
     }
@@ -327,9 +218,12 @@ bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
       obErrorLog.ThrowError("InChI code", "Unsupported stereo type has been ignored.", obWarning);
     }
   }
-  
+  // Tidy up the stereo chemistry by removing any objects that are not
+  // consistent with OB's symmetry analysis
+  StereoFrom0D(pmol);
+
   pmol->EndModify();
-  FreeStructFromStdINCHI( &out );
+  FreeStructFromINCHI( &out );
   return true;
 }
 
@@ -352,7 +246,8 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   //Although the OBMol may be altered, it is restored before exit.
   OBMol* pmol = dynamic_cast<OBMol*>(pOb);
   if(pmol==NULL) return false;
-  
+  if(pmol->NumAtoms()==0) return true; // PR#2864334
+
   OBMol& mol = *pmol;
 
   stringstream molID;
@@ -365,20 +260,14 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
   inchi_Input inp;
   memset(&inp,0,sizeof(inchi_Input));
-  bool Is0D=true;
-  if(mol.GetDimension()!=3)
-  {
-    mol.FindChiralCenters();
-    if(mol.GetDimension()==2)
-    {
-      //Add pseudo z coordinates for wedge and hash bonds
-      FOR_ATOMS_OF_MOL(a,mol)
-      {
-        if(a->IsChiral())
-          CalcSignedVolume(mol, &*a, false);
-      }
-    }
-  }
+
+  // Prepare stereo information for 2D, 3D
+  map<OBBond*, OBStereo::BondDirection> updown;
+  map<OBBond*, OBStereo::Ref> from;
+  map<OBBond*, OBStereo::Ref>::const_iterator from_cit;
+  if (mol.GetDimension() == 3 || (mol.GetDimension()==2 && pConv->IsOption("s", pConv->OUTOPTIONS)!=NULL))
+    TetStereoToWedgeHash(mol, updown, from);
+  set<OBBond*> unspec_ctstereo = GetUnspecifiedCisTrans(mol);
 
   OBAtom* patom;
   vector<inchi_Atom> inchiAtoms(mol.NumAtoms());
@@ -392,32 +281,71 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     iat.x = patom->GetX();
     iat.y = patom->GetY();
     iat.z = patom->GetZ();
-    if(iat.x!=0 || iat.y!=0 || iat.z!=0)
-      Is0D=false;
-    
+
     int nbonds = 0;
-    vector<OBEdgeBase*>::iterator itr;
+    vector<OBBond*>::iterator itr;
     OBBond *pbond;
     for (pbond = patom->BeginBond(itr);pbond;pbond = patom->NextBond(itr))
     {
-      // do each bond only once. Seems necessary to avoid problems with stereo
-      if(pbond->GetNbrAtomIdx(patom)<patom->GetIdx()) continue;
+      from_cit = from.find(pbond);
+      // Do each bond only once. If the bond is a stereobond
+      // ensure that the BeginAtom is the 'from' atom.
+      if( (from_cit==from.end() && patom->GetIdx() != pbond->GetBeginAtomIdx()) ||
+          (from_cit!=from.end() && from_cit->second != patom->GetId()) )
+        continue;
+
       iat.neighbor[nbonds]      = pbond->GetNbrAtomIdx(patom)-1;
       int bo = pbond->GetBO();
       if(bo==5)
         bo=4;
       iat.bond_type[nbonds]     = bo;
-      iat.bond_stereo[nbonds++] = INCHI_BOND_STEREO_NONE; //not used
+
+      OBStereo::BondDirection stereo = OBStereo::NotStereo;
+      if (mol.GetDimension()==2 && pConv->IsOption("s", pConv->OUTOPTIONS)==NULL) {
+        if (pbond->IsWedge())
+          stereo = OBStereo::UpBond;
+        else if (pbond->IsHash())
+          stereo = OBStereo::DownBond;
+        else if (pbond->IsWedgeOrHash())
+          stereo = OBStereo::UnknownDir;
+      } 
+      else if (from_cit!=from.end()) { // It's a stereo bond
+        stereo = updown[pbond];
+      }
+
+      if (mol.GetDimension() != 0) {
+        inchi_BondStereo2D bondstereo2D = INCHI_BOND_STEREO_NONE;
+        if (stereo != OBStereo::NotStereo) {
+          switch (stereo) {
+            case OBStereo::UpBond:
+              bondstereo2D = INCHI_BOND_STEREO_SINGLE_1UP;
+              break;
+            case OBStereo::DownBond:
+              bondstereo2D = INCHI_BOND_STEREO_SINGLE_1DOWN;
+              break;
+            case OBStereo::UnknownDir:
+              bondstereo2D = INCHI_BOND_STEREO_SINGLE_1EITHER;
+              break;
+            default:
+              ; // INCHI_BOND_STEREO_NONE
+          }
+        }
+        // Is it a double bond with unspecified stereochemistry?
+        if (unspec_ctstereo.find(pbond)!=unspec_ctstereo.end())
+          bondstereo2D = INCHI_BOND_STEREO_DOUBLE_EITHER;
+        iat.bond_stereo[nbonds] = bondstereo2D;
+      }
+      nbonds++;
     }
-  
+
     strcpy(iat.elname,etab.GetSymbol(patom->GetAtomicNum()));
     iat.num_bonds = nbonds;
     //Let inchi add implicit Hs unless element is H
-    iat.num_iso_H[0] = *iat.elname=='H' ? 0 : -1; 
+    iat.num_iso_H[0] = *iat.elname=='H' ? 0 : -1;
     if(patom->GetIsotope())
     {
       iat.isotopic_mass = ISOTOPIC_SHIFT_FLAG +
-        patom->GetIsotope() - (int)(isotab.GetExactMass(patom->GetAtomicNum())+0.5);
+        patom->GetIsotope() - (int)(etab.GetMass(patom->GetAtomicNum())+0.5);
     }
     else
       iat.isotopic_mass = 0 ;
@@ -430,127 +358,83 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     }
     iat.charge  = patom->GetFormalCharge();
   }
-  
+
   inp.atom = &inchiAtoms[0];
 
-  //Restore zero z coordinate which may have ben modified for chiral 2D molecules
-  if(mol.GetDimension()==2)
-  {
-    FOR_ATOMS_OF_MOL(a,mol)
-    {
-      a->SetVector(a->x(), a->y(), 0.0);
-    }
-  }
-
   vector<inchi_Stereo0D> stereoVec;
-  
-  if(Is0D)
-  {
-    //Tetrahedral stereo
-//    mol.FindChiralCenters(); done above
-    OBAtom* patom;
-    vector<OBNodeBase*>::iterator itr;
-    if(mol.IsChiral())
-    {
-      for(patom = mol.BeginAtom(itr);patom;patom = mol.NextAtom(itr))
-      {
-        if(patom->IsChiral())
-        {
-          inchi_Stereo0D stereo;
-          stereo.central_atom = patom->GetIdx()-1;
-          stereo.type = INCHI_StereoType_Tetrahedral;
-          OBChiralData* cd=(OBChiralData*)patom->GetData(OBGenericDataType::ChiralData);
-          if (!cd) {
-            obErrorLog.ThrowError(__FUNCTION__, molID.str()+": Tetrahedral stereo info not available", obInfo);
-            break;
-          }
-            
-          vector<unsigned int>refs = cd->GetAtom4Refs(input);
-          if(refs.size()<4)
-          {
-            obErrorLog.ThrowError(__FUNCTION__, molID.str()+": Tetrahedral stereo info not available", obInfo);
-            break;
-          }
-          for(int i=0;i<4; ++i)
-            stereo.neighbor[i] = refs[i]-1;
 
-          stereo.parity = INCHI_PARITY_UNKNOWN;
-          if(patom->IsPositiveStereo() || patom->IsClockwise())
+  if(mol.GetDimension()==0)
+  {
+    std::vector<OBGenericData*>::iterator data;
+    std::vector<OBGenericData*> stereoData = mol.GetAllData(OBGenericDataType::StereoData);
+    for (data = stereoData.begin(); data != stereoData.end(); ++data) {
+      if (static_cast<OBStereoBase*>(*data)->GetType() == OBStereo::Tetrahedral) {
+        OBTetrahedralStereo *ts = dynamic_cast<OBTetrahedralStereo*>(*data);
+        OBTetrahedralStereo::Config config = ts->GetConfig();
+
+        if(config.specified) {
+          inchi_Stereo0D stereo;
+          stereo.type = INCHI_StereoType_Tetrahedral;
+          stereo.central_atom = static_cast<AT_NUM> (config.center);
+
+          bool has_implicit = false; // Does chirality involve implicit lone pair?
+          if (config.from == OBStereo::ImplicitRef || config.refs[0] == OBStereo::ImplicitRef) {
+            has_implicit = true;
+            config = ts->GetConfig(OBStereo::ImplicitRef); // Make the 'from' atom the lone pair
+          }
+
+          if (!has_implicit)
+            stereo.neighbor[0] = static_cast<AT_NUM> (config.from);
+          else
+            stereo.neighbor[0] = stereo.central_atom;
+          for(int i=0; i<3; ++i)
+            stereo.neighbor[i + 1] = static_cast<AT_NUM> (config.refs[i]);
+
+          if (config.winding == OBStereo::Clockwise)
             stereo.parity = INCHI_PARITY_EVEN;
-          if(patom->IsNegativeStereo() || patom->IsAntiClockwise())
+          else
             stereo.parity = INCHI_PARITY_ODD;
+
           stereoVec.push_back(stereo);
         }
       }
     }
-    
-    //Double bond stereo
+
+    //Double bond stereo (still inside 0D section)
     //Currently does not handle cumulenes
-    vector<OBBond*> UpDown;
-    vector<OBBond*>::iterator uditr;
-    OBBond *pbond;
-    vector<OBEdgeBase*>::iterator bitr;
-    for (pbond = mol.BeginBond(bitr);pbond;pbond = mol.NextBond(bitr))
-    {
-     if(pbond->IsUp() || pbond->IsDown())
-       UpDown.push_back(pbond);
-    }
-    if(!UpDown.empty())
-    {
-      //For all double bonds look for up/down bonds attached
-      for (pbond = mol.BeginBond(bitr);pbond;pbond = mol.NextBond(bitr))
-      {
-        if(pbond->IsDouble())
-        {
-          OBAtom* patom, *pA=NULL, *pB=NULL;
-          OBBond* pX, *pY;;
-          for(uditr=UpDown.begin();uditr!=UpDown.end();++uditr)
-          {
-            patom = GetCommonAtom(pbond, *uditr);
-            if(patom && !pA)
-            {
-              //first one in pA
-              pA = patom;
-              pX = *uditr;
-            }
-            else if(patom && pA)
-            {
-              //second one in pB
-              pB = patom;
-              pY = *uditr;
-              break;
-            }
-          }
-          if(pA && pB)
-          {
-            inchi_Stereo0D stereo;
-            stereo.central_atom = NO_ATOM;
-            stereo.type = INCHI_StereoType_DoubleBond;
-            stereo.neighbor[0]= pX->GetNbrAtomIdx(pA)-1;
-            stereo.neighbor[1] = pA->GetIdx()-1;
-            stereo.neighbor[2] = pB->GetIdx()-1;
-            stereo.neighbor[3]= pY->GetNbrAtomIdx(pB)-1;
-            if((pX->IsUp() && pY->IsUp())||(pX->IsDown() && pY->IsDown()))
-            {
-              if((pX->GetNbrAtom(pA))->HasDoubleBond())
-                stereo.parity = INCHI_PARITY_EVEN;
-              else
-                stereo.parity = INCHI_PARITY_ODD;
-            }
-            else
-            {
-              if((pX->GetNbrAtom(pA))->HasDoubleBond())
-                stereo.parity = INCHI_PARITY_ODD;
-              else
-                stereo.parity = INCHI_PARITY_EVEN;
-            }
-            stereoVec.push_back(stereo);
-          }
+    for (data = stereoData.begin(); data != stereoData.end(); ++data) {
+      if (static_cast<OBStereoBase*>(*data)->GetType() == OBStereo::CisTrans) {
+        OBCisTransStereo *ts = dynamic_cast<OBCisTransStereo*>(*data);
+        OBCisTransStereo::Config config = ts->GetConfig();
+
+        if(config.specified) {
+          inchi_Stereo0D stereo;
+          stereo.central_atom = NO_ATOM;
+          stereo.type = INCHI_StereoType_DoubleBond;
+          OBStereo::Refs refs = config.refs;
+          unsigned long start = refs[0];
+          if (refs[0]==OBStereo::ImplicitRef)
+            start = refs[1];
+          unsigned long end = refs[3];
+          if (refs[3]==OBStereo::ImplicitRef)
+            end = refs[2];
+
+          stereo.neighbor[0] = static_cast<AT_NUM> (start);
+          stereo.neighbor[1] = static_cast<AT_NUM> (config.begin);
+          stereo.neighbor[2] = static_cast<AT_NUM> (config.end);
+          stereo.neighbor[3] = static_cast<AT_NUM> (end);
+
+          if (ts->IsTrans(start, end))
+            stereo.parity = INCHI_PARITY_EVEN;
+          else
+            stereo.parity = INCHI_PARITY_ODD;
+
+          stereoVec.push_back(stereo);
         }
       }
     }
   }
-  
+
   char* opts = GetInChIOptions(pConv, false);
   inp.szOptions = opts;
 
@@ -562,7 +446,7 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   inchi_Output inout;
   memset(&inout,0,sizeof(inchi_Output));
 
-  int ret = GetStdINCHI(&inp, &inout);
+  int ret = GetINCHI(&inp, &inout);
 
   delete[] opts;
   if(ret!=inchi_Ret_OKAY)
@@ -570,14 +454,15 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     if(inout.szMessage)
     {
       string mes(inout.szMessage);
-      if(pConv->IsOption("w")) 
+      if(pConv->IsOption("w"))
       {
         string::size_type pos;
-        string targ[3];
+        string targ[4];
         targ[0] = "Omitted undefined stereo";
         targ[1] = "Charges were rearranged";
         targ[2] = "Proton(s) added/removed";
-        for(int i=0;i<3;++i)
+        targ[3] = "Metal was disconnected";
+        for(int i=0;i<4;++i)
         {
           pos = mes.find(targ[i]);
           if(pos!=string::npos)
@@ -600,20 +485,30 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       return false;
     }
   }
-  
+
   string ostring = inout.szInChI;
+
+  //Truncate the InChI if requested
+  const char* truncspec = pConv->IsOption("T");
+  if(truncspec)
+  {
+    string trunc(truncspec);
+    EditInchi(ostring, trunc);
+  }
+
   if(pConv->IsOption("K")) //Generate InChIKey and add after InChI on same line
   {
     char szINCHIKey[28];
     GetStdINCHIKeyFromStdINCHI(inout.szInChI, szINCHIKey);
     ostring = szINCHIKey;
   }
+
   if(pConv->IsOption("t"))
   {
     ostring += ' ';
     ostring +=  mol.GetTitle();
   }
-    
+
   ostream &ofs = *pConv->GetOutStream();
 
   if(pConv->IsOption("U"))
@@ -622,7 +517,7 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       allInchi.clear();
     //Just add to set and don't output, except at the end
     allInchi.insert(ostring);
-    
+
     if(pConv->IsLast())
     {
       nSet::iterator itr;
@@ -655,7 +550,7 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     else
     {
       ofs << "Molecules " << firstID << "and " << molID.str();
-      ofs << InChIErrorMessage(CompareInchi(firstInchi.c_str(), inout.szInChI)) << endl; 
+      ofs << InChIErrorMessage(CompareInchi(firstInchi.c_str(), inout.szInChI)) << endl;
     }
   }
 
@@ -664,12 +559,22 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 }
 
 //////////////////////////////////////////////////////////
-char InChIFormat::CompareInchi(const char* Inchi1, const char* Inchi2)
+char InChIFormat::CompareInchi(const string& Inchi1, const string& Inchi2)
 {
   //Returns 0 if identical or an char identifying the layer where they first differed
   string s1(Inchi1), s2(Inchi2);
 
-  //Remove anything after the end of the Inchi
+  if(s1.size()<s2.size())
+    s1.swap(s2);
+  string::size_type pos;
+  for(pos=0;pos<s1.size();++pos)
+  {
+    if(pos==s2.size() || s1[pos]!=s2[pos])
+      return s1[s1.rfind('/',pos)+1];
+  }
+  return 0;
+
+/*  //Remove anything after the end of the Inchi
   string::size_type pos;
   pos = s1.find_first_of(" \t\n");
   if(pos!=string::npos)
@@ -684,7 +589,7 @@ char InChIFormat::CompareInchi(const char* Inchi1, const char* Inchi2)
   unsigned int i;
   if(layers1.size()<layers2.size())
     layers1.swap(layers2); //layers1 is the longest
-  
+
   for(i=1;i<layers2.size();++i)
   {
     if(layers1[i]!=layers2[i])
@@ -695,16 +600,17 @@ char InChIFormat::CompareInchi(const char* Inchi1, const char* Inchi2)
       return ch;
     }
   }
-  if(layers1.size()==layers2.size())	
+  if(layers1.size()==layers2.size())
     return 0;
   else
     return layers1[i][0];
+*/
 }
 
 string InChIFormat::InChIErrorMessage(const char ch)
 {
   string s;
-  switch (ch) 
+  switch (ch)
   {
   case 0:
     s = " are identical";
@@ -751,180 +657,6 @@ OBAtom* InChIFormat::GetCommonAtom(OBBond* pb1, OBBond* pb2)
   return NULL; //not adjacent bonds
 }
 
-//******************************************************
-class InChICompareFormat : public OBMoleculeFormat
-{
-public:
-  InChICompareFormat()
-  {
-      OBConversion::RegisterFormat("k",this);
-  }
-
-  virtual const char* Description() //required
-  {
-      return 
-"Compare molecules using InChI\n"
-"The first molecule is compared with the rest\n"
-"e.g. babel first.smi second.mol third.cml -ok\n"
-"Same as  -oinchi -xet  and can take the same options as InChI format.\n"
-;
-  };
-  
-  virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv)
-  {
-    pConv->AddOption("e",OBConversion::OUTOPTIONS);
-    pConv->AddOption("t",OBConversion::OUTOPTIONS);
-    return theInChIFormat.WriteMolecule(pOb,pConv);
-  };
-  
-  virtual unsigned int Flags() { return NOTREADABLE;};
-};
-
-InChICompareFormat theInChICompareFormat;
-
-//*********************************************************
-class TestFormat : public OBMoleculeFormat
-{
-public:
-  //Register this format type ID
-  TestFormat() 
-  {
-    OBConversion::RegisterFormat("test",this);
-    OBConversion::RegisterOptionParam("O", this, 1);
-    OBConversion::RegisterOptionParam("m", this);
-  }
-
-  virtual const char* Description()
-  { return
-"Test format\n"
-"Does a round trip conversion and compares before and after using InChI.\n"
-"Uses the input format unless -xO set\n"
-"Read/Write Option e.g. -xOsmi\n"
-" O<ext> Test the format that has the specified ID\n"
-" m      Output message for each successful molecule\n"
-" X <Option string> List of InChI options:\n\n"
-
-"You can also use commandline options appropriate for the target format.\n\n"
-
-"InputFormat => OBMol => SpecifiedFormat => OBMol\n"
-"Compare InChIs generated from each of the OBMols\n"
-;
-  };
-
-  virtual unsigned int Flags() { return NOTREADABLE;}
-  virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
-
-};
-/////////////////////////////////////////////////////////////////
-TestFormat theTestFormat;
-/////////////////////////////////////////////////////////////////
-bool TestFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
-{
-  OBMol* pmol = dynamic_cast<OBMol*>(pOb);
-
-  ostream &ofs = *pConv->GetOutStream();
-  OBMol &mol = *pmol;
-  istream* origInStream = pConv->GetInStream(); 
-  ostream* origOutStream = pConv->GetOutStream(); 
-  stringstream ssinchi1, ssinchi2;
-  
-  static int nMols;
-  static int nFailures;
-  if(pConv->GetOutputIndex()==1)
-    nMols=nFailures=0;
-  nMols++;
-
-  //mol has already been input using the input format
-
-  //Send its InChI to a stringstream
-  OBFormat* pInchi = OBConversion::FindFormat("inchi");
-  if(!pInchi)
-  {	
-    obErrorLog.ThrowError(__FUNCTION__, "InChIFormat needs to be installed to use TestFormat", obWarning);
-    return false;
-  }
-  pConv->AddOption("w",OBConversion::OUTOPTIONS);//no trivial warnings
-  pConv->SetOutFormat(pInchi);
-  if(!pConv->Write(pmol,&ssinchi1))
-    return false;
-
-  //Use a new OBConversion to write and then read using the target format
-  //(Using the same OBConversion gave problems with CMLFormat)
-  OBConversion NewConv(*pConv); //copy
-
-
-  const char* pTargetExt = pConv->IsOption("O");
-#ifdef HAVE_SSTREAM
-  stringstream errorMsg;
-#else
-  strstream errorMsg;
-#endif
-  if(pTargetExt)
-  {
-    OBFormat* pTargetFormat = OBConversion::FindFormat(pTargetExt);
-    if(!pTargetFormat)
-    {
-      errorMsg << pTargetExt <<  " format is not available" << endl;
-      obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obError);
-      return false;
-    }
-    if(!NewConv.SetInFormat(pTargetFormat))
-    {
-      errorMsg << pTargetExt << " format being tested needs to be readable" << endl;
-      obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obError);
-      return false;
-    }
-  }
-
-  if(!NewConv.SetOutFormat(NewConv.GetInFormat()))
-  {
-    errorMsg << "The input format being tested needs also to be writeable" << endl;
-    obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obError);
-    return false;
-  }
-  //Output with the target format to a stringstream ss
-  stringstream ss;
-  NewConv.SetOneObjectOnly();
-  NewConv.SetInStream(&ss); //Quirk in CMLFormat: needs to haveinput stream set before writing
-  if(!NewConv.Write(pmol,&ss))
-    return false;
-
-  //Read it back in again using the target format
-  OBMol Remol;
-  if(!NewConv.Read(&Remol))
-    return false;
-
-  //Take the InChI of the reconverted molecule
-  pConv->SetOutFormat(pInchi);
-  if(!pConv->Write(&Remol,&ssinchi2))
-    return false;
-  
-  pConv->SetInStream(origInStream);
-  pConv->SetOutStream(origOutStream);
-  pConv->SetOutFormat(this);
-
-  char ch = InChIFormat::CompareInchi(ssinchi1.str().c_str(), ssinchi2.str().c_str());
-
-  if(ch || pConv->IsOption("m"))//On failure, or anyway if specified
-  {
-    stringstream molID;
-    if(strlen(mol.GetTitle())==0)
-      molID << "Mol #" << nMols;
-    else
-      molID << mol.GetTitle();
-    ofs << molID.str() << " in " << pConv->GetInFilename();
-    ofs << " and its conversion" << InChIFormat::InChIErrorMessage(ch) << endl;
-    if(ch)
-      ++nFailures;
-  }
-
-  if(pConv->IsLast())
-  {
-   char s = nFailures==1 ? ' ' : 's';
-   ofs << '\n' << nFailures << " failure" << s << endl;
-  }
-  return true;
-}
 
 //Returns pointer to InChI options string, which needs to be deleted with delete[]
 //If there are no options returns an empty string
@@ -947,7 +679,6 @@ char* InChIFormat::GetInChIOptions(OBConversion* pConv, bool Reading)
     copy(useropts.begin(), useropts.end(), back_inserter(optsvec));
   }
 
-  /* In Standard InChI these are not used
   //Add a couple InChI options built in to OB
   if(opttyp==OBConversion::OUTOPTIONS)
   {
@@ -962,7 +693,7 @@ char* InChIFormat::GetInChIOptions(OBConversion* pConv, bool Reading)
       optsvec.push_back(tmp2);
     }
   }
-  */
+
 
 #ifdef WIN32
     string ch(" /");
@@ -980,4 +711,69 @@ char* InChIFormat::GetInChIOptions(OBConversion* pConv, bool Reading)
   return opts;
 }
 
+bool InChIFormat::EditInchi(std::string& inchi, std::string& spec)
+{
+  std::vector<std::string> vec;
+  std::vector<std::string>::iterator itr;
+  tokenize(vec, spec, " \t/");
+  for(itr=vec.begin();itr!=vec.end();++itr)
+  {
+    if(*itr=="formula")
+    {
+      std::string::size_type pos = inchi.find('/', inchi.find('/')+1); //2nd /
+      if(pos!=string::npos)
+        inchi.erase(pos);
+    }
+    else if(*itr=="connect")
+      RemoveLayer(inchi,"/h",true);
+    else if(*itr=="nochg")
+    {
+      RemoveLayer(inchi,"/p");
+      RemoveLayer(inchi,"/q");
+    }
+    else if(*itr=="nosp3")
+    {
+      RemoveLayer(inchi,"/t");
+      RemoveLayer(inchi,"/m");
+      RemoveLayer(inchi,"/s");
+    }
+    else if(*itr=="noEZ")
+      RemoveLayer(inchi,"/b");
+    else if(*itr=="noiso")
+      RemoveLayer(inchi,"/i");
+    else if(*itr=="nostereo")
+    {
+      RemoveLayer(inchi,"/t");
+      RemoveLayer(inchi,"/m");
+      RemoveLayer(inchi,"/s");
+      RemoveLayer(inchi,"/b");
+    }
+    else if(!(*itr).empty())
+    {
+      obErrorLog.ThrowError(__FUNCTION__,
+      spec + " not recognized as a truncation specification",obError, onceOnly);
+      return false;
+    }
+  }
+  return true;
+}
+
+void InChIFormat::RemoveLayer (std::string& inchi, const std::string& str, bool all)
+{
+  std::string::size_type pos = inchi.find(str);
+  if(pos!=string::npos)
+    inchi.erase(pos, (all ? string::npos : inchi.find('/', pos+1) - pos));
+}
+
+
+//************************************************************************
+InChICompareFormat theInChICompareFormat;
+
+bool InChICompareFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
+{
+  pConv->AddOption("e",OBConversion::OUTOPTIONS);
+  pConv->AddOption("t",OBConversion::OUTOPTIONS);
+  return theInChIFormat.WriteMolecule(pOb,pConv);
+}
+
 }//namespace OpenBabel
