@@ -1,17 +1,18 @@
 /**********************************************************************
 Copyright (C) 2002 by Steffen Reith <streit@streit.cc>
-Some portions Copyright (C) 2003-2005 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2003-2006 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2004 by Chris Morley
- 
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
+#include <openbabel/babelconfig.h>
 
 /* ---- C includes ---- */
 #include <math.h>
@@ -19,17 +20,13 @@ GNU General Public License for more details.
 #include <stdlib.h>
 
 /* ---- OpenBabel include ---- */
-#include "babelconfig.h"
-#include "mol.h"
-#include "obconversion.h"
+#include <openbabel/obmolecformat.h>
+#include <openbabel/math/vector3.h>
 
 /* ---- C++ includes ---- */
 #include <string>
-#if defined(HAVE_SSTREAM)
 #include <sstream>
-#else
-#include <strstream>
-#endif
+#include <cctype>
 
 /* ---- Max. length of a atom-label ---- */
 #define StrLen 32
@@ -65,118 +62,86 @@ GNU General Public License for more details.
 /* ---- Convert RAD to DEG ---- */
 #define RAD2DEG(r) (((double) 180.0 * r) / PI)
 
+//! \return the geometric centroid to an array of coordinates in double* format
+// Note: Based on center_coords from obutil.cpp. This function is declared in matrix3x3.h but there is no function
+//       definition there.
+OpenBabel::vector3 my_center_coords(double *c, unsigned int size)
+{
+  if (size == 0)
+    {
+      return(OpenBabel::VZero);
+    }
+	unsigned int i;
+  double x=0.0, y=0.0, z=0.0;
+  for (i = 0;i < size;++i)
+    {
+      x += c[i*3];
+      y += c[i*3+1];
+      z += c[i*3+2];
+    }
+  x /= (double) size;
+  y /= (double) size;
+  z /= (double) size;
+  OpenBabel::vector3 v(x,y,z);
+  return(v);
+}
+
+
 using namespace std;
 namespace OpenBabel
 {
 
-class PovrayFormat : public OBFormat
-{
-public:
+  class PovrayFormat : public OBMoleculeFormat
+  {
+  public:
     //Register this format type ID
     PovrayFormat()
     {
-        OBConversion::RegisterFormat("pov",this);
+      OBConversion::RegisterFormat("pov",this);
     }
 
     virtual const char* Description() //required
     {
-        return
-            "POV-Ray input format\n \
-            No comments yet\n";
+      return
+      "POV-Ray input format\n"
+      "Generate an input file for the open source POV-Ray ray tracer.\n"
+      "Write Options e.g. -xt\n"
+      " c Add a black and white checkerboard\n"
+      " f Add a mirror sphere\n"
+      " m <model-type> BAS (ball-and-stick), SPF (space-fill) or CST (capped sticks) \n"
+      "    The default option is ball-and-stick. To choose space-fill, you would use the following command line::\n\n"
+      "      obabel aspirin.mol -O aspirin.pov -xm SPF\n\n"
+      " s Add a sky (with clouds)\n"
+      " t Use transparent textures\n"
+      ;
     };
 
-  virtual const char* SpecificationURL()
-  {return "http://www.povray.org/";}; //optional
+    virtual const char* SpecificationURL()
+    {return "http://www.povray.org/";}; //optional
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
     virtual unsigned int Flags()
     {
-        return NOTREADABLE | WRITEONEONLY;
+      return NOTREADABLE | WRITEONEONLY;
     };
 
     ////////////////////////////////////////////////////
     /// The "API" interface functions
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
 
-    ////////////////////////////////////////////////////
-    /// The "Convert" interface functions
-    virtual bool WriteChemObject(OBConversion* pConv)
-    {
-        //Retrieve the target OBMol
-        OBBase* pOb = pConv->GetChemObject();
-        OBMol* pmol = dynamic_cast<OBMol*> (pOb);
-        bool ret=false;
-        if(pmol)
-            ret=WriteMolecule(pmol,pConv);
+  private:
+    string model_type;
+    bool sky, sphere, trans_texture, checkerboard;
+    void OutputHeader(ostream &ofs, OBMol &mol, string prefix);
 
-	std::string auditMsg = "OpenBabel::Write molecule ";
-	std::string description(Description());
-        auditMsg += description.substr( 0, description.find('\n') );
-        obErrorLog.ThrowError(__FUNCTION__,
-                              auditMsg,
-                              obAuditMsg);
-        delete pOb;
-        return ret;
-    };
-};
+  };
 
-//Make an instance of the format class
-PovrayFormat thePovrayFormat;
+  //Make an instance of the format class
+  PovrayFormat thePovrayFormat;
 
-/* ---- Make a prefix from title of molecule ---- */
-string MakePrefix(const char* title)
-{
-    int pos; /* Position in prefix */
-    char *titleCpy = (char*) NULL;
-    char *str = (char*) NULL;
-    char *last = (char*) NULL;
-
-    /* ---- Copy molecule title to 'str' ---- */
-    if ((titleCpy = strdup(title)) == (char *) NULL)
-        return string("NoMemory");
-
-    /* --- Find last '/' and set 'str' to it if needed ----- */
-    last = strrchr(titleCpy, '/');
-    if (last != (char *) NULL)
-        str = (last + 1);
-    else
-        str = titleCpy;
-
-    /* ---- Check for nonempty string ---- */
-    if (strlen(str) == 0)
-        return string("InValid");
-
-    /* ---- Look for first . and replace with \0 ----- */
-    pos = 0;
-    while((str[pos] != '\0') && (str[pos] != '.'))
-    {
-
-        /* ---- Remove all tabs and spaces ---- */
-        if ((str[pos] == ' ') || (str[pos] == '\t'))
-            str[pos] = '_';
-
-        /* ---- Check next position ---- */
-        pos++;
-
-    }
-
-    /* ---- If we have found a '.' cut the string there ---- */
-    str[pos] = '\0';
-
-    /* ---- Cast to C++ string-type the above operations are a mess with C++ strings ---- */
-    string prefix(str);
-
-    /* ---- Free allocated memory ---- */
-    free(titleCpy);
-
-    /* ---- Return the prefix ---- */
-    return prefix;
-
-}
-
-void OutputHeader(ostream &ofs, OBMol &mol, string prefix)
-{
+  void PovrayFormat::OutputHeader(ostream &ofs, OBMol &mol, string prefix)
+  {
     time_t akttime;                              /* Systemtime                        */
     char timestr[TIME_STR_SIZE + 1] = "";        /* Timestring                        */
     size_t time_res;                             /* Result of strftime                */
@@ -187,22 +152,105 @@ void OutputHeader(ostream &ofs, OBMol &mol, string prefix)
                         TIME_STR_SIZE,
                         "%a %b %d %H:%M:%S %Z %Y",
                         localtime((time_t *) &akttime)
-                       );
+                        );
 
     /* ---- Write some header information ---- */
-    ofs << "//Povray V3.1 code generated by Open Babel" << endl;
+    ofs << "//Povray v3 code generated by Open Babel" << endl;
     ofs << "//Author: Steffen Reith <streit@streit.cc>" << endl;
+    ofs << "//Update (2010): Noel O'Boyle and Steven Wathen" << endl;
 
     /* ---- Include timestamp in header ---- */
     ofs << "//Date: " << timestr << endl << endl;
 
+    /* ---- Set configurable global settings ---- */
+    ofs << "//Set some global parameters for display options" << endl;
+    ofs << "#declare " << model_type << " = true;" << endl;
+    string trans_tex_setting = trans_texture ? "true" : "false";
+    ofs << "#declare TRANS = " << trans_tex_setting << ";" << endl << endl;
+
+    /* ---- Background, camera and light source ---- */
+    OpenBabel::vector3 centroid = my_center_coords(mol.GetCoordinates(), mol.NumAtoms());
+
+    ofs << "#include \"colors.inc\"\n" << endl;
+
+    ofs <<  "// create a regular point light source\n"
+            "light_source {\n"
+            "  <" << centroid.x()  + 2.0 << "," << centroid.y() + 3.0 << "," << centroid.z() - 8.0 << ">\n"
+            "  color rgb <1,1,1>    // light's color\n"
+            "}\n" << endl;
+
+    if (sky) {
+      ofs << "// Add some nice sky with clouds\n"
+"sky_sphere {\n"
+"    pigment {\n"
+"      gradient y\n"
+"      color_map {\n"
+"        [0.0 1.0 color SkyBlue  color NavyBlue]\n"
+"      }\n"
+"      scale 2\n"
+"      translate -1\n"
+"    }\n"
+"    pigment {\n"
+"      bozo\n"
+"      turbulence 0.65\n"
+"      octaves 6\n"
+"      omega 0.7\n"
+"      lambda 2\n"
+"      color_map {\n"
+"          [0.0 0.1 color rgb <0.85, 0.85, 0.85>\n"
+"                   color rgb <0.75, 0.75, 0.75>]\n"
+"          [0.1 0.5 color rgb <0.75, 0.75, 0.75>\n"
+"                   color rgbt <1, 1, 1, 1>]\n"
+"          [0.5 1.0 color rgbt <1, 1, 1, 1>\n"
+"                   color rgbt <1, 1, 1, 1>]\n"
+"      }\n"
+"      scale <0.2, 0.5, 0.2>\n"
+"    }\n"
+"    rotate -135*x\n"
+"  }\n" << endl;
+    }
+    else { // Simple background
+      ofs << "// set a color of the background (sky)" << endl;
+      ofs << "background { color rgb <0.95 0.95 0.95> }\n" << endl;
+    }
+
+    ofs <<  "// perspective (default) camera\n"
+            "camera {\n"
+            "  location  <" << centroid.x() << "," << centroid.y() << "," << centroid.z() - 10.0 << ">\n"
+            "  look_at   <" << centroid.x() << "," << centroid.y() << "," << centroid.z() << ">\n"
+            "  right     x*image_width/image_height\n"
+            "}\n" << endl;
+
+    /* ---- Checkerboard and mirror sphere ---- */
+    if (sphere) {
+      ofs <<
+"// a mirror sphere\n"
+"sphere\n"
+"{ <" << centroid.x()  + 8.0 << "," << centroid.y() - 4 << "," << centroid.z() + 8.0 << ">,4\n"
+"  pigment { rgb <0,0,0> } // A perfect mirror with no color\n"
+"  finish { reflection 1 } // It reflects all\n"
+"}\n" << endl;
+    }
+    if (checkerboard) {
+      ofs <<
+"// simple Black on White checkerboard... it's a classic\n"
+"plane {\n"
+" -y, " << -(centroid.y()-8) << "\n"
+" pigment {\n"
+"  checker color Black color White\n"
+"  scale 2\n"
+" }\n"
+"}\n" << endl;
+    }
+
+
     /* ---- Include header statement for babel ---- */
     ofs << "//Include header for povray" << endl;
-    ofs << "#include \"babel31.inc\"" << endl << endl;
+    ofs << "#include \"babel_povray3.inc\"" << endl << endl;
 
     /* ---- You should do a spacefill model for molecules without bonds ---- */
     if (mol.NumBonds() == 0)
-    {
+      {
 
         /* ---- Check if a spacefill-model is selected ---- */
         ofs << "#if (BAS | CST)\"" << endl;
@@ -210,24 +258,24 @@ void OutputHeader(ostream &ofs, OBMol &mol, string prefix)
         ofs << "#warning \"You should do a spacefill-model\"" << endl;
         ofs << "#end" << endl << endl;
 
-    }
+      }
 
     /* ---- Set version ---- */
-    ofs << "//Use PovRay3.1" << endl;
-    ofs << "#version 3.1;" << endl << endl;
+    ofs << "//Use PovRay3.6" << endl;
+    ofs << "#version 3.6;" << endl << endl;
 
     /* ---- Print of name of molecule (#\b depends on size of babel.inc!) ---- */
     ofs << "//Print name of molecule while rendering" << endl;
     ofs << "#render \"\\b\\b " << mol.GetTitle() << "\\n\\n\"" << endl << endl;
 
-}
+  }
 
-void CalcBoundingBox(OBMol &mol,
-                     double &min_x, double &max_x,
-                     double &min_y, double &max_y,
-                     double &min_z, double &max_z
-                    )
-{
+  void CalcBoundingBox(OBMol &mol,
+                       double &min_x, double &max_x,
+                       double &min_y, double &max_y,
+                       double &min_z, double &max_z
+                       )
+  {
     /* ---- Init bounding-box variables ---- */
     min_x = (double) 0.0;
     max_x = (double) 0.0;
@@ -238,57 +286,57 @@ void CalcBoundingBox(OBMol &mol,
 
     /* ---- Check all atoms ---- */
     for(unsigned int i = 1; i <= mol.NumAtoms(); ++i)
-    {
+      {
 
         /* ---- Get a pointer to ith atom ---- */
         OBAtom *atom = mol.GetAtom(i);
 
         /* ---- Check for minimal/maximal x-position ---- */
         if (atom -> GetX() < min_x)
-            min_x = atom -> GetX();
+          min_x = atom -> GetX();
         if (atom -> GetX() > max_x)
-            max_x = atom -> GetX();
+          max_x = atom -> GetX();
 
         /* ---- Check for minimal/maximal y-position ---- */
         if (atom -> GetY() < min_y)
-            min_y = atom -> GetY();
+          min_y = atom -> GetY();
         if (atom -> GetY() > max_y)
-            max_y = atom -> GetY();
+          max_y = atom -> GetY();
 
         /* ---- Check for minimal/maximal z-position ---- */
         if (atom -> GetZ() < min_z)
-            min_z = atom -> GetZ();
+          min_z = atom -> GetZ();
         if (atom -> GetZ() > max_z)
-            max_z = atom -> GetZ();
+          max_z = atom -> GetZ();
 
-    }
+      }
 
-}
+  }
 
-void OutputAtoms(ostream &ofs, OBMol &mol, string prefix)
-{
+  void OutputAtoms(ostream &ofs, OBMol &mol, string prefix)
+  {
     /* ---- Write all coordinates ---- */
     ofs << "//Coodinates of atoms 1 - " << mol.NumAtoms() << endl;
     unsigned int i;
     for(i = 1; i <= mol.NumAtoms(); ++i)
-    {
+      {
 
         /* ---- Get a pointer to ith atom ---- */
         OBAtom *atom = mol.GetAtom(i);
 
         /* ---- Write position of atom i ---- */
         ofs << "#declare " << prefix << "_pos_" << i << " = <"
-        << atom -> GetX() << ","
-        << atom -> GetY() << ","
-        << atom -> GetZ()
-        << ">;" << endl;
+            << atom -> GetX() << ","
+            << atom -> GetY() << ","
+            << atom -> GetZ()
+            << ">;" << endl;
 
-    }
+      }
 
     /* ---- Write povray-description of all atoms ---- */
     ofs << endl << "//Povray-description of atoms 1 - " << mol.NumAtoms() << endl;
     for(i = 1; i <= mol.NumAtoms(); ++i)
-    {
+      {
 
         /* ---- Get a pointer to ith atom ---- */
         OBAtom *atom = mol.GetAtom(i);
@@ -296,22 +344,22 @@ void OutputAtoms(ostream &ofs, OBMol &mol, string prefix)
         /* ---- Write full description of atom i ---- */
         ofs << "#declare " << prefix << "_atom" << i << " = ";
         ofs << "object {" << endl
-        << "\t  Atom_" << etab.GetSymbol(atom->GetAtomicNum()) << endl
-        << "\t  translate " << prefix << "_pos_" << i << endl << "\t }" << endl;
+            << "\t  Atom_" << etab.GetSymbol(atom->GetAtomicNum()) << endl
+            << "\t  translate " << prefix << "_pos_" << i << endl << "\t }" << endl;
 
-    }
+      }
 
     /* ---- Add empty line ---- */
     ofs << endl;
 
-}
+  }
 
 
-void OutputBASBonds(ostream &ofs, OBMol &mol, string prefix)
-{
+  void OutputBASBonds(ostream &ofs, OBMol &mol, string prefix)
+  {
     /* ---- Write povray-description of all bonds---- */
     for(unsigned int i = 0; i < mol.NumBonds(); ++i)
-    {
+      {
 
         double x1,y1,z1,x2,y2,z2; /* Start and stop coordinates of a bond       */
         double dist;              /* Distance between (x1|y1|z1) and (x2|y2|z2) */
@@ -339,74 +387,76 @@ void OutputBASBonds(ostream &ofs, OBMol &mol, string prefix)
         phi = (double) 0.0;
         theta = (double) 0.0;
         if (fabs(dist) >= EPSILON)
-            phi = acos((y2-y1)/dist);
+          phi = acos((y2-y1)/dist);
         if (fabs(dy) >= EPSILON)
-            theta = acos((x2-x1)/dy);
+          theta = acos((x2-x1)/dy);
 
         /* ---- Full description of bond i ---- */
         ofs << "#declare " << prefix << "_bond" << i
-        << " = object {" << endl << "\t  bond_" << bond -> GetBondOrder() << endl;
+            << " = object {" << endl << "\t  bond_" << bond -> GetBondOrder() << endl;
 
         /* ---- Scale bond if needed ---- */
         if (fabs(dist) >= EPSILON)
-        {
+          {
 
             /* ---- Print povray scale-statement (x-Axis) ---- */
             ofs << "\t  scale <" << dist << ",1.0000,1.0000>\n";
 
-        }
+          }
 
         /* ---- Rotate (Phi) bond if needed ---- */
         if (fabs(RAD2DEG(-phi) + (double) 90.0) >= EPSILON)
-        {
+          {
 
             /* ---- Rotate along z-axis ---- */
             ofs << "\t  rotate <0.0000,0.0000,"
-            << RAD2DEG(-phi) + (double) 90.0
-            << ">" << endl;
+                << RAD2DEG(-phi) + (double) 90.0
+                << ">" << endl;
 
 
-        }
+          }
 
         /* ---- Check angle between (x1|0|z1) and (x2|0|z2) ---- */
         if (theta >= EPSILON)
-        {
+          {
 
             /* ---- Check direction ---- */
             if ((z2 - z1) >= (double) 0.0)
-            {
+              {
 
                 /* ---- Rotate along y-Axis (negative) ---- */
                 ofs << "\t  rotate <0.0000,"
-                << RAD2DEG((double) -1.0 * theta) << ",0.0000>"
-                << endl;
+                    << RAD2DEG((double) -1.0 * theta) << ",0.0000>"
+                    << endl;
 
-            }
+              }
             else
-            {
+              {
 
                 /* ---- Rotate along y-Axis (positive) ---- */
                 ofs << "\t  rotate <0.0000,"
-                << RAD2DEG(theta) << ",0.0000>"
-                << endl;
+                    << RAD2DEG(theta) << ",0.0000>"
+                    << endl;
 
-            }
+              }
 
-        }
+          }
 
         /* ---- Translate bond to start ---- */
         ofs << "\t  translate " << prefix << "_pos_" << bond -> GetBeginAtomIdx()
-        << endl << "\t }" << endl;
+            << endl << "\t }" << endl;
 
-    }
+      }
 
-}
+  }
+
+  void OutputCSTBonds(ostream &ofs, OBMol &mol, string prefix)
+  {
+    string bond_type;
 
-void OutputCSTBonds(ostream &ofs, OBMol &mol, string prefix)
-{
     /* ---- Write povray-description of all bonds---- */
     for(unsigned int i = 0; i < mol.NumBonds(); ++i)
-    {
+      {
 
         double x1,y1,z1,x2,y2,z2; /* Start and stop coordinates of a bond       */
         double dist;              /* Distance between (x1|y1|z1) and (x2|y2|z2) */
@@ -434,9 +484,9 @@ void OutputCSTBonds(ostream &ofs, OBMol &mol, string prefix)
         phi = (double) 0.0;
         theta = (double) 0.0;
         if (fabs(dist) >= EPSILON)
-            phi = acos((y2-y1)/dist);
+          phi = acos((y2-y1)/dist);
         if (fabs(dy) >= EPSILON)
-            theta = acos((x2-x1)/dy);
+          theta = acos((x2-x1)/dy);
 
         /* ---- Begin of description of bond i (for a capped sticks model) ---- */
         ofs << "#declare " << prefix << "_bond" << i << " = object {" << endl;
@@ -446,53 +496,55 @@ void OutputCSTBonds(ostream &ofs, OBMol &mol, string prefix)
         ofs << "\t   object {" << endl << "\t    bond_" << bond -> GetBondOrder()  << "\n";
 
         /* ---- Add a pigment - statement for start-atom of bond ---- */
+        bond_type = bond->GetBeginAtom() -> GetType();
+        bond_type.erase(remove_if(bond_type.begin(), bond_type.end(), bind1st(equal_to<char>(), '.')));
         ofs << "\t    pigment{color Color_"
-        << bond -> GetBeginAtom() -> GetType()
-        << "}" << endl;
+            << bond_type
+            << "}" << endl;
 
         /* ---- Scale bond if needed ---- */
         if (fabs((double) 2.0 * dist) >= EPSILON)
-        {
+          {
 
             /* ---- Print povray scale-statement (x-Axis) ---- */
             ofs << "\t    scale <" << (double) 0.5 * dist << ",1.0000,1.0000>" << endl;
 
-        }
+          }
 
         /* ---- Rotate (Phi) bond if needed ---- */
         if (fabs(RAD2DEG(-phi) + (double) 90.0) >= EPSILON)
-        {
+          {
 
             /* ---- Rotate along z-axis ---- */
             ofs << "\t    rotate <0.0000,0.0000,"
-            << RAD2DEG(-phi) + (double) 90.0
-            << ">" << endl;
+                << RAD2DEG(-phi) + (double) 90.0
+                << ">" << endl;
 
-        }
+          }
 
         /* ---- Check angle between (x1|0|z1) and (x2|0|z2) ---- */
         if (theta >= EPSILON)
-        {
+          {
 
             /* ---- Check direction ---- */
             if ((z2 - z1) >= (double) 0.0)
-            {
+              {
 
                 /* ---- Rotate along y-Axis (negative) ---- */
                 ofs << "\t    rotate <0.0000,"
-                << RAD2DEG((double) -1.0 *theta) << ",0.0000>"
-                << endl;
+                    << RAD2DEG((double) -1.0 *theta) << ",0.0000>"
+                    << endl;
 
-            }
+              }
             else
-            {
+              {
 
                 /* ---- Rotate along y-Axis (positive) ---- */
                 ofs << "\t    rotate <0.0000," << RAD2DEG(theta) << ",0.0000>" << endl;
 
-            }
+              }
 
-        }
+          }
 
         /* ---- Translate bond to start ---- */
 
@@ -505,54 +557,57 @@ void OutputCSTBonds(ostream &ofs, OBMol &mol, string prefix)
         ofs << "\t   object {" << endl << "\t    bond_" << bond -> GetBondOrder() << endl;
 
         /* ---- Add a pigment - statement for end-atom of bond i ---- */
+        bond_type = bond->GetEndAtom() -> GetType();
+        bond_type.erase(remove_if(bond_type.begin(), bond_type.end(), bind1st(equal_to<char>(), '.')));
+
         ofs << "\t    pigment{color Color_"
-        << bond -> GetEndAtom() -> GetType()
-        << "}" << endl;
+            << bond_type
+            << "}" << endl;
 
         /* ---- Scale bond if needed ---- */
         if (fabs((double) 2.0 * dist) >= EPSILON)
-        {
+          {
 
             /* ---- Print povray scale-statement (x-Axis) ---- */
             ofs << "\t    scale <" << (double) 0.5 * dist << ",1.0000,1.0000>" << endl;
 
-        }
+          }
 
         /* ---- Rotate (Phi) bond if needed ---- */
         if (fabs(RAD2DEG(-phi) + (double) 270.0) >= EPSILON)
-        {
+          {
 
             /* ---- Rotate along z-axis (oposite to start half) ---- */
             ofs << "\t    rotate <0.0000,0.0000,"
-            << (RAD2DEG(-phi) + (double) 90.0) + (double) 180.0
-            << ">" << endl;
+                << (RAD2DEG(-phi) + (double) 90.0) + (double) 180.0
+                << ">" << endl;
 
-        }
+          }
 
         /* ---- Check angle between (x1|0|z1) and (x2|0|z2) ---- */
         if (fabs(theta) >= EPSILON)
-        {
+          {
 
             /* ---- Check direction ---- */
             if ((z2 - z1) >= (double) 0.0)
-            {
+              {
 
                 /* ---- Rotate along y-Axis (negative) (oposite orientation) ---- */
                 ofs << "\t    rotate <0.0000,"
-                << RAD2DEG((double) -1.0 * theta)
-                << ",0.0000>"
-                << endl;
+                    << RAD2DEG((double) -1.0 * theta)
+                    << ",0.0000>"
+                    << endl;
 
-            }
+              }
             else
-            {
+              {
 
                 /* ---- Rotate along y-Axis (positive) (oposite orientation) ---- */
                 ofs << "\t    rotate <0.0000," << RAD2DEG(theta) << ",0.0000>" << endl;
 
-            }
+              }
 
-        }
+          }
 
         /* ---- Translate bond to end ---- */
         ofs << "\t    translate " << prefix << "_pos_" << bond -> GetEndAtomIdx() << endl;
@@ -563,12 +618,12 @@ void OutputCSTBonds(ostream &ofs, OBMol &mol, string prefix)
         /* ---- End of description of bond i ---- */
         ofs << "\t  }" << endl << "\t }" << endl << endl;
 
-    }
+      }
 
-}
+  }
 
-void OutputUnions(ostream &ofs, OBMol &mol, string prefix)
-{
+  void OutputUnions(ostream &ofs, OBMol &mol, string prefix)
+  {
     /* ---- Build union of all atoms ---- */
     ofs << endl << "//All atoms of molecule " << prefix << endl;
     ofs << "#ifdef (TRANS)" << endl;
@@ -579,17 +634,17 @@ void OutputUnions(ostream &ofs, OBMol &mol, string prefix)
 
     /* ---- Write definition of all atoms ---- */
     for(unsigned int i = 1; i <= mol.NumAtoms(); ++i)
-    {
+      {
 
         /* ---- Write definition of atom i ---- */
         ofs << "\t  object{" << prefix << "_atom" << i << "}" << endl;
 
-    }
+      }
     ofs << "\t }" << endl << endl;
 
     /* ---- Check for number of bonds ---- */
     if(mol.NumBonds() > 0)
-    {
+      {
 
         /* ---- Do a BAS or CST model ? ---- */
         ofs << "//Bonds only needed for ball and sticks or capped sticks models" << endl;
@@ -598,34 +653,34 @@ void OutputUnions(ostream &ofs, OBMol &mol, string prefix)
 
         /* ---- Description of all bonds ---- */
         for(unsigned int i = 0; i < mol.NumBonds(); ++i)
-        {
+          {
 
             /* ---- Write Definition of Bond i ---- */
             ofs << "\t  object{" << prefix << "_bond" << i << "}" << endl;
 
-        }
+          }
 
         /* ---- End of povray-conditional for ball and sticks ---- */
         ofs << "\t }" << endl << "#end" << endl << endl;
 
-    }
+      }
 
-}
+  }
 
-void OutputMoleculeBonds(ostream &ofs,
-                         string prefix,
-                         double min_x, double max_x,
-                         double min_y, double max_y,
-                         double min_z, double max_z
-                        )
-{
+  void OutputMoleculeBonds(ostream &ofs,
+                           string prefix,
+                           double min_x, double max_x,
+                           double min_y, double max_y,
+                           double min_z, double max_z
+                           )
+  {
     /* ---- Write a comment ---- */
     ofs << endl << "//Definition of molecule " << prefix << endl;
 
     /* ---- Check for space-fill model ---- */
     ofs << "#if (SPF)" << endl;
     ofs << "#declare " << prefix << " = object{"
-    << endl << "\t  " << prefix << "_atoms" << endl;
+        << endl << "\t  " << prefix << "_atoms" << endl;
 
     /* ---- Here we do BAS oder CST models ---- */
     ofs << "#else" << endl;
@@ -641,8 +696,8 @@ void OutputMoleculeBonds(ostream &ofs,
     ofs << "#if (TRANS)" << endl;
     ofs << "\t  difference {" << endl;
     ofs << "\t   object{" << prefix << "_bonds}" << endl
-    << "\t   object{" << prefix << "_atoms}" << endl
-    << "\t  }" << endl;
+        << "\t   object{" << prefix << "_atoms}" << endl
+        << "\t  }" << endl;
 
     /* ---- Do a solid model ? ---- */
     ofs << "#else" << endl;
@@ -655,51 +710,70 @@ void OutputMoleculeBonds(ostream &ofs,
 
     /* ---- Add comment (bounding box) ---- */
     ofs << "//\t  bounded_by {" << endl
-    << "//\t   box {" << endl
-    << "//\t    <"
-    << min_x - MAXRADIUS <<  ","
-    << min_y - MAXRADIUS <<  ","
-    << min_z - MAXRADIUS << ">" << endl;
+        << "//\t   box {" << endl
+        << "//\t    <"
+        << min_x - MAXRADIUS <<  ","
+        << min_y - MAXRADIUS <<  ","
+        << min_z - MAXRADIUS << ">" << endl;
 
     ofs << "//\t    <"
-    << max_x + MAXRADIUS << ","
-    << max_y + MAXRADIUS << ","
-    << max_z + MAXRADIUS << ">" << endl;
+        << max_x + MAXRADIUS << ","
+        << max_y + MAXRADIUS << ","
+        << max_z + MAXRADIUS << ">" << endl;
 
     ofs << "\t }" << endl << endl;
 
-}
+  }
 
-void OutputMoleculeNoBonds(ostream &ofs, string prefix)
-{
+  void OutputMoleculeNoBonds(ostream &ofs, string prefix)
+  {
     /* ---- Print description of molecule without bonds ---- */
     ofs << endl << "//Definition of Molecule " << prefix << " (no bonds)" << endl;
     ofs << "#declare " << prefix << " = object {" << prefix << "_atoms}" << endl << endl;
 
-}
+  }
 
-void OutputCenterComment(ostream &ofs,
-                         string prefix,
-                         double min_x, double max_x,
-                         double min_y, double max_y,
-                         double min_z, double max_z
-                        )
-{
+  void OutputCenterComment(ostream &ofs,
+                           string prefix,
+                           double min_x, double max_x,
+                           double min_y, double max_y,
+                           double min_z, double max_z
+                           )
+  {
     /* ---- Print center comment (Warn: Vector is multiplied by -1.0)---- */
     ofs << "//Center of molecule " << prefix << " (bounding box)" << endl;
     ofs << "#declare " << prefix << "_center = <"
-    << (double) -1.0 * (min_x + max_x) / (double) 2.0 << ","
-    << (double) -1.0 * (min_y + max_y) / (double) 2.0 << ","
-    << (double) -1.0 * (min_z + max_z) / (double) 2.0 << ">" << endl << endl;
-}
+        << (double) -1.0 * (min_x + max_x) / (double) 2.0 << ","
+        << (double) -1.0 * (min_y + max_y) / (double) 2.0 << ","
+        << (double) -1.0 * (min_z + max_z) / (double) 2.0 << ">;" << endl << endl;
+  }
 
-////////////////////////////////////////////////////////////////
+  ////////////////////////////////////////////////////////////////
 
-bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
-{
+  bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
+  {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
     if(pmol==NULL)
-        return false;
+      return false;
+
+    // Model-type should be one of "bas", "spf" or "cas"
+    model_type = "BAS"; // Default is ball-and-stick
+    const char* tmp = pConv->IsOption("m");
+    if (tmp) {
+      model_type = string(tmp);
+      // Convert to uppercase
+      std::transform(model_type.begin(), model_type.end(), model_type.begin(), static_cast< int(*)(int) >(std::toupper));
+      if (model_type != "BAS" && model_type != "SPF" && model_type != "CST") {
+        obErrorLog.ThrowError(__FUNCTION__, "Unknown model type specified. Using the default instead (\"BAS\", ball-and-stick).\n",obWarning);
+        model_type = "BAS";
+      }
+    }
+
+    // Set private class variables for options
+    trans_texture = pConv->IsOption("t") ? true : false;
+    sky = pConv->IsOption("s") ? true : false;
+    checkerboard = pConv->IsOption("c") ? true : false;
+    sphere = pConv->IsOption("f") ? true : false;
 
     //Define some references so we can use the old parameter names
     ostream &ofs = *pConv->GetOutStream();
@@ -708,47 +782,35 @@ bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
     static long num = 0;
     double min_x, max_x, min_y, max_y, min_z, max_z; /* Edges of bounding box */
-    string prefix;
 
-    /* ---- We use the molecule-title as our prefix ---- */
-    if(title != (const char*) NULL)
-        prefix = MakePrefix(title);
-    else if (mol.GetTitle() != (const char *) NULL)
-        prefix = MakePrefix(mol.GetTitle());
-    else
-        prefix = MakePrefix("Unknown");
+    /* ---- We use mol_${num}_ as our prefix ---- */
+    stringstream ss;
+    ss << "mol_" << num;
+    string prefix = ss.str();
 
     /* ---- Check if we have already written a molecule to this file ---- */
     if (num == 0)
-    {
+      {
 
         /* ---- Print the header ---- */
         OutputHeader(ofs, mol, prefix);
 
-    }
+      }
     else
-    {
+      {
 
         /* ---- Convert the unique molecule-number to a string and set the prefix ---- */
-#if defined(HAVE_SSTREAM)
         ostringstream numStr;
         numStr << num << ends;
         prefix += numStr.str().c_str();
-#else
-
-        ostrstream numStr;
-        numStr << num << ends;
-        prefix += numStr.str();
-#endif
-
-    }
+      }
 
     /* ---- Print positions and descriptions of all atoms ---- */
     OutputAtoms(ofs, mol, prefix);
 
     /* ---- Check #bonds ---- */
     if (mol.NumBonds() > 0)
-    {
+      {
 
         /* ---- Write an comment ---- */
         ofs << "//Povray-description of bonds 1 - " << mol.NumBonds() << endl;
@@ -771,7 +833,7 @@ bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
         /* ---- End of povray-conditional for capped sticks ---- */
         ofs << "#end // (CST-Bonds)" << endl << endl;
 
-    }
+      }
 
     /* ---- Print out unions of atoms and bonds ---- */
     OutputUnions(ofs, mol, prefix);
@@ -781,7 +843,7 @@ bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
     /* ---- Check #bonds ---- */
     if (mol.NumBonds() > 0)
-    {
+      {
 
         /* ---- Print out description of molecule ---- */
         OutputMoleculeBonds(ofs,
@@ -790,14 +852,14 @@ bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
                             min_y, max_y,
                             min_z, max_z);
 
-    }
+      }
     else
-    {
+      {
 
         /* ---- Now we can define the molecule without bonds ---- */
         OutputMoleculeNoBonds(ofs, prefix);
 
-    }
+      }
 
     /* ---- Insert declaration for centering the molecule ---- */
     OutputCenterComment(ofs,
@@ -806,11 +868,15 @@ bool PovrayFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
                         min_y, max_y,
                         min_z, max_z);
 
+
+    /* ---- Insert the molecule ---- */
+    ofs << prefix << endl;
+
     /* ---- Increment the static molecule output-counter ---- */
     num++;
 
     /* ---- Everything is ok! ---- */
     return(true);
-}
+  }
 
 } //namespace OpenBabel
