@@ -1,10 +1,10 @@
 /**********************************************************************
 Copyright (C) 2005 by Chris Morley
- 
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -16,9 +16,10 @@ GNU General Public License for more details.
 #include "openbabel/reaction.h"
 #include "openbabel/xml.h"
 #include "openbabel/kinetics.h"
+#include "openbabel/text.h"
 
 using namespace std;
-using std::tr1::shared_ptr;
+//using std::tr1::shared_ptr;
 namespace OpenBabel
 {
 
@@ -38,22 +39,34 @@ public:
 
   const char* Description()
   {
-      return 
-"CML Reaction format\n"
-"Minimal implementation\n"
-"This implementation uses libxml2.\n"
-"Write options (e.g. -x1a)\n"
-"1  output CML V1.0  or\n"
-"2  output CML V2.0 (default)\n"
-"a  output array format for atoms and bonds\n"
-"l  molecules NOT in MoleculeList\n"
-"h  use hydrogenCount for all hydrogens\n"
-"x  omit XML declaration\n"
-"r  omit rate constant data\n"
-"N<prefix> add namespace prefix to elements\n"
-"M  add obr prefix on non-CMLReact elements\n"
-"p  add properties to molecules\n"
-"\n";
+    return
+      "CML Reaction format\n"
+      "A minimal implementation of the CML Reaction format\n"
+      "This implementation uses libxml2.\n"
+      "Write Options (e.g. -x1a)\n"
+      " 1  output CML1 (rather than CML2)\n"
+      " a  output array format for atoms and bonds\n"
+      " l  molecules NOT in MoleculeList\n"
+      " h  use hydrogenCount for all hydrogens\n"
+      " x  omit XML declaration\n"
+      " r  omit rate constant data\n"
+      " N<prefix> add namespace prefix to elements\n"
+      " M  add obr prefix on non-CMLReact elements\n"
+      " p  add properties to molecules\n\n"
+
+      "The implementation of this format which reads and writes to and from\n"
+      "OBReaction objects is fairly minimal at present. (Currently the only\n"
+      "other reaction format in OpenBabel is RXN.) During reading, only the\n"
+      "elements <reaction>, <reactant>, <product> and <molecule>  are acted\n"
+      "upon (the last through CML). The molecules can be collected together\n"
+      "in a list at the start of the file and referenced in the reactant and\n"
+      "product via e.g. <molecule ref=\"mol1\">.\n\n"
+
+      "On writing, the list format can be specified with the -xl option. The\n"
+      "list containers are <moleculeList> and <reactionList> and the overall\n"
+      "wrapper is <mechanism>. These are non-standard CMLReact element names\n"
+      "and would have to be changed (in the code) to <list>,<list> and <cml>\n"
+      "if this was unacceptable.\n\n";
   }
 
   virtual const char* TargetClassDescription()
@@ -93,6 +106,8 @@ private:
   ostringstream ssout; //temporary output
   bool MolsHaveBeenAdded; //separate OBMols during output
   OBRateData* _pRD; //used on input
+  string _text; //used on output to delay text so it can be after reactions
+  ostream* _pOut;//used on output to save the original output stream
 };
 
 //Make an instance of the format class
@@ -122,9 +137,9 @@ bool CMLReactFormat::ReadChemObject(OBConversion* pConv)
             auditMsg,
             obAuditMsg);
 
-   //Do transformation and return reaction, if it has either reactants or products 
+   //Do transformation and return reaction, if it has either reactants or products
   if(ret && (pReact->NumReactants()!=0 || pReact->NumProducts()!=0)) //Do transformation and return molecule
-    return pConv->AddChemObject(pReact->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS)))!=0;
+    return pConv->AddChemObject(pReact->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS),pConv))!=0;
   else
   {
     delete pReact;
@@ -177,7 +192,7 @@ bool CMLReactFormat::DoElement(const string& name)
     }
     else
     {
-      shared_ptr<OBMol> sp(new OBMol);	
+      shared_ptr<OBMol> sp(new OBMol);
       OBFormat* pCMLFormat = OBConversion::FindFormat("cml");
       if(!pCMLFormat)
         return false;
@@ -303,40 +318,67 @@ bool CMLReactFormat::WriteChemObject(OBConversion* pConv)
     //If sent a molecule, add it to the list.
     //Molecules added in this way have precedence over internal molecules.
     //They can therefore have structure and properties, perhaps obtained from
-    //more than one place using the -C option. Thermodynamic data can also be 
+    //more than one place using the -C option. Thermodynamic data can also be
     //upgraded from that in molecules in OBReaction object (and so in Chemkin or
     //CMLReact input file) by including a separate file with updates.
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
-    if(pmol==NULL)
-      return false;
-    if(pConv->GetOutputIndex()==1)
-      OMols.clear();
-    shared_ptr<OBMol> sp(pmol);
-    AddMolToList(sp, OMols);
-    pConv->SetOutputIndex(-1); //Signals that molecules have been added
 
-    bool ret=true;
-    if(pConv->IsLast())
+    if(pConv->GetOutputIndex()==1)
     {
-      //Only molecules have been supplied; output them via CMLFormat
-      OBFormat* pCMLFormat = pConv->FindFormat("cml");
-      if(pCMLFormat==NULL)
+      _pOut = pConv->GetOutStream();//Save the original output stream
+      OMols.clear();
+    }
+
+    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    if(pmol!=NULL)
+    {
+      shared_ptr<OBMol> sp(pmol);
+      AddMolToList(sp, OMols);
+      pConv->SetOutputIndex(-1); //Signals that molecules have been added
+
+      bool ret=true;
+      if(pConv->IsLast())
       {
-        obErrorLog.ThrowError(__FUNCTION__,
-          "CML format for molecules is needed by CMLReactformat and is not available\n",obError);
-          return false;
+        //Only molecules have been supplied; output them via CMLFormat
+        OBFormat* pCMLFormat = pConv->FindFormat("cml");
+        if(pCMLFormat==NULL)
+        {
+          obErrorLog.ThrowError(__FUNCTION__,
+            "CML format for molecules is needed by CMLReactformat and is not available\n",obError);
+            return false;
+        }
+        unsigned int n=0;
+        MolMap::iterator mapitr;
+        for(mapitr=OMols.begin();mapitr!=OMols.end() && ret; ++mapitr)
+        {
+          pConv->SetOutputIndex(++n);      //we have to increment and
+          pConv->SetLast(n==OMols.size()); //setLast manually because we are not using Convert()
+          ret = pCMLFormat->WriteMolecule(mapitr->second.get(), pConv);
+        }
       }
-      unsigned int n=0;
-      MolMap::iterator mapitr;
-      for(mapitr=OMols.begin();mapitr!=OMols.end() && ret; ++mapitr)
-      {
-        pConv->SetOutputIndex(++n);      //we have to increment and
-        pConv->SetLast(n==OMols.size()); //setLast manually because we are not using Convert()
-        ret = pCMLFormat->WriteMolecule(mapitr->second.get(), pConv);
-      }
+      return ret;
+    }
+    else
+    //If sent text as an OBText object, output the text up to the insertion point
+    //and the rest of the text in _text. The content of _text is output at the end.
+    {
+      OBText* ptext = dynamic_cast<OBText*>(pOb);
+      if(ptext==NULL)
+        return false;
+      string::size_type pos = 0;
+      string frontText(ptext->GetText(pos));
+      *_pOut << frontText;//Output(to orig outstream) text up to insertion point
+      _text = ptext->GetText(pos);  //Save text after insertion point to be output at the end
+
+      //if any of the text contains an xml declaration, do not write again
+      // and also omit <cml>...</cml> wrapper
+      if(frontText.find("<?xml ")!=string::npos)
+        pConv->AddOption("ReactionsNotStandalone");
+
+      pConv->SetOutputIndex(pConv->GetOutputIndex()-1);//not an output we want to count
+      return true;
     }
-    return ret;
   }
+
   bool ret=false;
   ret=WriteMolecule(pReact,pConv);
 
@@ -346,6 +388,13 @@ bool CMLReactFormat::WriteChemObject(OBConversion* pConv)
   obErrorLog.ThrowError(__FUNCTION__, auditMsg, obAuditMsg);
 
   delete pOb;
+
+  if(pConv->IsLast() && !_text.empty())
+  {
+    *_pOut << _text; //Last part of an OBText object to original output stream
+    _text.clear();
+  }
+
   return ret;
 }
 
@@ -367,7 +416,7 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   //With normal form, if more than one reaction to be output,
   //write <cml> at start and </cml> at end.
   //With list form, use ref attributes when refering to molecules and put
-  //actual molecules in a separate list. 
+  //actual molecules in a separate list.
   // <cml><moleculeList>...</moleculeList><reactionList>...</reactionList></cml>
 
   //The following element and attribute names may be output
@@ -384,24 +433,24 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   static const xmlChar C_REACTANTLIST[] = "reactantList";
   static const xmlChar C_PRODUCTLIST[]  = "productList";
   static const xmlChar C_REF[]          = "ref";
-  static const xmlChar C_TITLE[]        = "title";
-  
-  bool list = _pxmlConv->IsOption("l")==NULL; //Output with molecules in a separate list
+  static const xmlChar C_TITLE[]        = "id"; //"title";
+  static const xmlChar C_REVERSIBLE[]   = "reversible";
 
-  ostream* pOut = pConv->GetOutStream(); //the original output stream
+
+  bool list = _pxmlConv->IsOption("l")==NULL; //Output with molecules in a separate list
 
   xmlChar* prefix = BAD_CAST _pxmlConv->IsOption("N");
   xmlChar* altprefix = _pxmlConv->IsOption("M") ? BAD_CAST "obr" : NULL;
 
   xmlChar* uri=NULL;
   xmlChar* alturi=NULL;
-  
-  _pxmlConv->AddOption("MolsNotStandalone",OBConversion::OUTOPTIONS); //inform CMLFormat 
+
+  _pxmlConv->AddOption("MolsNotStandalone",OBConversion::OUTOPTIONS); //inform CMLFormat
 
 //  OBConversion MolConv(*_pxmlConv); //new copy to use to write associated CML molecules
 //	MolConv.SetAuxConv(NULL); //temporary until a proper OBConversion copy constructor written
 //	MolConv.SetOneObjectOnly();
-   
+
   OBFormat* pCMLFormat = _pxmlConv->FindFormat("cml");
   if(pCMLFormat==NULL)
   {
@@ -409,7 +458,7 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       "CML format for molecules is needed by CMLReactformat and is not available\n",obError);
     return false;
   }
-  
+
   bool MolsHaveBeenAdded=false;
   if(pConv->GetOutputIndex()==0)
   {
@@ -420,6 +469,7 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   //For first reaction
   if(pConv->GetOutputIndex()==1) //OBConversion::Convert() is still using original pConv
   {
+    _pOut = pConv->GetOutStream();//Save the original output stream
     if(list)
     {
       //Use a temporary stringstream for list format so that moleculeList can be first
@@ -428,20 +478,24 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
         OMols.clear();
 
       _pxmlConv->SetOutStream(&ssout);
+      //Do also for original object; GetDerived() copies the old to the new on subsequent reactions
+      pConv->SetOutStream(&ssout);
     }
     if(!_pxmlConv->IsOption("x") && !_pxmlConv->IsOption("ReactionsNotStandalone"))
     {
       xmlTextWriterStartDocument(writer(), NULL, NULL, NULL);
       uri=BAD_CAST NamespaceURI();
-      alturi = BAD_CAST "http://www.gaseq.co.uk/obr"; //TODO: better alturi
+      alturi = BAD_CAST "http://www.gaseq.co.uk/obr"; //@todo: better alturi
     }
 
     if(list)
     {
-      xmlTextWriterStartElementNS(writer(), prefix, C_LISTWRAPPER, uri);
+      if(!pConv->IsOption("ReactionsNotStandalone"))
+      {
+        xmlTextWriterStartElementNS(writer(), prefix, C_LISTWRAPPER, uri);
       if(altprefix)
         xmlTextWriterWriteAttributeNS(writer(),BAD_CAST "xmlns",altprefix,NULL,alturi);
-
+      }
       xmlTextWriterStartElementNS(writer(), prefix, C_REACTIONLIST, NULL);
     }
     else if(!_pxmlConv->IsLast() && !_pxmlConv->IsOption("ReactionsNotStandalone"))
@@ -452,6 +506,8 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   xmlTextWriterStartElementNS(writer(), prefix, C_REACTION, NULL);
   if(!pReact->GetTitle().empty())
     xmlTextWriterWriteFormatAttribute(writer(), C_TITLE,"%s", pReact->GetTitle().c_str());
+  if(pReact->IsReversible())
+    xmlTextWriterWriteFormatAttribute(writer(), C_REVERSIBLE,"%s", "true");
 
   xmlTextWriterStartElementNS(writer(), prefix, C_REACTANTLIST, NULL);
   unsigned i;
@@ -475,7 +531,7 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     xmlTextWriterEndElement(writer());//reactant
   }
   xmlTextWriterEndElement(writer());//reactantList
-  
+
   xmlTextWriterStartElementNS(writer(), prefix, C_PRODUCTLIST, NULL);
   for(i=0;i<pReact->NumProducts();i++)
   {
@@ -535,7 +591,9 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       }
       xmlTextWriterEndElement(writer());//moleculeList
 
-      xmlTextWriterEndElement(writer());//LISTWRAPPER
+      if(!_pxmlConv->IsOption("ReactionsNotStandalone"))
+        xmlTextWriterEndElement(writer());//LISTWRAPPER
+
       xmlTextWriterEndDocument(writer());
       OutputToStream();
 
@@ -546,8 +604,9 @@ bool CMLReactFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       if(reaclistPos!=string::npos)
         mollistPos = s.find("<moleculeList",reaclistPos+1);
       footerPos = s.find("</cml");
-
-      *pOut << s.substr(0, reaclistPos)                      //header
+      if(footerPos==string::npos)// cml tag may have been supressed
+        footerPos = s.size();
+      *_pOut << s.substr(0, reaclistPos)                      //header
             << s.substr(mollistPos, footerPos-mollistPos)    //moleculeList
             << s.substr(reaclistPos, mollistPos-reaclistPos) //reactionList
             << s.substr(footerPos) << endl;                  //footer
@@ -580,7 +639,7 @@ string CMLReactFormat::AddMolToList(shared_ptr<OBMol> spmol, MolMap& mmap)
   }
   else
   {
-    //If id is a filename with a path or an extension, which can 
+    //If id is a filename with a path or an extension, which can
     //happen if no explicit title given to a molecule,
     // remove path and extension and use the filename as id
     string::size_type pos;
@@ -609,7 +668,7 @@ string CMLReactFormat::AddMolToList(shared_ptr<OBMol> spmol, MolMap& mmap)
       {
         spmol.swap(spnew);
         mapitr->second = spmol; //replace with new molecule
-      //The OBMol originally in map will be deleted when local shared_ptr goes out of scope. 
+      //The OBMol originally in map will be deleted when local shared_ptr goes out of scope.
       }
     }
   }
@@ -671,7 +730,7 @@ bool CMLReactFormat::WriteRateData(OBReaction* pReact, xmlChar* altprefix)
   {
   case OBRateData::TROE:
     xmlTextWriterStartElementNS(writer(), altprefix, C_TROEPARAMS, NULL);
-    xmlTextWriterWriteFormatString(writer(),"%g %g %g %g", 
+    xmlTextWriterWriteFormatString(writer(),"%g %g %g %g",
       pRD->GetTroeParam(0),pRD->GetTroeParam(1),pRD->GetTroeParam(2),pRD->GetTroeParam(3));
     xmlTextWriterEndElement(writer());
     //fallthrough
