@@ -5,7 +5,7 @@ Copyright (C) 2004 by Chris Morley
 Some portions Copyright (C) 2005-2006 by Geoffrey Hutchison
 
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ GNU General Public License for more details.
 #include <stdlib.h>
 
 #include <openbabel/obconversion.h>
+//#include <openbabel/mol.h>
 #include <openbabel/locale.h>
 
 #ifdef HAVE_LIBZ
@@ -83,7 +84,7 @@ namespace OpenBabel {
   //***************************************************
 
   /** @class OBConversion obconversion.h <openbabel/obconversion.h>
-      OBConversion maintains a list of the available formats, 
+      OBConversion maintains a list of the available formats,
       provides information on them, and controls the conversion process.
 
       A conversion is carried out by the calling routine, usually in a
@@ -93,9 +94,9 @@ namespace OpenBabel {
       the Convert() function is called, which allows a single input file
       to be converted, or the extended functionality of FullConvert()
       is used. This allows multiple input and output files, allowing:
-      - aggregation      - the contents of many input files converted 
+      - aggregation      - the contents of many input files converted
       and sent to one output file;
-      - splitting        - the molecules from one input file sent to 
+      - splitting        - the molecules from one input file sent to
       separate output files;
       - batch conversion - each input file converted to an output file.
 
@@ -110,7 +111,7 @@ namespace OpenBabel {
       OBConversion can also be used with an "API" interface
       called from programs which manipulate chemical objects. Input/output is
       done with the Read() and Write() functions which work with any
-      chemical object, but need to have its type specified. (The 
+      chemical object, but need to have its type specified. (The
       ReadMolecule() and WriteMolecule() functions of the format classes
       can also be used directly.)
 
@@ -125,21 +126,21 @@ namespace OpenBabel {
       @code
       OBConversion conv(&cin,&cout);
       if(conv.SetInAndOutFormats("SMI","MOL"))
-      {	
+      {
          OBMol mol;
          if(conv.Read(&mol))
-            // ...manipulate molecule 
-		
+            // ...manipulate molecule
+
          conv->Write(&mol);
       }
       @endcode
-	
+
       A two stage construction is used to allow error handling
       if the format ID is not recognized. This is necessary now that the
       formats are dynamic and errors are not caught at compile time.
-      OBConversion::Read() is a templated function so that objects derived
-      from OBBase can also be handled, in addition to OBMol, if the format
-      routines are written appropriately.
+      OBConversion::Read() uses a pointer to OBBase, so that, in addition
+      to OBMol, other kinds of objects, such as reactions, can also be handled
+      if the format routines are written appropriately.
 
       <b>To make a molecule from a SMILES string.</b>
       @code
@@ -155,7 +156,7 @@ namespace OpenBabel {
 
       @code
       #include <openbabel/obconversion.h> //mol.h is not needed
-      ...set up an istream is and an ostream os 
+      ...set up an istream is and an ostream os
       OBConversion conv(&is,&os);
       if(conv.SetInAndOutFormats("SMI","MOL"))
       {
@@ -166,7 +167,7 @@ namespace OpenBabel {
 
       <b>To add automatic format conversion to an existing program.</b>
 
-      The existing program inputs from the file identified by the 
+      The existing program inputs from the file identified by the
       const char* filename into the istream is. The file is assumed to have
       a format ORIG, but other formats, identified by their file extensions,
       can now be used.
@@ -177,7 +178,7 @@ namespace OpenBabel {
       OBConversion conv;
       OBFormat* inFormat = conv.FormatFromExt(filename);
       OBFormat* outFormat = conv.GetFormat("ORIG");
-      istream* pIn = &ifs; 
+      istream* pIn = &ifs;
       stringstream newstream;
       if(inFormat && outFormat)
       {
@@ -185,33 +186,28 @@ namespace OpenBabel {
          conv.Convert(pIn,&newstream);
          pIn=&newstream;
       }
-      //else error; new features not available; fallback to original functionality 
+      //else error; new features not available; fallback to original functionality
 
       ...Carry on with original code using pIn
       @endcode
-
-      In certain Windows builds, a degree of independence from OpenBabel can be
-      achieved using DLLs. This code would be linked with obconv.lib.
-      At runtime the following DLLs would be in the executable directory:
-      obconv.dll, obdll.dll, one or more *.obf format files.
   */
-    
+
   int OBConversion::FormatFilesLoaded = 0;
 
 //  OBFormat* OBConversion::pDefaultFormat=NULL;
 
-  OBConversion::OBConversion(istream* is, ostream* os) : 
+  OBConversion::OBConversion(istream* is, ostream* os) :
     pInFormat(NULL),pOutFormat(NULL), Index(0), StartNumber(1),
     EndNumber(0), Count(-1), m_IsFirstInput(true), m_IsLast(true),
     MoreFilesToCome(false), OneObjectOnly(false), CheckedForGzip(false),
-    NeedToFreeInStream(false), NeedToFreeOutStream(false), 
+    NeedToFreeInStream(false), NeedToFreeOutStream(false),
     pOb1(NULL), pAuxConv(NULL),pLineEndBuf(NULL),wInpos(0),wInlen(0)
   {
     pInStream=is;
     pOutStream=os;
     if (FormatFilesLoaded == 0)
       FormatFilesLoaded = LoadFormatFiles();
-	
+
     //These options take a parameter
     RegisterOptionParam("f", NULL, 1,GENOPTIONS);
     RegisterOptionParam("l", NULL, 1,GENOPTIONS);
@@ -250,7 +246,7 @@ namespace OpenBabel {
   }
   ///////////////////////////////////////////////
 
-  OBConversion::~OBConversion() 
+  OBConversion::~OBConversion()
   {
     if(pAuxConv!=this)
       if(pAuxConv)
@@ -271,12 +267,14 @@ namespace OpenBabel {
       pOutStream=NULL;
       NeedToFreeOutStream = false;
     }
+//    delete pLineEndBuf;
+//   pLineEndBuf=NULL;
   }
   //////////////////////////////////////////////////////
 
   /// Class information on formats is collected by making an instance of the class
-  /// derived from OBFormat(only one is usually required). RegisterFormat() is called 
-  /// from its constructor. 
+  /// derived from OBFormat(only one is usually required). RegisterFormat() is called
+  /// from its constructor.
   ///
   /// If the compiled format is stored separately, like in a DLL or shared library,
   /// the initialization code makes an instance of the imported OBFormat class.
@@ -292,7 +290,7 @@ namespace OpenBabel {
     //	if(FormatFilesLoaded) return 0;
     //	FormatFilesLoaded=true; //so will load files only once
 #if  defined(USING_DYNAMIC_LIBS)
-    //Depending on availablilty, look successively in 
+    //Depending on availablilty, look successively in
     //FORMATFILE_DIR, executable directory,or current directory
     string TargetDir;
 #ifdef FORMATFILE_DIR
@@ -300,7 +298,7 @@ namespace OpenBabel {
 #endif
 
     DLHandler::getConvDirectory(TargetDir);
-	
+
     vector<string> files;
     if(!DLHandler::findFiles(files,DLHandler::getFormatFilePattern(),TargetDir)) return 0;
 
@@ -325,7 +323,7 @@ namespace OpenBabel {
     {
       static vector<string> vec(3);
       vec[1] = string("define");
-      vec[2] = string("plugindefines.txt"); 
+      vec[2] = string("plugindefines.txt");
       pdef->MakeInstance(vec);
     }
     return count;
@@ -357,7 +355,7 @@ namespace OpenBabel {
   bool OBConversion::SetOutFormat(OBFormat* pOut)
   {
     pOutFormat=pOut;
-    return !(pOutFormat->Flags() & NOTWRITABLE);
+    return pOut && !(pOutFormat->Flags() & NOTWRITABLE);
   }
   //////////////////////////////////////////////////////
   bool OBConversion::SetInFormat(const char* inID)
@@ -376,9 +374,9 @@ namespace OpenBabel {
   }
 
   //////////////////////////////////////////////////////
-  int OBConversion::Convert(istream* is, ostream* os) 
+  int OBConversion::Convert(istream* is, ostream* os)
   {
-    if (is) { 
+    if (is) {
       pInStream=is;
       CheckedForGzip = false; // haven't checked this for gzip yet
     }
@@ -410,7 +408,7 @@ namespace OpenBabel {
 #endif
 
     //The FilteringInputStreambuf delivers characters to the istream, pInStream,
-    //and receives characters this stream's original rdbuf. 
+    //and receives characters this stream's original rdbuf.
     //It filters them, converting CRLF and CR line endings to LF.
     //seek and tellg requests to the stream are passed through to the original
     //rdbuf. A FilteringInputStreambuf is installed only for appropriate formats
@@ -420,29 +418,35 @@ namespace OpenBabel {
     int count = Convert();
 
     pOutStream = pOrigOutStream;
+#ifdef HAVE_LIBZ
+    if ( CheckedForGzip ){ // Bug reported by Gert Thijs
+		delete zIn;
+		pInStream = is;
+	}
+#endif
     return count;
   }
 
 
   ////////////////////////////////////////////////////
   /// Actions the "convert" interface.
-  ///	Calls the OBFormat class's ReadMolecule() which 
+  ///	Calls the OBFormat class's ReadMolecule() which
   ///	 - makes a new chemical object of its chosen type (e.g. OBMol)
   ///	 - reads an object from the input file
   ///	 - subjects the chemical object to 'transformations' as specified by the Options
-  ///	 - calls AddChemObject to add it to a buffer. The previous object is first output 
+  ///	 - calls AddChemObject to add it to a buffer. The previous object is first output
   ///	   via the output Format's WriteMolecule(). During the output process calling
   /// IsFirst() and GetIndex() (the number of objects including the current one already output.
-  /// allows more control, for instance writing \<cml\> and \</cml\> tags for multiple molecule outputs only. 
+  /// allows more control, for instance writing \<cml\> and \</cml\> tags for multiple molecule outputs only.
   ///
   ///	AddChemObject does not save the object passed to it if it is NULL (as a result of a DoTransformation())
   ///	or if the number of the object is outside the range defined by
-  ///	StartNumber and EndNumber.This means the start and end counts apply to all chemical objects 
+  ///	StartNumber and EndNumber.This means the start and end counts apply to all chemical objects
   ///	found whether or not they	are output.
-  ///	
-  ///	If ReadMolecule returns false the input conversion loop is exited. 
   ///
-  int OBConversion::Convert() 
+  ///	If ReadMolecule returns false the input conversion loop is exited.
+  ///
+  int OBConversion::Convert()
   {
     if(pInStream==NULL || pOutStream==NULL)
       {
@@ -465,7 +469,7 @@ namespace OpenBabel {
       OneObjectOnly=true;
 
     //Input loop
-    while(ReadyToInput && pInStream->good()) //Possible to omit? && pInStream->peek() != EOF 
+    while(ReadyToInput && pInStream->good()) //Possible to omit? && pInStream->peek() != EOF
       {
         if(pInStream==&cin)
           {
@@ -474,13 +478,41 @@ namespace OpenBabel {
           }
         else
           rInpos = pInStream->tellg();
-		
+
         bool ret=false;
         try
           {
             ret = pInFormat->ReadChemObject(this);
+/*            if (ret && IsOption("readconformers", GENOPTIONS)) {
+              std::streampos pos = pInStream->tellg();
+              OBMol nextMol;
+              OBConversion conv;
+              conv.SetOutFormat("smi");
+              std::string ref_smiles = conv.WriteString(pOb1);
+              while (pInStream->good()) {
+                if (!pInFormat->ReadMolecule(&nextMol, this))
+                  break;
+                std::string smiles = conv.WriteString(&nextMol);
+                if (smiles == ref_smiles) {
+                  OBMol *pmol = dynamic_cast<OBMol*>(pOb1);
+                  if (!pmol)
+                    break;
+                  unsigned int numCoords = nextMol.NumAtoms() * 3;
+                  double *coords = nextMol.GetCoordinates();
+                  double *conformer = new double [numCoords];
+                  for (unsigned int i = 0; i < numCoords; ++i)
+                    conformer[i] = coords[i];
+                  pmol->AddConformer(conformer);
+                  pos = pInStream->tellg();
+                } else {
+                  break;
+                }
+              }
+              pInStream->seekg(pos, std::ios::beg);
+            }
+*/
             SetFirstInput(false);
-          }		
+          }
         catch(...)
           {
             if(!IsOption("e", GENOPTIONS) && !OneObjectOnly)
@@ -494,26 +526,27 @@ namespace OpenBabel {
           {
             //error or termination request: terminate unless
             // -e option requested and successfully can skip past current object
-            if(!IsOption("e", GENOPTIONS) || pInFormat->SkipObjects(0,this)!=1) 
+            if(!IsOption("e", GENOPTIONS) || pInFormat->SkipObjects(0,this)!=1)
               break;
           }
         if(OneObjectOnly)
           break;
         // Objects supplied to AddChemObject() which may output them after a delay
         //ReadyToInput may be made false in AddChemObject()
-        // by WriteMolecule() returning false  or by Count==EndNumber		
+        // by WriteMolecule() returning false  or by Count==EndNumber
       }
-	
+
     //Output last object
     m_IsLast= !MoreFilesToCome;
 
-    //Output is deferred until the end with -C option
-    if(pOutFormat && (!IsOption("C",GENOPTIONS) || m_IsLast))
-      if(pOb1 && !pOutFormat->WriteChemObject(this))
+    //Output is always occurs at the end with the --OutputAtEnd option
+    bool oae = IsOption("OutputAtEnd",GENOPTIONS)!=NULL;
+    if(pOutFormat && (!oae || m_IsLast))
+      if((oae || pOb1) && !pOutFormat->WriteChemObject(this))
         Index--;
-	
+
     //Put AddChemObject() into non-queue mode
-    Count= -1; 
+    Count= -1;
     EndNumber=StartNumber=0; pOb1=NULL;//leave tidy
     MoreFilesToCome=false;
     OneObjectOnly=false;
@@ -534,7 +567,7 @@ namespace OpenBabel {
             //Try to skip objects now
             int ret = pInFormat->SkipObjects(StartNumber-1,this);
             if(ret==-1) //error
-              return false; 
+              return false;
             if(ret==1) //success:objects skipped
               {
                 Count = StartNumber-1;
@@ -564,38 +597,39 @@ namespace OpenBabel {
 
   //////////////////////////////////////////////////////
   ///	Called by ReadMolecule() to deliver an object it has read from an input stream.
-  /// Used in two modes: 
+  /// Used in two modes:
   ///  - When Count is negative it is left negative and the routine is just a store
   ///    for an OBBase object.  The negative value returned tells the calling
   ///    routine that no more objects are required.
   ///  - When count is >=0, probably set by Convert(), it acts as a queue of 2:
   ///    writing the currently stored value before accepting the supplied one. This delay
   ///    allows output routines to respond differently when the written object is the last.
-  ///    Count is incremented with each call, even if pOb=NULL. 
-  ///    Objects are not added to the queue if the count is outside the range  
-  ///    StartNumber to EndNumber. There is no upper limit if EndNumber is zero. 
+  ///    Count is incremented with each call, even if pOb=NULL.
+  ///    Objects are not added to the queue if the count is outside the range
+  ///    StartNumber to EndNumber. There is no upper limit if EndNumber is zero.
   ///    The return value is Count ((>0) or 0 if WriteChemObject returned false.
   int OBConversion::AddChemObject(OBBase* pOb)
   {
-    if(Count<0) 
+    if(Count<0)
       {
         pOb1=pOb;
         return Count; // <0
       }
     Count++;
     if(Count>=(int)StartNumber)//keeps reading objects but does nothing with them
-      {	
+      {
         if(Count==(int)EndNumber)
           ReadyToInput=false; //stops any more objects being read
 
-        rInlen = pInStream->tellg() - rInpos;
+        rInlen = (pInStream->tellg() - rInpos);
+         // - (pLineEndBuf ? pLineEndBuf->getCorrection() : 0); //correction for CRLF
 
         if(pOb)
           {
             if(pOb1 && pOutFormat) //see if there is an object ready to be output
               {
                 //Output object
-                if (!pOutFormat->WriteChemObject(this))  
+                if (!pOutFormat->WriteChemObject(this))
                   {
                     //faultly write, so finish
                     --Index;
@@ -654,6 +688,11 @@ namespace OpenBabel {
     return OBFormat::FindType(ID);
   }
 
+  OBFormat* OBConversion::FindFormat(const std::string ID)
+  {
+    return OBFormat::FindType(ID.c_str());
+  }
+
   //////////////////////////////////////////////////
   const char* OBConversion::GetTitle() const
   {
@@ -669,7 +708,7 @@ namespace OpenBabel {
   {
     OneObjectOnly=b;
     m_IsLast=b;
-  }	
+  }
 
   /////////////////////////////////////////////////////////
   OBFormat* OBConversion::FormatFromExt(const char* filename)
@@ -703,6 +742,11 @@ namespace OpenBabel {
     return FindFormat( file.c_str() ); //if no format found
   }
 
+  OBFormat* OBConversion::FormatFromExt(const std::string filename)
+  {
+    return FormatFromExt(filename.c_str());
+  }
+
   OBFormat* OBConversion::FormatFromMIME(const char* MIME)
   {
     return OBFormat::FormatFromMIME(MIME);
@@ -710,7 +754,7 @@ namespace OpenBabel {
 
   bool	OBConversion::Read(OBBase* pOb, std::istream* pin)
   {
-    if(pin) { 
+    if(pin) {
       pInStream=pin;
       CheckedForGzip = false; // haven't set this stream to gzip (yet)
     }
@@ -736,14 +780,13 @@ namespace OpenBabel {
 
     // Set the locale for number parsing to avoid locale issues: PR#1785463
     obLocale.SetLocale();
-    
+
     // Also set the C++ stream locale
     locale originalLocale = pInStream->getloc(); // save the original
     locale cNumericLocale(originalLocale, "C", locale::numeric);
     pInStream->imbue(cNumericLocale);
 
     bool success = pInFormat->ReadMolecule(pOb, this);
-
     // return the C locale to the original one
     obLocale.RestoreLocale();
     // Restore the original C++ locale as well
@@ -763,12 +806,13 @@ namespace OpenBabel {
   {
     //Do not install filtering input stream if a binary or XML format
     //or if already installed in the current InStream (which may have changed).
-    //Deleting any old LErdbuf before contructing a new one ensures there is 
+    //Deleting any old LErdbuf before contructing a new one ensures there is
     //only one for each OBConversion object. It is deleted in the destructor.
 
     if(pInFormat && !(pInFormat->Flags() & (READBINARY | READXML)) && pInStream->rdbuf()!=pLineEndBuf)
     {
       delete pLineEndBuf;
+      pLineEndBuf = NULL;
       pLineEndBuf = new LErdbuf(pInStream->rdbuf());
       pInStream->rdbuf(pLineEndBuf);
     }
@@ -857,7 +901,7 @@ namespace OpenBabel {
 
     ofstream *ofs = new ofstream;
     NeedToFreeOutStream = true; // make sure we clean this up later
-    ios_base::openmode omode = 
+    ios_base::openmode omode =
       pOutFormat->Flags() & WRITEBINARY ? ios_base::out|ios_base::binary : ios_base::out;
 
     ofs->open(filePath.c_str(),omode);
@@ -909,7 +953,7 @@ namespace OpenBabel {
 
     ifstream *ifs = new ifstream;
     NeedToFreeInStream = true; // make sure we free this
-    ios_base::openmode imode = 
+    ios_base::openmode imode =
       pInFormat->Flags() & READBINARY ? ios_base::in|ios_base::binary : ios_base::in;
 
     ifs->open(filePath.c_str(),imode);
@@ -944,6 +988,8 @@ namespace OpenBabel {
     pInStream = ifs;
     InFilename = infilepath;
 
+    if(outfilepath.empty())//Don't open an outfile with an empty name.
+      return true;
     ofstream *ofs = new ofstream;
     NeedToFreeOutStream = true; // make sure we clean this up later
     ofs->open(outfilepath.c_str(),ios_base::out|ios_base::binary);//always open in binary mode
@@ -960,7 +1006,7 @@ namespace OpenBabel {
   ////////////////////////////////////////////
   const char* OBConversion::Description()
   {
-    return 
+    return
       "Conversion options\n"
       "-f <#> Start import at molecule # specified\n"
       "-l <#> End import at molecule # specified\n"
@@ -1022,7 +1068,7 @@ namespace OpenBabel {
         string::size_type posname= InFile.find_last_of("\\/");
         ofname.replace(pos,1, InFile, posname+1, posdot-posname-1);
       }
-    return ofname;	
+    return ofname;
   }
 
   ////////////////////////////////////////////////
@@ -1037,7 +1083,7 @@ namespace OpenBabel {
         snprintf(num, 33, "%d", Count);
         ofname.replace(pos,1, num);
       }
-    return ofname;		
+    return ofname;
   }
   ////////////////////////////////////////////////////
   bool OBConversion::CheckForUnintendedBatch(const string& infile, const string& outfile)
@@ -1054,7 +1100,7 @@ namespace OpenBabel {
     pos = outfile.rfind('.');
     if(pos != string::npos)
       inname2 = infile.substr(0,pos);
-    if(inname1==inname2) 
+    if(inname1==inname2)
       obErrorLog.ThrowError(__FUNCTION__,
 "This was a batch operation. For splitting, use non-empty base name for the output files", obWarning);
 
@@ -1074,7 +1120,7 @@ namespace OpenBabel {
      Done if FileList has more than one file name and OutputFileName does
      not contain * . All the chemical objects are converted and sent
      to the single output file.
- 
+
      Splitting
      Done if FileList contains a single file name and OutputFileName
      contains a * . Each chemical object in the input file is converted
@@ -1104,17 +1150,17 @@ namespace OpenBabel {
   int OBConversion::FullConvert(std::vector<std::string>& FileList, std::string& OutputFileName,
                                 std::vector<std::string>& OutputFileList)
   {
-	
+
     istream* pIs=NULL;
     ostream* pOs=NULL;
     ifstream is;
     ofstream os;
-    stringstream ssOut;
+    stringstream ssOut, ssIn;
     bool HasMultipleOutputFiles=false;
     int Count=0;
     SetFirstInput();
     bool CommonInFormat = pInFormat ? true:false; //whether set in calling routine
-    ios_base::openmode omode = 
+    ios_base::openmode omode =
       pOutFormat->Flags() & WRITEBINARY ? ios_base::out|ios_base::binary : ios_base::out;
     obErrorLog.ClearLog();
     try
@@ -1136,6 +1182,7 @@ namespace OpenBabel {
                   {
                     if(*itr==OutputFileName)
                       {
+
                         pOs = &ssOut;
                         break;
                       }
@@ -1203,14 +1250,21 @@ namespace OpenBabel {
                   {
                     InFilename = *itr;
                     ifstream ifs;
-                    if(!OpenAndSetFormat(CommonInFormat, &ifs))
+                    if(!OpenAndSetFormat(CommonInFormat, &ifs, &ssIn))
                       continue;
+                    if(ifs)
+                      pIs = &ifs;
+                    else
+                      pIs = &ssIn;
+
+                    //pIs = ifs ? &ifs : &ssIn;
+
 
                     if(HasMultipleOutputFiles)
                       {
                         //Batch conversion
                         string batchfile = BatchFileName(OutputFileName,*itr);
-                        
+
                         //With inputs like babel test.xxx -oyyy -m
                         //the user may have wanted to do a splitting operation
                         //Issue a message and abort if xxx==yyy which would overwrite input file
@@ -1219,20 +1273,20 @@ namespace OpenBabel {
 
                         if(ofs.is_open()) ofs.close();
                         ofs.open(batchfile.c_str(), omode);
-                        if(!ofs) 
+                        if(!ofs)
                           {
                             obErrorLog.ThrowError(__FUNCTION__,"Cannot open " + batchfile, obError);
                             return Count;
                           }
                         OutputFileList.push_back(batchfile);
                         SetOutputIndex(0); //reset for new file
-                        Count += Convert(&ifs,&ofs);					
+                        Count += Convert(pIs,&ofs);
                       }
                     else
                       {
                         //Aggregation
                         if(itr!=tempitr) SetMoreFilesToCome();
-                        Count = Convert(&ifs,pOs);					
+                        Count = Convert(pIs,pOs);
                       }
                   }
 
@@ -1250,18 +1304,21 @@ namespace OpenBabel {
                 return Count;
               }
             else
-              {			
+              {
                 //Single input file
                 InFilename = FileList[0];
-                if(!OpenAndSetFormat(CommonInFormat, &is))
+                if(!OpenAndSetFormat(CommonInFormat, &is, &ssIn))
                   return 0;
-                pIs=&is;
+                if(is)
+                  pIs =&is;
+                else
+                  pIs = &ssIn;
 
                 if(HasMultipleOutputFiles)
                   {
                     //Splitting
                     //Output is put in a temporary stream and written to a file
-                    //with an augmenting name only when it contains a valid object. 
+                    //with an augmenting name only when it contains a valid object.
                     int Indx=1;
                     SetInStream(&is);
 #ifdef HAVE_LIBZ
@@ -1293,7 +1350,7 @@ namespace OpenBabel {
                             obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + incrfile, obError);
                             return Count;
                           }
-						
+
                         OutputFileList.push_back(incrfile);
 #ifdef HAVE_LIBZ
 #ifndef DISABLE_WRITE_COMPRESSION
@@ -1341,10 +1398,22 @@ namespace OpenBabel {
     return Count;
   }
 
-  bool OBConversion::OpenAndSetFormat(bool SetFormat, ifstream* is)
+  bool OBConversion::OpenAndSetFormat(bool SetFormat, ifstream* is, stringstream* ss)
   {
     //Opens file using InFilename and sets pInFormat if requested
-    if(!SetFormat)
+    if(ss && InFilename[0]=='-')
+      {
+        //InFilename is actually  -:SMILES
+        is->setstate(ios::failbit); // do not use the input filestream...
+        InFilename.erase(0, 2);
+        if(SetFormat || SetInFormat("smi"))
+          {
+            ss->clear();
+            ss->str(InFilename); //...use the stringstream instead
+            return true;
+          }
+      }
+      else if(!SetFormat)
       {
         pInFormat = FormatFromExt(InFilename.c_str());
         if(pInFormat==NULL)
@@ -1353,12 +1422,15 @@ namespace OpenBabel {
             string ext;
             if(pos!=string::npos)
               ext = InFilename.substr(pos);
-            obErrorLog.ThrowError(__FUNCTION__, "Cannot read input format \"" 
+            obErrorLog.ThrowError(__FUNCTION__, "Cannot read input format \""
                                   + ext + '\"' + " for file \"" + InFilename + "\"",obError);
             return false;
           }
       }
 
+#ifndef ALL_READS_BINARY
+  #define ALL_READS_BINARY //now the default
+#endif
     ios_base::openmode imode;
 #ifdef ALL_READS_BINARY //Makes unix files compatible with VC++6
     imode = ios_base::in|ios_base::binary;
@@ -1377,6 +1449,33 @@ namespace OpenBabel {
   }
 
   ///////////////////////////////////////////////
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+/**<pre>
+Built-in options for conversion of molecules
+Additional options :
+-d Delete hydrogens (make implicit)
+-h Add hydrogens (make explicit)
+-p <pH> Add hydrogens appropriate for this pH
+-b Convert dative bonds e.g.[N+]([O-])=O to N(=O)=O
+-r Remove all but the largest contiguous fragment
+-c Center Coordinates
+-C Combine mols in first file with others having same name
+--filter <filterstring> Filter: convert only when tests are true:\n
+--add <list> Add properties from descriptors\n
+--delete <list> Delete properties in list\n
+--append <list> Append properties or descriptors in list to title:\n
+-s\smarts\ Convert only molecules matching SMARTS:\n
+-v\smarts\ Convert only molecules NOT matching SMARTS: (not displayed in GUI)\n
+--join Join all input molecules into a single output molecule
+--separate Output disconnected fragments separately
+--property <attrib> <value> add or replace a property (SDF)
+--title <title> Add or replace molecule title
+--addtotitle <text> Append to title
+--writeconformers Output multiple conformers separately
+--addindex Append output index to title
+</pre>
+**/
+#endif
   void OBConversion::AddOption(const char* opt, Option_type opttyp, const char* txt)
   {
     //Also updates an option
@@ -1423,7 +1522,7 @@ namespace OpenBabel {
 
   OBConversion::OPAMapType& OBConversion::OptionParamArray(Option_type typ)
   {
-    static OPAMapType* opa = new OPAMapType[3];
+    static OPAMapType opa[3];
     return opa[typ];
   }
 
@@ -1440,8 +1539,8 @@ namespace OpenBabel {
             string description("API");
             if(pFormat)
               description=pFormat->Description();
-            obErrorLog.ThrowError(__FUNCTION__, 
-                                  "The number of parameters needed by option \"" + name + "\" in " 
+            obErrorLog.ThrowError(__FUNCTION__,
+                                  "The number of parameters needed by option \"" + name + "\" in "
                                   + description.substr(0,description.find('\n'))
                                   + " differs from an earlier registration.", obError);
             return;
@@ -1508,6 +1607,50 @@ namespace OpenBabel {
      OptionsArray[typ]=pSourceConv->OptionsArray[typ];
   }
 
+  int OBConversion::NumInputObjects()
+  {
+    istream& ifs = *GetInStream();
+    ifs.clear(); //it may have been at eof
+    //Save position of the input stream
+    streampos pos = ifs.tellg();
+    if(!ifs)
+      return -1;
+
+    //check that the input format supports SkipObjects()
+    if(GetInFormat()->SkipObjects(0, this)==0)
+    {
+      obErrorLog.ThrowError(__FUNCTION__,
+        "Input format does not have a SkipObjects function.", obError);
+      return -1;
+    }
+
+    //counts objects only between the values of -f and -l options
+    int nfirst=1, nlast=numeric_limits<int>::max();
+    const char* p;
+    if( (p=IsOption("f", GENOPTIONS)) ) // extra parens to indicate truth value
+      nfirst=atoi(p);
+    if( (p=IsOption("l", GENOPTIONS)) ) // extra parens to indicate truth value
+      nlast=atoi(p);
+
+    ifs.seekg(0); //rewind
+    //Compressed files currently show an error here.***TAKE CHANCE: RESET ifs****
+    ifs.clear();
+
+    OBFormat* pFormat = GetInFormat();
+    int count=0;
+    //skip each object but stop after nlast objects
+    while(ifs && pFormat->SkipObjects(1, this)>0  && count<nlast)
+      ++count;
+
+    ifs.clear(); //clear eof
+    ifs.seekg(pos); //restore old position
+
+    count -= nfirst-1;
+    return count;
+  }
+
+
+
   //The following function and typedef are deprecated, and are present only
   //for backward compatibility.
   //Use OBConversion::GetSupportedInputFormat(), OBConversion::GetSupportedOutputFormat(),
@@ -1520,7 +1663,7 @@ namespace OpenBabel {
   {
 
     pFormat = NULL;
-    if(str==NULL) 
+    if(str==NULL)
       itr = OBPlugin::Begin("formats");
     else
       itr++;
@@ -1546,6 +1689,17 @@ namespace OpenBabel {
     return true;
   }
 
+  /**
+   * @example obconversion_readstring.cpp
+   * Reading a smiles string.
+   */
+
+  /**
+   * @example obconversion_readstring.py
+   * Reading a smiles string in python.
+   */
+
+
 }//namespace OpenBabel
 
 //! \file obconversion.cpp
