@@ -4,7 +4,7 @@ obmolecformat.cpp - Implementation of subclass of OBFormat for conversion of OBM
 Copyright (C) 2005 Chris Morley
 
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 #include <openbabel/obmolecformat.h>
 #include <openbabel/obiter.h>
+#include <iterator>
 
 using namespace std;
 namespace OpenBabel
@@ -31,7 +32,7 @@ namespace OpenBabel
   bool OBMoleculeFormat::ReadChemObjectImpl(OBConversion* pConv, OBFormat* pFormat)
   {
     std::istream &ifs = *pConv->GetInStream();
-    if (!ifs.good()) //Possible to omit? ifs.peek() == EOF || 
+    if (!ifs.good())
       return false;
 
     OBMol* pmol = new OBMol;
@@ -46,7 +47,7 @@ namespace OpenBabel
     if(pConv->IsOption("C",OBConversion::GENOPTIONS))
       return DeferMolOutput(pmol, pConv, pFormat);
 
-    bool ret;
+    bool ret=true;
    if(pConv->IsOption("separate",OBConversion::GENOPTIONS))
    {
      //On first call, separate molecule and put fragments in MolArray.
@@ -54,16 +55,27 @@ namespace OpenBabel
      //Done this way so that each fragment can be written to its own file (with -m option)
      if(!StoredMolsReady)
      {
-       ret = pFormat->ReadMolecule(pmol,pConv); 
-       if(ret && (pmol->NumAtoms() > 0 || (pFormat->Flags()&ZEROATOMSOK)))
-         MolArray = pmol->Separate(); //use un-transformed molecule
-       //Add an appropriate title to each fragment
-       for(int i=0;i<MolArray.size();++i)
+       while(ret) //do all the molecules in the file
        {
-         stringstream ss;
-         ss << pmol->GetTitle() << '#' << i+1;
-         string title = ss.str();
-         MolArray[i].SetTitle(title);
+         ret = pFormat->ReadMolecule(pmol,pConv);
+
+         if(ret && (pmol->NumAtoms() > 0 || (pFormat->Flags()&ZEROATOMSOK)))
+         {
+           vector<OBMol> SepArray = pmol->Separate(); //use un-transformed molecule
+           //Add an appropriate title to each fragment
+           if(SepArray.size()>1)
+             for(int i=0;i<SepArray.size();++i)
+             {
+               stringstream ss;
+               ss << pmol->GetTitle() << '#' << i+1;
+               string title = ss.str();
+               SepArray[i].SetTitle(title);
+             }
+           else
+              SepArray[0].SetTitle(pmol->GetTitle());
+
+           copy(SepArray.begin(),SepArray.end(),back_inserter(MolArray));
+         }
        }
        reverse(MolArray.begin(),MolArray.end());
        StoredMolsReady = true;
@@ -81,7 +93,7 @@ namespace OpenBabel
        OBMol* pMolCopy = new OBMol( MolArray.back());
        MolArray.pop_back();
        ret = pConv->AddChemObject(
-           pMolCopy->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS)))!=0;
+           pMolCopy->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS), pConv))!=0;
      }
      if(!ret)
        StoredMolsReady = false;
@@ -90,15 +102,15 @@ namespace OpenBabel
      return ret;
    }
 
-    ret=pFormat->ReadMolecule(pmol,pConv); 
+    ret=pFormat->ReadMolecule(pmol,pConv);
 
     OBMol* ptmol = NULL;
-    //Molecule is valid if it has some atoms 
+    //Molecule is valid if it has some atoms
     //or the format allows zero-atom molecules and it has a title
     if(ret && (pmol->NumAtoms() > 0 || (pFormat->Flags()&ZEROATOMSOK && *pmol->GetTitle())))
     {
-      ptmol = static_cast<OBMol*>(pmol->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS)));
-      if(ptmol && (pConv->IsOption("j",OBConversion::GENOPTIONS) 
+      ptmol = static_cast<OBMol*>(pmol->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS),pConv));
+      if(ptmol && (pConv->IsOption("j",OBConversion::GENOPTIONS)
                 || pConv->IsOption("join",OBConversion::GENOPTIONS)))
       {
         //With j option, accumulate all mols in one stored in this class
@@ -140,10 +152,11 @@ namespace OpenBabel
 
     //Retrieve the target OBMol
     OBBase* pOb = pConv->GetChemObject();
+
     OBMol* pmol = dynamic_cast<OBMol*> (pOb);
     bool ret=false;
     if(pmol)
-      { 
+      {
         if(pmol->NumAtoms()==0)
           {
             std::string auditMsg = "OpenBabel::Molecule ";
@@ -161,24 +174,59 @@ namespace OpenBabel
         obErrorLog.ThrowError(__FUNCTION__,
                               auditMsg,
                               obAuditMsg);
-        
-        ret=pFormat->WriteMolecule(pmol,pConv);
-      }
-    delete pOb; //move so that non-OBMol objects are deleted 9March2006
+
+        ret = DoOutputOptions(pOb, pConv);
+
+        if(ret)
+          ret = pFormat->WriteMolecule(pmol,pConv);
+    }
+
+#ifdef HAVE_SHARED_POINTER
+    //If sent a OBReaction* (rather than a OBMol*) output the consituent molecules
+    OBReaction* pReact = dynamic_cast<OBReaction*> (pOb);
+    if(pReact)
+      ret = OutputMolsFromReaction(pReact, pConv, pFormat);
+#endif
+    delete pOb;
     return ret;
   }
 
+  bool OBMoleculeFormat::DoOutputOptions(OBBase* pOb, OBConversion* pConv)
+  {
+    if(pConv->IsOption("addoutindex", OBConversion::GENOPTIONS)) {
+      stringstream ss;
+      ss << pOb->GetTitle() << " " << pConv->GetOutputIndex();
+      pOb->SetTitle(ss.str().c_str());
+    }
+
+    OBMol* pmol = dynamic_cast<OBMol*> (pOb);
+    bool ret=false;
+    if(pmol) {
+      if(pConv->IsOption("writeconformers", OBConversion::GENOPTIONS)) {
+        //The last conformer is written in the calling function
+        unsigned int c = 0;
+        for (; c < pmol->NumConformers()-1; ++c) {
+          pmol->SetConformer(c);
+          if(!pConv->GetOutFormat()->WriteMolecule(pmol, pConv))
+            break;
+        }
+        pmol->SetConformer(c);
+      }
+    }
+    return true;
+  }
+
   /*! Instead of sending molecules for output via AddChemObject(), they are
-    saved in here in OBMoleculeFormat or discarded. By default they are 
+    saved in here in OBMoleculeFormat or discarded. By default they are
     saved only if they are in the first input file. Parts of subsequent
     molecules, such as chemical structure, coordinates and OBGenericData
     can replace the parts in molecules with the same title that have already
     been stored, subject to a set of rules. After all input files have been
     read, the stored molecules (possibly now having augmented properties) are
     sent to the output format.
-    
+
     Is a static function with *this as parameter so that it can be called from other
-    format classes like XMLMoleculeFormat which are not derived from OBMoleculeFormat. 
+    format classes like XMLMoleculeFormat which are not derived from OBMoleculeFormat.
   */
   bool OBMoleculeFormat::DeferMolOutput(OBMol* pmol, OBConversion* pConv, OBFormat* pF )
   {
@@ -189,8 +237,9 @@ namespace OpenBabel
       {
         IsFirstFile=true;
         IMols.clear();
+        pConv->AddOption("OutputAtEnd", OBConversion::GENOPTIONS);
       }
-    else 
+    else
       {
         if((std::streamoff)pConv->GetInStream()->tellg()<=0)
           IsFirstFile=false;//File has changed
@@ -210,7 +259,7 @@ namespace OpenBabel
         string::size_type pos = title.find_first_of("\t\r\n"); //some title have other data appended
         if(pos!=string::npos)
           title.erase(pos);
-    
+
         map<std::string, OBMol*>::iterator itr;
         itr = IMols.find(title);
         if(itr!=IMols.end())
@@ -242,31 +291,31 @@ namespace OpenBabel
     delete pmol;
     return true;
   }
-  
-  /*! Makes a new OBMol on the heap by combining two molecules according to the rule below. 
+
+  /*! Makes a new OBMol on the heap by combining two molecules according to the rule below.
     If both have OBGenericData of the same type, or OBPairData with the
     same attribute,  the version from pFirst is used.
     Returns a pointer to a new OBMol which will need deleting by the calling program
-    (probably by being sent to an output format). 
+    (probably by being sent to an output format).
     If the molecules cannot be regarded as being the same structure a NULL
     pointer is returned and an error message logged.
-    
+
     pFirst and pSecond and the objects they point to are not changed. (const
     modifiers difficult because class OBMol not designed appropriately)
-    
+
     Combining molecules: rules for each of the three parts
     Title:
     Use the title of pFirst unless it has none, when use that of pSecond.
     Warning if neither molecule has a title.
-    
+
     Structure
     - a structure with atoms replaces one with no atoms
     - a structure with bonds replaces one with no bonds,
     provided the formula is the same, else an error.
-    - structures with atoms and bonds are compared by InChI; error if not the same. 
+    - structures with atoms and bonds are compared by InChI; error if not the same.
     - a structure with 3D coordinates replaces one with 2D coordinates
     - a structure with 2D coordinates replace one with 0D coordinates
-    
+
     OBGenericData
     OBPairData
   */
@@ -292,7 +341,7 @@ namespace OpenBabel
       {
         if(pFirst->GetSpacedFormula()!=pSecond->GetSpacedFormula())
           {
-            obErrorLog.ThrowError(__FUNCTION__, 
+            obErrorLog.ThrowError(__FUNCTION__,
                                   "Molecules with name = " + title + " have different formula",obError);
             return NULL;
           }
@@ -315,8 +364,8 @@ namespace OpenBabel
 
     OBMol* pMain = swap ? pSecond : pFirst;
     OBMol* pOther = swap ? pFirst : pSecond;
-    
-    *pNewMol = *pMain; //Now copies all data 
+
+    *pNewMol = *pMain; //Now copies all data
 
     //Copy some OBGenericData from the OBMol which did not provide the structure
     vector<OBGenericData*>::iterator igd;
@@ -349,7 +398,7 @@ namespace OpenBabel
     pConv->SetOneObjectOnly(false);
     for(itr=IMols.begin();itr!=IMols.end();++itr,++i)
       {
-        if(!(itr->second)->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS)))
+        if(!(itr->second)->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS), pConv))
           continue;
         pConv->SetOutputIndex(i);
         if(itr==lastitr)
@@ -372,7 +421,7 @@ namespace OpenBabel
 
   bool OBMoleculeFormat::DeleteDeferredMols()
   {
-    //Empties IMols, deteting the OBMol objects whose pointers are stored there 
+    //Empties IMols, deteting the OBMol objects whose pointers are stored there
     std::map<std::string, OBMol*>::iterator itr;
     for(itr=IMols.begin();itr!=IMols.end();++itr)
       {
@@ -382,6 +431,50 @@ namespace OpenBabel
     return false;
   }
 
+  ///////////////////////////////////////////////////////////////////
+#ifdef HAVE_SHARED_POINTER
+  bool OBMoleculeFormat::OutputMolsFromReaction
+    (OBReaction* pReact, OBConversion* pConv, OBFormat* pFormat)
+  {
+    //Output all the constituent molecules of the reaction
+
+    //Collect the molecules first, just for convenience
+    vector<shared_ptr<OBMol> > mols;
+    unsigned i;
+    for(i=0;i<pReact->NumReactants();i++)
+      mols.push_back(pReact->GetReactant(i));
+    for(i=0;i<pReact->NumProducts();i++)
+      mols.push_back(pReact->GetProduct(i));
+
+    if(pReact->GetAgent())
+      mols.push_back(pReact->GetAgent());
+    if(pReact->GetTransitionState())
+      mols.push_back(pReact->GetTransitionState());
+
+    pConv->SetOutputIndex(pConv->GetOutputIndex() - 1); // The OBReaction object is not output
+    if((pFormat->Flags() & WRITEONEONLY) && mols.size()>1)
+    {
+      stringstream ss;
+      ss << "There are " << mols.size() << " molecules to be output,"
+         << "but this format is for single molecules only";
+      obErrorLog.ThrowError(__FUNCTION__, ss.str(), obWarning);
+      mols.resize(1);
+    }
+    bool ok = true;
+    for(i=0;i<mols.size() && ok;++i)
+    {
+      if(mols[i])
+      {
+        //Have to do set these manually because not using "Convert" interface
+        pConv->SetLast(i==mols.size()-1);
+        pConv->SetOutputIndex(pConv->GetOutputIndex()+1);
+        ok = pFormat->WriteMolecule(
+          mols[i]->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS), pConv),pConv);
+      }
+    }
+    return ok;
+  }
+#endif
   //////////////////////////////////////////////////////////////////
   /** Attempts to read the index file datafilename.obindx successively
       from the following directories:
@@ -391,7 +484,7 @@ namespace OpenBabel
       - in a subdirectory of the BABEL_DATADIR directory with the version of OpenBabel as its name
       An index of type NameIndexType is then constructed. NameIndexType is defined
       in obmolecformat.h and may be a std::tr1::unordered_map (a hash_map) or std::map.
-      In any case it is searched by 
+      In any case it is searched by
       @code
       NameIndexType::iterator itr = index.find(molecule_name);
       if(itr!=index.end())
@@ -403,19 +496,20 @@ namespace OpenBabel
       it using the format pInFormat, and written to the directory containing the datafile.
       This means that this function can be used without worrying whether there is an index.
       It will be slow to execute the first time, but subsequent uses get the speed benefit
-      of indexed access to the datafile. 
+      of indexed access to the datafile.
 
       The serialization and de-serialization of the NameIndexType is entirely in
-      this routine and could possibly be improved. Currently re-hashing is done 
+      this routine and could possibly be improved. Currently re-hashing is done
       every time the index is read.
   **/
+
   bool OBMoleculeFormat::ReadNameIndex(NameIndexType& index,
                                        const string& datafilename, OBFormat* pInFormat)
   {
     struct headertype
     {
       char filename[256];
-      unsigned size;
+      size_t size;
     } header;
 
     NameIndexType::iterator itr;
@@ -429,7 +523,7 @@ namespace OpenBabel
         string datafilepath = OpenDatafile(datastream, datafilename);
         if(!datastream)
           {
-            obErrorLog.ThrowError(__FUNCTION__, 
+            obErrorLog.ThrowError(__FUNCTION__,
                                   datafilename + " was not found or could not be opened",  obError);
             return false;
           }
@@ -446,7 +540,7 @@ namespace OpenBabel
             mol.Clear();
             pos = datastream.tellg();
           }
-        obErrorLog.ThrowError(__FUNCTION__, 
+        obErrorLog.ThrowError(__FUNCTION__,
                               "Prepared an index for " + datafilepath, obAuditMsg);
         //Save index to file
         ofstream dofs((datafilepath + ".obindx").c_str(), ios_base::out|ios_base::binary);
@@ -456,15 +550,15 @@ namespace OpenBabel
         header.filename[sizeof(header.filename) - 1] = '\0';
         header.size = index.size();
         dofs.write((const char*)&header, sizeof(headertype));
-	
+
         for(itr=index.begin();itr!=index.end();++itr)
           {
             //#chars; chars;  ofset(4bytes).
-            const char n = itr->first.size();
+            const char n = static_cast<char> (itr->first.size());
             dofs.put(n);
             dofs.write(itr->first.c_str(),n);
             dofs.write((const char*)&itr->second,sizeof(unsigned));
-          }			
+          }
       }
     else
       {
