@@ -1,14 +1,14 @@
 /**********************************************************************
 optransform.cpp: makes option to transform molecule as specified in a datafile
 Copyright (C) 2008 Chris Morley
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -65,11 +65,11 @@ bool OpTransform::Initialize()
   else //When filename is * use data in lines following
     for(int i=4;i<_textlines.size();++i)
       ParseLine(_textlines[i].c_str());
-      
-          
+
+
   // return the locale to the original one
   obLocale.RestoreLocale();
-  
+
   return true;
 }
 
@@ -103,7 +103,7 @@ void OpTransform::ParseLine(const char *buffer)
     }
   }
 }
-bool OpTransform::Do(OBBase* pOb, OpMap*, const char*)
+bool OpTransform::Do(OBBase* pOb, const char* OptionText, OpMap* pOptions, OBConversion* pConv)
 {
   OBMol* pmol = dynamic_cast<OBMol*>(pOb);
   if(!pmol)
