@@ -6,7 +6,7 @@ Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2004-2006 by Chris Morley
 
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -149,21 +149,28 @@ int main(int argc,char *argv[])
                     }
                   break;
 				
-                /*case 'F':
-                  if(!Conv.SetOutFormat("fpt"))
-                    cout << "FingerprintFormat needs to be loaded" << endl;
-                  else
-                    {
-                      Conv.AddOption("F",OBConversion::OUTOPTIONS);
-                      Conv.Write(NULL);
-                    }
-                  return 0;
-				        */
                 case 'L': //display a list of plugin type or classes
                   {
                     const char* param=NULL;
                     if(argc>arg+1)
                       param = argv[arg+2];
+
+                    // First assume first arg is a plugin type and
+                    // param is a subtype, like babel -L ops gen3D
+                    // or first arg is a plugin ID, like babel -L cml
+                    OBPlugin* plugin;
+                    if(OBPlugin::GetPlugin("plugins", argv[arg+1])
+                      && (plugin = OBPlugin::GetPlugin(argv[arg+1], param))
+                      || (plugin = OBPlugin::GetPlugin(NULL, argv[arg+1])))
+                    {
+                      //Output details of subtype
+                      string txt;
+                      plugin->Display(txt, "verbose", argv[arg+1]);
+                      cout << "One of the " << plugin->TypeID() << '\n' << txt << endl;
+                      return 0;
+                    }
+                    //...otherwise assume it is a plugin type, like babel -L forcefields
+                    //Output list of subtypes
                     OBPlugin::List(argv[arg+1], param);
                     return 0;
                   }
@@ -275,7 +282,7 @@ int main(int argc,char *argv[])
       FileList.pop_back();
     }
   
-#ifdef _WIN32
+#if defined(_WIN32) && defined(USING_DYNAMIC_LIBS)
   //Expand wildcards in input filenames and add to FileList
   vector<string> tempFileList(FileList);
   FileList.clear();
@@ -424,11 +431,10 @@ void help()
   cout << "wildcard chars (* and ?).The molecules are aggregated in the output file.\n" << endl;
   cout << OBConversion::Description(); // Conversion options
   cout << "-H Outputs this help text" << endl;
-  cout << "-Hxxx (xxx is file format ID e.g. -Hcml) gives format info" <<endl; 
-  cout << "-Hall Outputs details of all formats" <<endl; 
   cout << "-V Outputs version number" <<endl; 
-  cout << "-L <BaseType> Lists plugin classes of this type" << endl;
-  cout << "    e.g. <fingerprints>, or <plugins> for a list of BaseTypes" << endl; 
+  cout << "-L <category> Lists plugin classes of this category, e.g. <formats>" << endl;
+  cout << "   Use just -L for a list of plugin categories." << endl; 
+  cout << "   Use -L <ID> e.g. -L sdf for details of a format or other plugin." << endl; 
   cout << "-m Produces multiple output files, to allow:" <<endl;
   cout << "   Splitting: e.g.        " << program_name << " infile.mol new.smi -m" <<endl;
   cout << "     puts each molecule into new1.smi new2.smi etc" <<endl;
@@ -447,9 +453,12 @@ void help()
   if(pAPI)
     cout << pAPI->Description();
   
-  cout << "The following file formats are recognized:" << endl;
-  OBPlugin::List("formats");
-  cout << "\nSee further specific info and options using -H<format-type>, e.g. -Hcml" << endl;
+  cout << "To see a list of recognized file formats use\n  babel -L formats\n"
+       << "To see details and specific options for a particular format, e.g CML, use\n  babel -L cml\n"
+       << endl;
+  //cout << "The following file formats are recognized:" << endl;
+  //OBPlugin::List("formats");
+  //cout << "\nSee further specific info and options using -H<format-type>, e.g. -Hcml" << endl;
 }
 
 /* OpenBabel man page*/
@@ -467,7 +476,7 @@ void help()
 *
 * Note that Open Babel can also be used as a library to interconvert
 * many file formats and to provide standard chemistry software routines.
-* See the Open Babel web pages (http://openbabel.sourceforge.net) for more
+* See the Open Babel web pages (http://openbabel.org) for more
 * information.
 *
 * \par OPTIONS
@@ -597,7 +606,7 @@ void help()
 *
 * Open Babel is currently maintained by \b Geoff \b Hutchison, \b Chris \b Morley and \b Michael \b Banck.
 *
-* For more contributors to Open Babel, see http://openbabel.sourceforge.net/THANKS.shtml
+* For more contributors to Open Babel, see http://openbabel.org/THANKS.shtml
 *
 * \par COPYRIGHT
 *  Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
@@ -611,5 +620,5 @@ void help()
 *  GNU General Public License for more details.
 *
 * \par SEE ALSO
-*   The web pages for Open Babel can be found at http://openbabel.sourceforge.net/
+*   The web pages for Open Babel can be found at http://openbabel.org/
 **/
